@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -28,15 +28,10 @@
 #include "core/proto.h"
 #include "core/processing.h"
 #include "core/undo.h"
+#include "core/siril_log.h"
 #include "core/OS_utils.h"
 #include "gui/progress_and_log.h"
-#include "gui/utils.h"
-#include "gui/image_display.h"
-#include "gui/image_interactions.h"
-#include "gui/message_dialog.h"
 #include "gui/histogram.h"
-#include "gui/dialogs.h"
-#include "gui/registration_preview.h"
 #include "io/single_image.h"
 #include "io/image_format_fits.h"
 #include "algos/colors.h"
@@ -267,6 +262,101 @@ void rgb_to_hsl(double r, double g, double b, double *h, double *s, double *l) {
 	*h /= 6;
 }
 
+// Single precision versions of the above
+void hsl_to_rgbf(float h, float s, float l, float * r, float * g,
+		float * b) {
+	float v;
+
+	assert(h >= 0.f && h <= 1.f);
+	if (h >= 1.f) h -= 1.f;		// this code doesn't work for h = 1
+	v = (l <= 0.5f) ? (l * (1.f + s)) : (l + s - l * s);
+	if (v <= 0.f) {
+		*r = *g = *b = 0.f;
+	} else {
+		float m;
+		float sv;
+		int sextant;
+		float fract, vsf, mid1, mid2;
+
+		m = l + l - v;
+		sv = (v - m) / v;
+		h *= 6.f;
+		sextant = h;
+		fract = h - sextant;
+		vsf = v * sv * fract;
+		mid1 = m + vsf;
+		mid2 = v - vsf;
+		switch (sextant) {
+			case 0:
+				*r = v;
+				*g = mid1;
+				*b = m;
+				break;
+			case 1:
+				*r = mid2;
+				*g = v;
+				*b = m;
+				break;
+			case 2:
+				*r = m;
+				*g = v;
+				*b = mid1;
+				break;
+			case 3:
+				*r = m;
+				*g = mid2;
+				*b = v;
+				break;
+			case 4:
+				*r = mid1;
+				*g = m;
+				*b = v;
+				break;
+			case 5:
+				*r = v;
+				*g = m;
+				*b = mid2;
+				break;
+		}
+	}
+}
+
+void rgb_to_hslf(float r, float g, float b, float *h, float *s, float *l) {
+	float v;
+	float m;
+	float vm;
+	float r2, g2, b2;
+
+	v = max(r, g);
+	v = max(v, b);
+	m = min(r, g);
+	m = min(m, b);
+	*h = 0.f;
+	*s = 0.f;	// init values
+
+	if ((*l = (m + v) / 2.f) <= 0.f) {
+		*l = 0.f;
+		return;
+	}
+	if ((*s = vm = v - m) > 0.f) {
+		*s /= (*l <= 0.5f) ? (v + m) : (2.f - v - m);
+	} else
+		return;
+
+	r2 = (v - r) / vm;
+	g2 = (v - g) / vm;
+	b2 = (v - b) / vm;
+
+	if (r == v)
+		*h = (g == m ? 5.f + b2 : 1.f - g2);
+	else if (g == v)
+		*h = (b == m ? 1.f + r2 : 3.f - b2);
+	else
+		*h = (r == m ? 3.f + g2 : 5.f - r2);
+
+	*h /= 6.f;
+}
+
 /* all variables are between 0 and 1. h takes 0 for grey */
 void rgb_to_hsv(double r, double g, double b, double *h, double *s, double *v) {
 	double cmax, cmin, delta;
@@ -352,11 +442,96 @@ void rgb_to_xyz(double r, double g, double b, double *x, double *y, double *z) {
 	g *= 100;
 	b *= 100;
 
-	*x = 0.412453 * r + 0.357580 * g + 0.180423 * b;
-	*y = 0.212671 * r + 0.715160 * g + 0.072169 * b;
-	*z = 0.019334 * r + 0.119193 * g + 0.950227 * b;
+	*x = 0.4124564 * r + 0.3575761 * g + 0.1804375 * b;
+	*y = 0.2126729 * r + 0.7151522 * g + 0.0721750 * b;
+	*z = 0.0193339 * r + 0.1191920 * g + 0.9503041 * b;
 }
 
+void rgb_to_xyzf(float r, float g, float b, float *x, float *y, float *z) {
+	r = (r <= 0.04045f) ? r / 12.92f : powf(((r + 0.055f) / 1.055f), 2.4f);
+	g = (g <= 0.04045f) ? g / 12.92f : powf(((g + 0.055f) / 1.055f), 2.4f);
+	b = (b <= 0.04045f) ? b / 12.92f : powf(((b + 0.055f) / 1.055f), 2.4f);
+
+	r *= 100.f;
+	g *= 100.f;
+	b *= 100.f;
+
+	*x = 0.4124564f * r + 0.3575761f * g + 0.1804375f * b;
+	*y = 0.2126729f * r + 0.7151522f * g + 0.0721750f * b;
+	*z = 0.0193339f * r + 0.1191920f * g + 0.9503041f * b;
+}
+
+/* These functions have a scale argument as sometimes it is convenient to have
+ * the Y channel output in the range [0..1] instead of [0..100]
+ */
+
+void linrgb_to_xyz(double r, double g, double b, double *x, double *y, double *z, gboolean scale) {
+	if (scale) {
+		r *= 100;
+		g *= 100;
+		b *= 100;
+	}
+
+	*x = 0.4124564 * r + 0.3575761 * g + 0.1804375 * b;
+	*y = 0.2126729 * r + 0.7151522 * g + 0.0721750 * b;
+	*z = 0.0193339 * r + 0.1191920 * g + 0.9503041 * b;
+}
+
+void linrgb_to_xyzf(float r, float g, float b, float *x, float *y, float *z, gboolean scale) {
+	if (scale) {
+		r *= 100.f;
+		g *= 100.f;
+		b *= 100.f;
+	}
+
+	*x = 0.4124564f * r + 0.3575761f * g + 0.1804375f * b;
+	*y = 0.2126729f * r + 0.7151522f * g + 0.0721750f * b;
+	*z = 0.0193339f * r + 0.1191920f * g + 0.9503041f * b;
+}
+
+void xyz_to_linrgb(double x, double y, double z, double *r, double *g, double *b, gboolean scale) {
+	if (scale) {
+		x /= 100.0;
+		y /= 100.0;
+		z /= 100.0;
+	}
+
+	*r =  3.2404542 * x - 1.5371385 * y - 0.4985314 * z;
+	*g = -0.9692660 * x + 1.8760108 * y + 0.0415560 * z;
+	*b =  0.0556434 * x - 0.2040259 * y + 1.0572252 * z;
+}
+
+void xyz_to_linrgbf(float x, float y, float z, float *r, float *g, float *b, gboolean scale) {
+	if (scale) {
+		x /= 100.f;
+		y /= 100.f;
+		z /= 100.f;
+	}
+
+	*r =  3.2404542f * x - 1.5371385f * y - 0.4985314f * z;
+	*g = -0.9692660f * x + 1.8760108f * y + 0.0415560f * z;
+	*b =  0.0556434f * x - 0.2040259f * y + 1.0572252f * z;
+}
+
+void rgb_to_yuvf(float red, float green, float blue, float *y, float *u, float *v) {
+	const float a = 1.f / sqrtf(3.f);
+	const float b = 1.f / sqrtf(2.f);
+	const float c = 2.f * a * sqrtf(2.f);
+	*y = a * (red + green + blue);
+	*u = b * (red - blue);
+	*v = c * (0.25f * red - 0.5f * green + 0.25f * blue);
+}
+
+void yuv_to_rgbf(float y, float u, float v, float *red, float *green, float *blue) {
+	const float a = 1.f / sqrtf(3.f);
+	const float b = 1.f / sqrtf(2.f);
+	const float c = a / b;
+	*red = (a * y) + (b * u) + (c * 0.5f * v);
+	*green = (a * y) - (c * v);
+	*blue = (a * y) - (b * u) + (c * 0.5f * v);
+}
+
+
 void xyz_to_LAB(double x, double y, double z, double *L, double *a, double *b) {
 	x /= 95.047;
 	y /= 100.000;
@@ -371,6 +546,20 @@ void xyz_to_LAB(double x, double y, double z, double *L, double *a, double *b) {
 	*b = 200.0 * (y - z);
 }
 
+void xyz_to_LABf(float x, float y, float z, float *L, float *a, float *b) {
+	x /= 95.047f;
+	y /= 100.000f;
+	z /= 108.883f;
+
+	x = (x > 0.008856452f) ? powf(x, 1.f / 3.0f) : (7.787037037f * x) + (16.f / 116.f);
+	y = (y > 0.008856452f) ? powf(y, 1.f / 3.0f) : (7.787037037f * y) + (16.f / 116.f);
+	z = (z > 0.008856452f) ? powf(z, 1.f / 3.0f) : (7.787037037f * z) + (16.f / 116.f);
+
+	*L = (116.0f * y) - 16.0f;
+	*a = 500.0f * (x - y);
+	*b = 200.0f * (y - z);
+}
+
 void LAB_to_xyz(double L, double a, double b, double *x, double *y, double *z) {
 	*y = (L + 16.0) / 116.0;
 	*x = a / 500.0 + (*y);
@@ -390,18 +579,52 @@ void LAB_to_xyz(double L, double a, double b, double *x, double *y, double *z) {
 	*z *= 108.883;
 }
 
+void LAB_to_xyzf(float L, float a, float b, float *x, float *y, float *z) {
+	*y = (L + 16.0f) / 116.0f;
+	*x = a / 500.0f + (*y);
+	*z = *y - b / 200.0f;
+
+	float x3, y3, z3;
+	x3 = (*x) * (*x) * (*x);
+	y3 = (*y) * (*y) * (*y);
+	z3 = (*z) * (*z) * (*z);
+
+	*x = (x3 > 0.008856452f) ? x3 : (*x - 16.f / 116.f) / 7.787037037f;
+	*y = (y3 > 0.008856452f) ? y3 : (*y - 16.f / 116.f) / 7.787037037f;
+	*z = (z3 > 0.008856452f) ? z3 : (*z - 16.f / 116.f) / 7.787037037f;
+
+	*x *= 95.047f;
+	*y *= 100.000f;
+	*z *= 108.883f;
+}
+
 void xyz_to_rgb(double x, double y, double z, double *r, double *g, double *b) {
 	x /= 100.0;
 	y /= 100.0;
 	z /= 100.0;
 
-	*r =  3.240479 * x - 1.537150 * y - 0.498535 * z;
-	*g = -0.969256 * x + 1.875992 * y + 0.041556 * z;
-	*b =  0.055648 * x - 0.204043 * y + 1.057311 * z;
+	*r =  3.2404542 * x - 1.5371385 * y - 0.4985314 * z;
+	*g = -0.9692660 * x + 1.8760108 * y + 0.0415560 * z;
+	*b =  0.0556434 * x - 0.2040259 * y + 1.0572252 * z;
 
 	*r = (*r > 0.0031308) ? 1.055 * (pow(*r, (1 / 2.4))) - 0.055 : 12.92 * (*r);
 	*g = (*g > 0.0031308) ? 1.055 * (pow(*g, (1 / 2.4))) - 0.055 : 12.92 * (*g);
 	*b = (*b > 0.0031308) ? 1.055 * (pow(*b, (1 / 2.4))) - 0.055 : 12.92 * (*b);
+
+}
+
+void xyz_to_rgbf(float x, float y, float z, float *r, float *g, float *b) {
+	x /= 100.0f;
+	y /= 100.0f;
+	z /= 100.0f;
+
+	*r =  3.2404542f * x - 1.5371385f * y - 0.4985314f * z;
+	*g = -0.9692660f * x + 1.8760108f * y + 0.0415560f * z;
+	*b =  0.0556434f * x - 0.2040259f * y + 1.0572252f * z;
+
+	*r = (*r > 0.0031308f) ? 1.055f * (powf(*r, (1.f / 2.4f))) - 0.055f : 12.92f * (*r);
+	*g = (*g > 0.0031308f) ? 1.055f * (powf(*g, (1.f / 2.4f))) - 0.055f : 12.92f * (*g);
+	*b = (*b > 0.0031308f) ? 1.055f * (powf(*b, (1.f / 2.4f))) - 0.055f : 12.92f * (*b);
 }
 
 // color index to temperature in kelvin
@@ -462,32 +685,16 @@ int equalize_cfa_fit_with_coeffs(fits *fit, float coeff1, float coeff2, const ch
 	return 0;
 }
 
-// idle function executed at the end of the extract_channels processing
-static gboolean end_extract_channels(gpointer p) {
-	struct extract_channels_data *args = (struct extract_channels_data *) p;
-	stop_processing_thread();
-	free(args->channel[0]);
-	free(args->channel[1]);
-	free(args->channel[2]);
-	free(args);
-	set_cursor_waiting(FALSE);
-
-	return FALSE;
-}
-
 static gpointer extract_channels_ushort(gpointer p) {
 	struct extract_channels_data *args = (struct extract_channels_data *) p;
 	WORD *buf[3] = { args->fit->pdata[RLAYER], args->fit->pdata[GLAYER],
 		args->fit->pdata[BLAYER] };
 	size_t n = args->fit->naxes[0] * args->fit->naxes[1];
 	struct timeval t_start, t_end;
-	args->process = TRUE;
 
 	if (args->fit->naxes[2] != 3) {
 		siril_log_message(
 				_("Siril cannot extract layers. Make sure your image is in RGB mode.\n"));
-		args->process = FALSE;
-		clearfits(args->fit);
 		return GINT_TO_POINTER(1);
 	}
 
@@ -496,11 +703,9 @@ static gpointer extract_channels_ushort(gpointer p) {
 	gettimeofday(&t_start, NULL);
 
 	switch (args->type) {
-		/* RGB space: nothing to do */
-		case 0:
-			break;
-			/* HSL space */
-		case 1:
+	case EXTRACT_RGB:
+		break;
+	case EXTRACT_HSL:
 #ifdef _OPENMP
 #pragma omp parallel for num_threads(com.max_thread) schedule(static)
 #endif
@@ -515,8 +720,7 @@ static gpointer extract_channels_ushort(gpointer p) {
 			buf[BLAYER][i] = round_to_WORD(l * USHRT_MAX_DOUBLE);
 		}
 		break;
-		/* HSV space */
-	case 2:
+	case EXTRACT_HSV:
 #ifdef _OPENMP
 #pragma omp parallel for num_threads(com.max_thread) schedule(static)
 #endif
@@ -531,8 +735,7 @@ static gpointer extract_channels_ushort(gpointer p) {
 			buf[BLAYER][i] = round_to_WORD(v * USHRT_MAX_DOUBLE);
 		}
 		break;
-		/* CIE L*a*b */
-	case 3:
+	case EXTRACT_CIELAB:
 #ifdef _OPENMP
 #pragma omp parallel for num_threads(com.max_thread) schedule(static)
 #endif
@@ -553,11 +756,12 @@ static gpointer extract_channels_ushort(gpointer p) {
 	}
 	gchar *fitfilter = g_strdup(args->fit->filter);
 	for (int i = 0; i < 3; i++) {
-		update_filter_information(args->fit, add_filter_str[i], TRUE);
-		save1fits16(args->channel[i], args->fit, i);
-		update_filter_information(args->fit, fitfilter, FALSE); //reinstate original filter name
+		if (args->channel[i]) {
+			update_filter_information(args->fit, add_filter_str[i], TRUE);
+			save1fits16(args->channel[i], args->fit, i);
+			update_filter_information(args->fit, fitfilter, FALSE); //reinstate original filter name
+		}
 	}
-	clearfits(args->fit);
 	g_free(fitfilter);
 	gettimeofday(&t_end, NULL);
 	show_time(t_start, t_end);
@@ -571,13 +775,10 @@ static gpointer extract_channels_float(gpointer p) {
 		args->fit->fpdata[BLAYER] };
 	struct timeval t_start, t_end;
 	size_t n = args->fit->naxes[0] * args->fit->naxes[1];
-	args->process = TRUE;
 
 	if (args->fit->naxes[2] != 3) {
 		siril_log_message(
-				_("Siril cannot extract layers. Make sure your image is in RGB mode.\n"));
-		args->process = FALSE;
-		clearfits(args->fit);
+				_("Siril cannot extract channels. Make sure your image is in RGB mode.\n"));
 		return GINT_TO_POINTER(1);
 	}
 
@@ -586,11 +787,9 @@ static gpointer extract_channels_float(gpointer p) {
 	gettimeofday(&t_start, NULL);
 
 	switch (args->type) {
-	/* RGB space: nothing to do */
-	case 0:
+	case EXTRACT_RGB:
 		break;
-		/* HSL space */
-	case 1:
+	case EXTRACT_HSL:
 #ifdef _OPENMP
 #pragma omp parallel for num_threads(com.max_thread) schedule(static)
 #endif
@@ -605,8 +804,7 @@ static gpointer extract_channels_float(gpointer p) {
 			buf[BLAYER][i] = (float) l;
 		}
 		break;
-		/* HSV space */
-	case 2:
+	case EXTRACT_HSV:
 #ifdef _OPENMP
 #pragma omp parallel for num_threads(com.max_thread) schedule(static)
 #endif
@@ -621,8 +819,7 @@ static gpointer extract_channels_float(gpointer p) {
 			buf[BLAYER][i] = (float) v;
 		}
 		break;
-		/* CIE L*a*b */
-	case 3:
+	case EXTRACT_CIELAB:
 #ifdef _OPENMP
 #pragma omp parallel for num_threads(com.max_thread) schedule(static)
 #endif
@@ -641,11 +838,12 @@ static gpointer extract_channels_float(gpointer p) {
 	}
 	gchar *fitfilter = g_strdup(args->fit->filter);
 	for (int i = 0; i < 3; i++) {
-		update_filter_information(args->fit, add_filter_str[i], TRUE);
-		save1fits32(args->channel[i], args->fit, i);
-		update_filter_information(args->fit, fitfilter, FALSE); //reinstate original filter name
+		if (args->channel[i]) {
+			update_filter_information(args->fit, add_filter_str[i], TRUE);
+			save1fits32(args->channel[i], args->fit, i);
+			update_filter_information(args->fit, fitfilter, FALSE); //reinstate original filter name
+		}
 	}
-	clearfits(args->fit);
 	g_free(fitfilter);
 	gettimeofday(&t_end, NULL);
 	show_time(t_start, t_end);
@@ -653,6 +851,7 @@ static gpointer extract_channels_float(gpointer p) {
 	return GINT_TO_POINTER(0);
 }
 
+// channels extraction computed in-place
 gpointer extract_channels(gpointer p) {
 	struct extract_channels_data *args = (struct extract_channels_data *)p;
 	gpointer retval = GINT_TO_POINTER(1);
@@ -661,84 +860,20 @@ gpointer extract_channels(gpointer p) {
 		retval = extract_channels_ushort(p);
 	else if (args->fit->type == DATA_FLOAT)
 		retval = extract_channels_float(p);
-	siril_add_idle(end_extract_channels, args);
+
+	clearfits(args->fit);
+	free(args->channel[0]);
+	free(args->channel[1]);
+	free(args->channel[2]);
+	free(args);
+	siril_add_idle(end_generic, NULL);
 	return retval;
 }
 
 /****************** Color calibration ************************/
-void on_button_bkg_selection_clicked(GtkButton *button, gpointer user_data) {
-	static GtkSpinButton *selection_black_value[4] = { NULL, NULL, NULL, NULL };
-
-	if ((!com.selection.h) || (!com.selection.w)) {
-		siril_message_dialog(GTK_MESSAGE_WARNING, _("There is no selection"),
-				_("Make a selection of the background area"));
-		return;
-	}
-
-	if (!selection_black_value[0]) {
-		selection_black_value[0] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(gui.builder, "spin_bkg_x"));
-		selection_black_value[1] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(gui.builder, "spin_bkg_y"));
-		selection_black_value[2] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(gui.builder, "spin_bkg_w"));
-		selection_black_value[3] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(gui.builder, "spin_bkg_h"));
-	}
-
-	gtk_spin_button_set_value(selection_black_value[0], com.selection.x);
-	gtk_spin_button_set_value(selection_black_value[1], com.selection.y);
-	gtk_spin_button_set_value(selection_black_value[2], com.selection.w);
-	gtk_spin_button_set_value(selection_black_value[3], com.selection.h);
-}
-
-void initialize_calibration_interface() {
-	static GtkAdjustment *selection_black_adjustment[4] = { NULL, NULL, NULL,
-		NULL };
-	static GtkAdjustment *selection_white_adjustment[4] = { NULL, NULL, NULL,
-		NULL };
-
-	if (!selection_black_adjustment[0]) {
-		selection_black_adjustment[0] = GTK_ADJUSTMENT(
-				gtk_builder_get_object(gui.builder, "adjustment_bkg_x"));
-		selection_black_adjustment[1] = GTK_ADJUSTMENT(
-				gtk_builder_get_object(gui.builder, "adjustment_bkg_y"));
-		selection_black_adjustment[2] = GTK_ADJUSTMENT(
-				gtk_builder_get_object(gui.builder, "adjustment_bkg_w"));
-		selection_black_adjustment[3] = GTK_ADJUSTMENT(
-				gtk_builder_get_object(gui.builder, "adjustment_bkg_h"));
-	}
-	if (!selection_white_adjustment[0]) {
-		selection_white_adjustment[0] = GTK_ADJUSTMENT(
-				gtk_builder_get_object(gui.builder, "adjustment_white_x"));
-		selection_white_adjustment[1] = GTK_ADJUSTMENT(
-				gtk_builder_get_object(gui.builder, "adjustment_white_y"));
-		selection_white_adjustment[2] = GTK_ADJUSTMENT(
-				gtk_builder_get_object(gui.builder, "adjustment_white_w"));
-		selection_white_adjustment[3] = GTK_ADJUSTMENT(
-				gtk_builder_get_object(gui.builder, "adjustment_white_h"));
-	}
-	gtk_adjustment_set_upper(selection_black_adjustment[0], gfit.rx);
-	gtk_adjustment_set_upper(selection_black_adjustment[1], gfit.ry);
-	gtk_adjustment_set_upper(selection_black_adjustment[2], gfit.rx);
-	gtk_adjustment_set_upper(selection_black_adjustment[3], gfit.ry);
-	gtk_adjustment_set_value(selection_black_adjustment[0], 0);
-	gtk_adjustment_set_value(selection_black_adjustment[1], 0);
-	gtk_adjustment_set_value(selection_black_adjustment[2], 0);
-	gtk_adjustment_set_value(selection_black_adjustment[3], 0);
-
-	gtk_adjustment_set_upper(selection_white_adjustment[0], gfit.rx);
-	gtk_adjustment_set_upper(selection_white_adjustment[1], gfit.ry);
-	gtk_adjustment_set_upper(selection_white_adjustment[2], gfit.rx);
-	gtk_adjustment_set_upper(selection_white_adjustment[3], gfit.ry);
-	gtk_adjustment_set_value(selection_white_adjustment[0], 0);
-	gtk_adjustment_set_value(selection_white_adjustment[1], 0);
-	gtk_adjustment_set_value(selection_white_adjustment[2], 0);
-	gtk_adjustment_set_value(selection_white_adjustment[3], 0);
-}
 
 /* This function equalize the background by giving equal value for all layers */
-static void background_neutralize(fits* fit, rectangle black_selection) {
+void background_neutralize(fits* fit, rectangle black_selection) {
 	int chan;
 	size_t i, n = fit->naxes[0] * fit->naxes[1];
 	imstats* stats[3];
@@ -747,7 +882,7 @@ static void background_neutralize(fits* fit, rectangle black_selection) {
 	assert(fit->naxes[2] == 3);
 
 	for (chan = 0; chan < 3; chan++) {
-		stats[chan] = statistics(NULL, -1, fit, chan, &black_selection, STATS_BASIC, TRUE);
+		stats[chan] = statistics(NULL, -1, fit, chan, &black_selection, STATS_BASIC, MULTI_THREADED);
 		if (!stats[chan]) {
 			siril_log_message(_("Error: statistics computation failed.\n"));
 			return;
@@ -781,73 +916,7 @@ static void background_neutralize(fits* fit, rectangle black_selection) {
 	invalidate_gfit_histogram();
 }
 
-void on_button_bkg_neutralization_clicked(GtkButton *button, gpointer user_data) {
-	static GtkSpinButton *selection_black_value[4] = { NULL, NULL, NULL, NULL };
-	rectangle black_selection;
-	int width, height;
-
-	if (!selection_black_value[0]) {
-		selection_black_value[0] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(gui.builder, "spin_bkg_x"));
-		selection_black_value[1] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(gui.builder, "spin_bkg_y"));
-		selection_black_value[2] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(gui.builder, "spin_bkg_w"));
-		selection_black_value[3] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(gui.builder, "spin_bkg_h"));
-	}
-	width = (int) gtk_spin_button_get_value(selection_black_value[2]);
-	height = (int) gtk_spin_button_get_value(selection_black_value[3]);
-
-	if ((!width) || (!height)) {
-		siril_message_dialog( GTK_MESSAGE_WARNING, _("There is no selection"),
-				_("Make a selection of the background area"));
-		return;
-	}
-	black_selection.x = gtk_spin_button_get_value(selection_black_value[0]);
-	black_selection.y = gtk_spin_button_get_value(selection_black_value[1]);
-	black_selection.w = gtk_spin_button_get_value(selection_black_value[2]);
-	black_selection.h = gtk_spin_button_get_value(selection_black_value[3]);
-
-	undo_save_state(&gfit, _("Background neutralization"));
-
-	set_cursor_waiting(TRUE);
-	background_neutralize(&gfit, black_selection);
-	delete_selected_area();
-
-	update_gfit_histogram_if_needed();
-	redraw(REMAP_ALL);
-	redraw_previews();
-	set_cursor_waiting(FALSE);
-}
-
-void on_button_white_selection_clicked(GtkButton *button, gpointer user_data) {
-	static GtkSpinButton *selection_white_value[4] = { NULL, NULL, NULL, NULL };
-
-	if (!selection_white_value[0]) {
-		selection_white_value[0] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(gui.builder, "spin_white_x"));
-		selection_white_value[1] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(gui.builder, "spin_white_y"));
-		selection_white_value[2] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(gui.builder, "spin_white_w"));
-		selection_white_value[3] = GTK_SPIN_BUTTON(
-				gtk_builder_get_object(gui.builder, "spin_white_h"));
-	}
-
-	if ((!com.selection.h) || (!com.selection.w)) {
-		siril_message_dialog( GTK_MESSAGE_WARNING, _("There is no selection"),
-				_("Make a selection of the white reference area"));
-		return;
-	}
-
-	gtk_spin_button_set_value(selection_white_value[0], com.selection.x);
-	gtk_spin_button_set_value(selection_white_value[1], com.selection.y);
-	gtk_spin_button_set_value(selection_white_value[2], com.selection.w);
-	gtk_spin_button_set_value(selection_white_value[3], com.selection.h);
-}
-
-static void get_coeff_for_wb(fits *fit, rectangle white, rectangle black,
+void get_coeff_for_wb(fits *fit, rectangle white, rectangle black,
 		double kw[], double bg[], double norm, double low, double high) {
 	int chan, i, j, n;
 	double tmp[3] = { 0.0, 0.0, 0.0 };
@@ -904,7 +973,7 @@ static void get_coeff_for_wb(fits *fit, rectangle white, rectangle black,
 
 	siril_log_message(_("Background reference:\n"));
 	for (chan = 0; chan < 3; chan++) {
-		imstats *stat = statistics(NULL, -1, fit, chan, &black, STATS_BASIC, TRUE);
+		imstats *stat = statistics(NULL, -1, fit, chan, &black, STATS_BASIC, MULTI_THREADED);
 		if (!stat) {
 			siril_log_message(_("Error: statistics computation failed.\n"));
 			return;
@@ -937,7 +1006,7 @@ static void get_coeff_for_wb(fits *fit, rectangle white, rectangle black,
 	}
 }
 
-static int calibrate(fits *fit, int layer, double kw, double bg, double norm) {
+int calibrate(fits *fit, int layer, double kw, double bg, double norm) {
 	size_t i, n = fit->naxes[0] * fit->naxes[1];
 	if (fit->type == DATA_USHORT) {
 		double bgNorm = bg * norm;
@@ -956,133 +1025,6 @@ static int calibrate(fits *fit, int layer, double kw, double bg, double norm) {
 	return 0;
 }
 
-static void white_balance(fits *fit, gboolean is_manual, rectangle white_selection,
-		rectangle black_selection) {
-	int chan;
-	double norm, low, high;
-	double kw[3] = { 0.0, 0.0, 0.0 };
-	double bg[3] = { 0.0, 0.0, 0.0 };
-	static GtkRange *scale_white_balance[3] = { NULL, NULL, NULL };
-	static GtkRange *scaleLimit[2] = { NULL, NULL };
-
-	if (scale_white_balance[RLAYER] == NULL) {
-		scale_white_balance[RLAYER] = GTK_RANGE(lookup_widget("scale_r"));
-		scale_white_balance[GLAYER] = GTK_RANGE(lookup_widget("scale_g"));
-		scale_white_balance[BLAYER] = GTK_RANGE(lookup_widget("scale_b"));
-
-		scaleLimit[0] = GTK_RANGE(lookup_widget("lowWhiteColorCalibScale"));
-		scaleLimit[1] = GTK_RANGE(lookup_widget("upWhiteColorCalibScale"));
-	}
-
-	assert(fit->naxes[2] == 3);
-	norm = get_normalized_value(fit);
-
-	if (is_manual) {
-		kw[RLAYER] = gtk_range_get_value(scale_white_balance[RLAYER]);
-		kw[GLAYER] = gtk_range_get_value(scale_white_balance[GLAYER]);
-		kw[BLAYER] = gtk_range_get_value(scale_white_balance[BLAYER]);
-
-	} else {
-		low = gtk_range_get_value(scaleLimit[0]);
-		high = gtk_range_get_value(scaleLimit[1]);
-		get_coeff_for_wb(fit, white_selection, black_selection, kw, bg, norm, low, high);
-	}
-#ifdef _OPENMP
-#pragma omp parallel for num_threads(com.max_thread) private(chan) schedule(static)
-#endif
-	for (chan = 0; chan < 3; chan++) {
-		if (kw[chan] == 1.0) continue;
-		calibrate(fit, chan, kw[chan], bg[chan], norm);
-	}
-
-	invalidate_stats_from_fit(fit);
-	invalidate_gfit_histogram();
-}
-
-void on_calibration_apply_button_clicked(GtkButton *button, gpointer user_data) {
-	rectangle black_selection, white_selection;
-	static GtkSpinButton *selection_black_value[4] = { NULL, NULL, NULL, NULL };
-	static GtkSpinButton *selection_white_value[4] = { NULL, NULL, NULL, NULL };
-	struct timeval t_start, t_end;
-
-	siril_log_color_message(_("Color Calibration: processing...\n"), "green");
-	gettimeofday(&t_start, NULL);
-
-	GtkToggleButton *manual = GTK_TOGGLE_BUTTON(lookup_widget("checkbutton_manual_calibration"));
-	gboolean is_manual = gtk_toggle_button_get_active(manual);
-
-	if (!selection_black_value[0]) {
-		selection_black_value[0] = GTK_SPIN_BUTTON(lookup_widget("spin_bkg_x"));
-		selection_black_value[1] = GTK_SPIN_BUTTON(lookup_widget("spin_bkg_y"));
-		selection_black_value[2] = GTK_SPIN_BUTTON(lookup_widget("spin_bkg_w"));
-		selection_black_value[3] = GTK_SPIN_BUTTON(lookup_widget("spin_bkg_h"));
-	}
-
-	if (!selection_white_value[0]) {
-		selection_white_value[0] = GTK_SPIN_BUTTON(lookup_widget("spin_white_x"));
-		selection_white_value[1] = GTK_SPIN_BUTTON(lookup_widget("spin_white_y"));
-		selection_white_value[2] = GTK_SPIN_BUTTON(lookup_widget("spin_white_w"));
-		selection_white_value[3] = GTK_SPIN_BUTTON(lookup_widget("spin_white_h"));
-	}
-
-	black_selection.x = gtk_spin_button_get_value(selection_black_value[0]);
-	black_selection.y = gtk_spin_button_get_value(selection_black_value[1]);
-	black_selection.w = gtk_spin_button_get_value(selection_black_value[2]);
-	black_selection.h = gtk_spin_button_get_value(selection_black_value[3]);
-
-	if ((!black_selection.w || !black_selection.h) && !is_manual) {
-		siril_message_dialog( GTK_MESSAGE_WARNING, _("There is no selection"),
-				_("Make a selection of the background area"));
-		return;
-	}
-
-	white_selection.x = gtk_spin_button_get_value(selection_white_value[0]);
-	white_selection.y = gtk_spin_button_get_value(selection_white_value[1]);
-	white_selection.w = gtk_spin_button_get_value(selection_white_value[2]);
-	white_selection.h = gtk_spin_button_get_value(selection_white_value[3]);
-
-	if ((!white_selection.w || !white_selection.h) && !is_manual) {
-		siril_message_dialog( GTK_MESSAGE_WARNING, _("There is no selection"),
-				_("Make a selection of the white reference area"));
-		return;
-	}
-
-	set_cursor_waiting(TRUE);
-	undo_save_state(&gfit, _("Color Calibration"));
-	white_balance(&gfit, is_manual, white_selection, black_selection);
-
-	gettimeofday(&t_end, NULL);
-
-	show_time(t_start, t_end);
-
-	delete_selected_area();
-
-	redraw(REMAP_ALL);
-	redraw_previews();
-	update_gfit_histogram_if_needed();
-	set_cursor_waiting(FALSE);
-}
-
-void on_calibration_close_button_clicked(GtkButton *button, gpointer user_data) {
-	siril_close_dialog("color_calibration");
-}
-
-void on_checkbutton_manual_calibration_toggled(GtkToggleButton *togglebutton,
-		gpointer user_data) {
-	GtkWidget *cc_box_red = lookup_widget("cc_box_red");
-	GtkWidget *scale_r = lookup_widget("scale_r");
-	GtkWidget *cc_box_green = lookup_widget("cc_box_green");
-	GtkWidget *scale_g = lookup_widget("scale_g");
-	GtkWidget *cc_box_blue = lookup_widget("cc_box_blue");
-	GtkWidget *scale_b = lookup_widget("scale_b");
-	gtk_widget_set_sensitive(cc_box_red, gtk_toggle_button_get_active(togglebutton));
-	gtk_widget_set_sensitive(scale_r, gtk_toggle_button_get_active(togglebutton));
-	gtk_widget_set_sensitive(cc_box_green, gtk_toggle_button_get_active(togglebutton));
-	gtk_widget_set_sensitive(scale_g, gtk_toggle_button_get_active(togglebutton));
-	gtk_widget_set_sensitive(cc_box_blue, gtk_toggle_button_get_active(togglebutton));
-	gtk_widget_set_sensitive(scale_b, gtk_toggle_button_get_active(togglebutton));
-}
-
 int pos_to_neg(fits *fit) {
 	size_t i, n = fit->naxes[0] * fit->naxes[1] * fit->naxes[2];
 	if (fit->type == DATA_USHORT) {
@@ -1107,103 +1049,3 @@ int pos_to_neg(fits *fit) {
 	return 0;
 }
 
-void negative_processing() {
-	set_cursor_waiting(TRUE);
-	undo_save_state(&gfit, _("Negative Transformation"));
-	pos_to_neg(&gfit);
-	invalidate_stats_from_fit(&gfit);
-	invalidate_gfit_histogram();
-	update_gfit_histogram_if_needed();
-	redraw(REMAP_ALL);
-	redraw_previews();
-	set_cursor_waiting(FALSE);
-}
-
-/**********************************************************************/
-
-void on_extract_channel_button_close_clicked(GtkButton *button,
-		gpointer user_data) {
-	siril_close_dialog("extract_channel_dialog");
-}
-
-void on_combo_extract_colors_changed(GtkComboBox *box, gpointer user_data) {
-	switch(gtk_combo_box_get_active(box)) {
-		default:
-		case 0: // RGB
-			gtk_label_set_text(GTK_LABEL(lookup_widget("label_extract_c1")), _("Red: "));
-			gtk_label_set_text(GTK_LABEL(lookup_widget("label_extract_c2")), _("Green: "));
-			gtk_label_set_text(GTK_LABEL(lookup_widget("label_extract_c3")), _("Blue: "));
-			break;
-		case 1: // HSL
-			gtk_label_set_text(GTK_LABEL(lookup_widget("label_extract_c1")), _("Hue: "));
-			gtk_label_set_text(GTK_LABEL(lookup_widget("label_extract_c2")), _("Saturation: "));
-			gtk_label_set_text(GTK_LABEL(lookup_widget("label_extract_c3")), _("Lightness: "));
-			break;
-		case 2: // HSV
-			gtk_label_set_text(GTK_LABEL(lookup_widget("label_extract_c1")), _("Hue: "));
-			gtk_label_set_text(GTK_LABEL(lookup_widget("label_extract_c2")), _("Saturation: "));
-			gtk_label_set_text(GTK_LABEL(lookup_widget("label_extract_c3")), _("Value: "));
-			break;
-		case 3: // CIE L*a*b*
-			gtk_label_set_text(GTK_LABEL(lookup_widget("label_extract_c1")), "L*: ");
-			gtk_label_set_text(GTK_LABEL(lookup_widget("label_extract_c2")), "a*: ");
-			gtk_label_set_text(GTK_LABEL(lookup_widget("label_extract_c3")), "b*: ");
-	}
-}
-
-void on_extract_channel_button_ok_clicked(GtkButton *button, gpointer user_data) {
-	static GtkEntry *channel_extract_entry[3] = { NULL, NULL, NULL };
-	static GtkComboBox *combo_extract_channel = NULL;
-
-	if (get_thread_run()) {
-		PRINT_ANOTHER_THREAD_RUNNING;
-		return;
-	}
-
-	struct extract_channels_data *args = malloc(sizeof(struct extract_channels_data));
-	if (!args) {
-		PRINT_ALLOC_ERR;
-		return;
-	}
-
-	if (combo_extract_channel == NULL) {
-		combo_extract_channel = GTK_COMBO_BOX(
-				lookup_widget("combo_extract_colors"));
-		channel_extract_entry[0] = GTK_ENTRY(
-				lookup_widget("Ch1_extract_channel_entry"));
-		channel_extract_entry[1] = GTK_ENTRY(
-				lookup_widget("Ch2_extract_channel_entry"));
-		channel_extract_entry[2] = GTK_ENTRY(
-				lookup_widget("Ch3_extract_channel_entry"));
-	}
-
-	args->type = gtk_combo_box_get_active(combo_extract_channel);
-	args->str_type = gtk_combo_box_get_active_id(combo_extract_channel);
-
-	args->channel[0] = g_strdup_printf("%s%s", gtk_entry_get_text(channel_extract_entry[0]), com.pref.ext);
-	args->channel[1] = g_strdup_printf("%s%s", gtk_entry_get_text(channel_extract_entry[1]), com.pref.ext);
-	args->channel[2] = g_strdup_printf("%s%s", gtk_entry_get_text(channel_extract_entry[2]), com.pref.ext);
-
-	if ((args->channel[0][0] != '\0') && (args->channel[1][0] != '\0')
-			&& (args->channel[2][0] != '\0')) {
-		args->fit = calloc(1, sizeof(fits));
-		set_cursor_waiting(TRUE);
-		if (copyfits(&gfit, args->fit, CP_ALLOC | CP_COPYA | CP_FORMAT, -1)) {
-			siril_log_message(_("Could not copy the input image, aborting.\n"));
-			free(args->fit);
-			free(args->channel[0]);
-			free(args->channel[1]);
-			free(args->channel[2]);
-			free(args);
-		} else {
-			copy_fits_metadata(&gfit, args->fit);
-			start_in_new_thread(extract_channels, args);
-		}
-	}
-	else {
-		free(args->channel[0]);
-		free(args->channel[1]);
-		free(args->channel[2]);
-		free(args);
-	}
-}
