@@ -7,23 +7,23 @@
 struct crop_sequence_data {
 	sequence *seq;
 	rectangle area;
-	const char *prefix;
+	char *prefix;
 	int retvalue;
 };
 
-int verbose_resize_gaussian(fits *, int, int, int);
-int verbose_rotate_image(fits *, double, int, int);
+int fits_binning(fits *fit, int factor, gboolean mean);
 
-void siril_rotate90();
-void siril_rotate270();
+int verbose_resize_gaussian(fits *image, int toX, int toY, opencv_interpolation interpolation, gboolean clamp);
+
+int verbose_rotate_image(fits *, rectangle, double, int, int, gboolean);
+int verbose_rotate_fast(fits *image, int angle);
 
-void mirrorx_gui(fits *fit);
-void mirrory_gui(fits *fit);
 void mirrorx(fits *fit, gboolean verbose);
 void mirrory(fits *fit, gboolean verbose);
 
 int crop(fits *fit, rectangle *bounds);
-void siril_crop();
 gpointer crop_sequence(struct crop_sequence_data *crop_sequence_data);
 
+const char *interp_to_str(opencv_interpolation interpolation);
+
 #endif /* SRC_ALGOS_GEOMETRY_H_ */
