@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -34,7 +34,7 @@
 
 static float SubSample(float *ptr, int img_wid, int x_size, int y_size);
 static void _smooth_image_float(float *buf, int width, int height);
-static double Gradient(float *buf, int width, int height);
+static double Gradient(const float *buf, int width, int height);
 
 // -------------------------------------------------------
 // Method to estimate quality.
@@ -127,7 +127,7 @@ double QualityEstimate_float(fits *fit, int layer) {
 
 		double q = Gradient(buf, x_samples, y_samples);
 
-		dval += (q * ((QSUBSAMPLE_MIN * QSUBSAMPLE_MIN)
+		dval += (q * ((double)(QSUBSAMPLE_MIN * QSUBSAMPLE_MIN)
 					/ (subsample * subsample)));
 		//printf("dval val : %f\n", dval);
 
@@ -165,7 +165,7 @@ static float SubSample(float *ptr, int img_wid, int x_size, int y_size) {
 	return val / (float)(x_size * y_size);
 }
 
-static double Gradient(float *buf, int width, int height) {
+static double Gradient(const float *buf, int width, int height) {
 	int pixels;
 	int x, y;
 	int yborder = (int) ((double) height * QMARGIN) + 1;
