@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -29,7 +29,9 @@
 #include "core/proto.h"
 #include "core/command.h" // process_close
 #include "core/OS_utils.h"
+#include "core/siril_log.h"
 #include "algos/colors.h"
+#include "algos/siril_wcs.h"
 #include "io/sequence.h"
 #include "io/single_image.h"
 #include "io/image_format_fits.h"
@@ -42,6 +44,7 @@
 #include "gui/dialogs.h"
 #include "gui/progress_and_log.h"
 #include "gui/sequence_list.h"
+#include "gui/colors.h"
 #include "registration/registration.h"
 #include "stacking/stacking.h"
 #include "opencv/opencv.h"
@@ -136,9 +139,9 @@ static void populate_filter_lists();
 static void coeff_clear();
 
 /* callbacks for programatic GTK */
-void on_layer_remove(GtkButton *button, gpointer user_data);
-gboolean on_color_button_press_event(GtkDrawingArea *widget, GdkEventButton *event, gpointer user_data);
-gboolean on_color_button_release_event(GtkDrawingArea *widget, GdkEventButton *event, gpointer user_data);
+void on_layer_remove(const GtkButton *button, gpointer user_data);
+gboolean on_color_button_press_event(const GtkDrawingArea *widget, GdkEventButton *event, gpointer user_data);
+gboolean on_color_button_release_event(const GtkDrawingArea *widget, GdkEventButton *event, gpointer user_data);
 gboolean on_color_button_motion_event(GtkWidget *widget, GdkEventMotion *event, gpointer user_data);
 gboolean draw_layer_color(GtkDrawingArea *widget, cairo_t *cr, gpointer data);
 void on_filechooser_file_set(GtkFileChooserButton *widget, gpointer user_data);
@@ -178,6 +181,7 @@ layer *create_layer(int index) {
 	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(ret->chooser),
 			GTK_FILE_FILTER(gtk_builder_get_object(gui.builder, "filefilter1")));
 			gtk_file_chooser_button_set_width_chars(ret->chooser, 16);
+	gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(ret->chooser), FALSE);
 	g_signal_connect(ret->chooser, "file-set", G_CALLBACK(on_filechooser_file_set), NULL);
 	g_object_ref(G_OBJECT(ret->chooser));	// don't destroy it on removal from grid
 
@@ -245,7 +249,7 @@ static void add_the_layer_add_button() {
 }
 
 /* callback of the '-' button that is clicked to remove a layer in the list */
-void on_layer_remove(GtkButton *button, gpointer user_data) {
+void on_layer_remove(const GtkButton *button, gpointer user_data) {
 	int layer, refresh = 0;
 	for (layer = 1; layers[layer]; layer++)
 		if (layers[layer]->remove_button == button)
@@ -373,15 +377,15 @@ void open_compositing_window() {
 			i++;
 		}
 		if (i > 0) {
-			gtk_combo_box_set_active(GTK_COMBO_BOX(aligncombo), com.reg_settings);
+			gtk_combo_box_set_active(GTK_COMBO_BOX(aligncombo), com.pref.gui.reg_settings);
 		}
 
 		compositing_loaded = 1;
 	} else {
 		/* not the first load, update the CWD just in case it changed in the meantime */
 		i = 0;
-		close_single_image();
 		close_sequence(FALSE);
+		close_single_image();
 		do {
 			gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(layers[i]->chooser), com.wd);
 			i++;
@@ -454,7 +458,7 @@ static void check_gfit_is_ours() {
 
 	/* create the new result image if it's the first opened image */
 	close_single_image();
-	if (copyfits(&layers[update_from_layer]->the_fit, &gfit, CP_ALLOC | CP_FORMAT | CP_EXPAND, -1)) {
+	if (copyfits(&layers[update_from_layer]->the_fit, &gfit, CP_ALLOC | CP_FORMAT | CP_INIT | CP_EXPAND, -1)) {
 		clearfits(&layers[update_from_layer]->the_fit);
 		siril_log_color_message(_("Could not display image, unloading it\n"), "red");
 		return;
@@ -462,7 +466,7 @@ static void check_gfit_is_ours() {
 
 	/* open the single image.
 	 * code taken from stacking.c:start_stacking() and read_single_image() */
-	clear_stars_list();
+	clear_stars_list(TRUE);
 	com.seq.current = UNRELATED_IMAGE;
 	if (!create_uniq_from_gfit(strdup(_("Unsaved compositing result")), FALSE))
 		com.uniq->comment = strdup(_("Compositing result image"));
@@ -482,6 +486,19 @@ static void check_gfit_is_ours() {
 	sequence_list_change_current();
 }
 
+static void update_metadata() {
+	int nb = number_of_images_loaded();
+	fits **f = malloc((nb + 1) * sizeof(fits *));
+	int j = 0;
+	for (int i = 0; layers[i] ; i++)
+		if (has_fit(i))
+			f[j++] = &layers[i]->the_fit;
+	f[j] = NULL;
+
+	merge_fits_headers_to_result2(&gfit, f);
+	load_WCS_from_memory(&gfit);
+	free(f);
+}
 
 /* callback for the file chooser's file selection: try to load the pointed file, allocate the
  * destination image if this is the first image, and update the result. */
@@ -501,7 +518,7 @@ void on_filechooser_file_set(GtkFileChooserButton *widget, gpointer user_data) {
 	if (!layers[layer]) return;	// not found
 	/* layer is the number of the opened layer */
 
-	filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget));
+	filename = siril_file_chooser_get_filename(GTK_FILE_CHOOSER(widget));
 	if (!filename) return;
 	if (layers[layer]->the_fit.rx == 0) {	// already loaded image
 		clearfits(&layers[layer]->the_fit);
@@ -517,7 +534,7 @@ void on_filechooser_file_set(GtkFileChooserButton *widget, gpointer user_data) {
 			gtk_widget_set_tooltip_text(GTK_WIDGET(layers[layer]->label), _("Only single channel images can be load"));
 			retval = 1;
 		} else {
-		/* Force first tab to be Red and not B&W if an image was already loaded */
+			/* Force first tab to be Red and not B&W if an image was already loaded */
 			GtkNotebook* Color_Layers = GTK_NOTEBOOK(gtk_builder_get_object(gui.builder, "notebook1"));
 			GtkWidget *page = gtk_notebook_get_nth_page(Color_Layers, RED_VPORT);
 			gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(Color_Layers), page, _("Red"));
@@ -543,7 +560,7 @@ void on_filechooser_file_set(GtkFileChooserButton *widget, gpointer user_data) {
 							layers[layer]->the_fit.ry, gfit.rx, gfit.ry);
 						sprintf(buf, _("OK upscaled from %ux%u"),
 								layers[layer]->the_fit.rx, layers[layer]->the_fit.ry);
-						cvResizeGaussian(&layers[layer]->the_fit, gfit.rx, gfit.ry, OPENCV_AREA);
+						cvResizeGaussian(&layers[layer]->the_fit, gfit.rx, gfit.ry, OPENCV_AREA, FALSE);
 						gtk_label_set_text(layers[layer]->label, buf);
 						gtk_widget_set_tooltip_text(GTK_WIDGET(layers[layer]->label), _("Image loaded, and upscaled"));
 				}
@@ -573,6 +590,7 @@ void on_filechooser_file_set(GtkFileChooserButton *widget, gpointer user_data) {
 	// enable the color balance finalization button
 	gtk_widget_set_sensitive(lookup_widget("composition_rgbcolor"), number_of_images_loaded() > 1);
 	update_result(1);
+	update_metadata();
 	update_MenuItem();
 }
 
@@ -613,8 +631,8 @@ void create_the_internal_sequence() {
 
 /* start aligning the layers: create an 'internal' sequence and run the selected method on it */
 void on_button_align_clicked(GtkButton *button, gpointer user_data) {
-	int i, j;
-	struct registration_args regargs;
+	int i = 0;
+	struct registration_args regargs = { 0 };
 	struct registration_method *method;
 	char *msg;
 	GtkComboBox *regcombo;
@@ -626,7 +644,6 @@ void on_button_align_clicked(GtkButton *button, gpointer user_data) {
 	method = reg_methods[gtk_combo_box_get_active(regcombo)];
 
 	regargs.seq = seq;
-	regargs.process_all_frames = TRUE;
 	get_the_registration_area(&regargs, method);
 	regargs.layer = 0;	// TODO: fix with dynamic layers list
 	regargs.run_in_thread = FALSE;
@@ -643,21 +660,16 @@ void on_button_align_clicked(GtkButton *button, gpointer user_data) {
 	com.run_thread = FALSE;	// fix for the cancelling check in processing
 
 	/* display the values */
-	if (luminance_mode) {
-		for (j=0, i=0; i<layers_count; i++) {
-			if (has_fit(i)) {
-				gtk_spin_button_set_value(layers[i]->spinbutton_x, roundf_to_int(seq->regparam[0][j].shiftx));
-				gtk_spin_button_set_value(layers[i]->spinbutton_y, roundf_to_int(seq->regparam[0][j].shifty));
-				j++;
-			}
-		}
-	} else {
-		for (j=0, i=1; i<layers_count; i++) {
-			if (has_fit(i)) {
-				gtk_spin_button_set_value(layers[i]->spinbutton_x, roundf_to_int(seq->regparam[0][j].shiftx));
-				gtk_spin_button_set_value(layers[i]->spinbutton_y, roundf_to_int(seq->regparam[0][j].shifty));
-				j++;
-			}
+	if (!luminance_mode)
+		i = 1;
+
+	for (int j=0; i<layers_count; i++) {
+		if (has_fit(i)) {
+			double dx, dy;
+			translation_from_H(seq->regparam[0][j].H, &dx, &dy);
+			gtk_spin_button_set_value(layers[i]->spinbutton_x, dx);
+			gtk_spin_button_set_value(layers[i]->spinbutton_y, dy);
+			j++;
 		}
 	}
 
@@ -682,10 +694,12 @@ float get_normalized_pixel_value(int fits_index, float layer_pixel_value) {
 static float get_composition_pixel_value(int fits_index, int reg_layer, int x, int y) {
 	int realX = x, realY = y;
 	if (seq && seq->regparam && reg_layer < seq->number && reg_layer >= 0) {
+		double dx, dy;
+		translation_from_H(seq->regparam[0][reg_layer].H, &dx, &dy);
 		// all images have one layer, hence the 0 below
-		realX = x - roundf_to_int(seq->regparam[0][reg_layer].shiftx);
+		realX = x - round_to_int(dx);
 		if (realX < 0 || realX >= gfit.rx) return 0.0f;
-		realY = y - roundf_to_int(seq->regparam[0][reg_layer].shifty);
+		realY = y - round_to_int(dy);
 		if (realY < 0 || realY >= gfit.ry) return 0.0f;
 	}
 	float pixel_value = layers[fits_index]->the_fit.fpdata[0][realX + realY * gfit.rx];
@@ -849,12 +863,12 @@ static void luminance_and_colors_align_and_compose() {
 				hsv_to_rgb(h, s, i, &pixel.red, &pixel.green, &pixel.blue);
 				break;
 			case CIELAB:
-				rgb_to_xyz(pixel.red, pixel.green, pixel.blue, &X, &Y, &Z);
+				linrgb_to_xyz(pixel.red, pixel.green, pixel.blue, &X, &Y, &Z, TRUE);
 				xyz_to_LAB(X, Y, Z, &i, &a, &b);
 				i = (double) get_composition_pixel_value(0, 0, x, y) / norm;
 				i *= 100.0;		// 0 < L < 100
 				LAB_to_xyz(i, a, b, &X, &Y, &Z);
-				xyz_to_rgb(X, Y, Z, &pixel.red, &pixel.green, &pixel.blue);
+				xyz_to_linrgb(X, Y, Z, &pixel.red, &pixel.green, &pixel.blue, TRUE);
 				break;
 			}
 
@@ -988,7 +1002,7 @@ gboolean draw_layer_color(GtkDrawingArea *widget, cairo_t *cr, gpointer data) {
 
 /* click on the colored area: button press, only configure for quick color edit with
  * right button */
-gboolean on_color_button_press_event(GtkDrawingArea *widget, GdkEventButton *event, gpointer user_data) {
+gboolean on_color_button_press_event(const GtkDrawingArea *widget, GdkEventButton *event, gpointer user_data) {
 	int layer;
 	for (layer = 0; layers[layer]; layer++)
 		if (layers[layer]->color_w == widget)
@@ -1004,7 +1018,7 @@ gboolean on_color_button_press_event(GtkDrawingArea *widget, GdkEventButton *eve
 }
 
 /* click on the colored area: open the color chooser dialog */
-gboolean on_color_button_release_event(GtkDrawingArea *widget, GdkEventButton *event, gpointer user_data) {
+gboolean on_color_button_release_event(const GtkDrawingArea *widget, GdkEventButton *event, gpointer user_data) {
 	int layer;
 	for (layer = 0; layers[layer]; layer++)
 		if (layers[layer]->color_w == widget)
@@ -1176,9 +1190,9 @@ void autoadjust(int force_redraw) {
 	/* update the real colours of layers from their saturated colour, based
 	 * on how much each colour of the composition overflows */
 	// amounts of normalization to be done on each layer's image for each channel
-	double to_redistribute_red = (max_pixel.red - 1.0) / (double)nb_images_red;
-	double to_redistribute_green = (max_pixel.green - 1.0) / (double)nb_images_green;
-	double to_redistribute_blue = (max_pixel.blue - 1.0) / (double)nb_images_blue;
+	double to_redistribute_red = nb_images_red == 0 ? 0.0 : (max_pixel.red - 1.0) / (double)nb_images_red;
+	double to_redistribute_green = nb_images_green == 0 ? 0.0 : (max_pixel.green - 1.0) / (double)nb_images_green;
+	double to_redistribute_blue = nb_images_blue == 0 ? 0.0 : (max_pixel.blue - 1.0) / (double)nb_images_blue;
 	for (layer = 1; layers[layer]; layer++) {
 		if (has_fit(layer)) {
 			double to_redistribute = 0.0;	// for this layer
@@ -1240,4 +1254,3 @@ void on_composition_rgbcolor_clicked(GtkButton *button, gpointer user_data){
 	gtk_widget_show(win);
 }
 
-
