@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2019 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -42,7 +42,9 @@ extern "C" {
 #define WIDEN(s) (s)
 #endif
 
+#ifdef HAVE_EXIV2
 #include <exiv2/exiv2.hpp>
+#endif
 
 #include "exif.h"
 
@@ -74,6 +76,7 @@ public:
  * Get the largest possible thumbnail from the image
  */
 int siril_get_thumbnail_exiv(const char *path, uint8_t **buffer, size_t *size, char **mime_type) {
+#ifdef HAVE_EXIV2
 	try {
 		std::unique_ptr<Exiv2::Image> image(Exiv2::ImageFactory::open(WIDEN(path)));
 		assert(image.get() != 0);
@@ -108,9 +111,56 @@ int siril_get_thumbnail_exiv(const char *path, uint8_t **buffer, size_t *size, c
 		//std::cerr << "[exiv2] "<< path << ": found thumbnail "<< preview.width() << "x" << preview.height() << std::endl;
 		memcpy(*buffer, tmp, _size);
 		return 0;
-	} catch (Exiv2::AnyError &e) {
+	} catch (Exiv2::Error &e) {
 		std::string s(e.what());
 		std::cerr << "[exiv2]: " << s << std::endl;
 		return 1;
 	}
+#else
+	return 1;
+#endif
+}
+
+#ifdef HAVE_EXIV2
+#if EXIV2_TEST_VERSION(0,28,0)
+typedef Exiv2::Image::UniquePtr ImagePtr;
+#else
+typedef Exiv2::Image::AutoPtr ImagePtr;
+#endif
+#endif
+
+gchar* siril_get_date_from_exif(const char *filename) {
+#ifdef HAVE_EXIV2
+	try {
+		ImagePtr image = Exiv2::ImageFactory::open(WIDEN(filename));
+		if (image.get() == 0) {
+			fprintf(stderr, "Error Cannot open the file.\n");
+			return NULL;
+		}
+
+		read_metadata_threadsafe(image);
+
+		Exiv2::ExifData &exif_data = image->exifData();
+		if (exif_data.empty()) {
+			fprintf(stderr, "Error: Unable to read EXIF metadata\n");
+			return NULL;
+		}
+
+		Exiv2::ExifData::const_iterator iter = exif_data.findKey(Exiv2::ExifKey("Exif.Image.DateTime"));
+		if (iter == exif_data.end()) {
+			fprintf(stderr, "Error: Unable to find the date in the EXIF metadata\n");
+			return NULL;
+		}
+
+		std::string date_str = iter->value().toString();
+
+		return g_strdup(date_str.c_str());
+	} catch (Exiv2::Error& e) {
+		std::string s(e.what());
+		std::cerr << "[exiv2]: " << s << std::endl;
+		return NULL;
+	}
+#else
+	return NULL;
+#endif
 }
