@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -24,6 +24,7 @@
 #include <assert.h>
 #include <ctype.h>
 #include <string.h>
+#include <gsl/gsl_statistics.h>
 #include "sequence_filtering.h"
 #include "proto.h" // is_readable_file()
 #include "registration/registration.h"
@@ -31,8 +32,7 @@
 #include "stacking/stacking.h"
 #include "gui/progress_and_log.h"
 #include "algos/sorting.h"
-
-#define MAX_FILTERS 5
+#include "core/siril_log.h"
 
 /******************* IMAGE FILTERING CRITERIA *******************/
 
@@ -86,6 +86,29 @@ int seq_filter_roundness(sequence *seq, int nb_img, double min_rnd) {
 	else return 0;
 }
 
+int seq_filter_background(sequence *seq, int nb_img, double max_bkg) {
+	int layer;
+	if (!seq->regparam) return 0;
+	layer = get_registration_layer(seq);
+	if (layer == -1) return 0;
+	if (!seq->regparam[layer]) return 0;
+	if (seq->regparam[layer][nb_img].roundness > 0.0f)
+		return seq->regparam[layer][nb_img].background_lvl <= max_bkg;
+	else return 0;
+}
+
+// we pass a double to keep all functions generic
+int seq_filter_nbstars(sequence *seq, int nb_img, double min_nbstars) {
+	int layer;
+	if (!seq->regparam) return 0;
+	layer = get_registration_layer(seq);
+	if (layer == -1) return 0;
+	if (!seq->regparam[layer]) return 0;
+	if (seq->regparam[layer][nb_img].roundness > 0.0f)
+		return seq->regparam[layer][nb_img].number_of_stars >= (int)min_nbstars;
+	else return 0;
+}
+
 /* filter for planetary */
 int seq_filter_quality(sequence *seq, int nb_img, double max_quality) {
 	int layer;
@@ -98,7 +121,7 @@ int seq_filter_quality(sequence *seq, int nb_img, double max_quality) {
 	else return 0;
 }
 
-/* browse the images to konw how many fit the criterion, from global data */
+/* browse the images to know how many fit the criterion, from global data */
 // ensure that com.seq is loaded before passing it as seq: sequence_is_loaded()
 int compute_nb_filtered_images(sequence *seq, seq_image_filter filtering_criterion, double filtering_parameter) {
 	int i, count = 0;
@@ -194,10 +217,10 @@ seq_image_filter create_multiple_filter_from_list(struct filtering_tuple *filter
 // creates the filtering criterion from a stacking configuration
 // raises an error if the configuration has duplicates
 // at least two images to be selected
-int convert_stack_data_to_filter(struct stacking_configuration *arg, struct stacking_args *stackargs) {
+int convert_parsed_filter_to_filter(struct seq_filter_config *arg, sequence *seq, seq_image_filter *criterion, double *param) {
 	int nb_filters = 0;
-	int layer = get_registration_layer(stackargs->seq);
-	struct filtering_tuple filters[5] = { { NULL, 0.0 } };
+	int layer = get_registration_layer(seq);
+	struct filtering_tuple filters[8] = { { NULL, 0.0 } };
 
 	if ((arg->f_fwhm_p > 0.0f && arg->f_fwhm > 0.0f) ||
 			(arg->f_wfwhm_p > 0.0f && arg->f_wfwhm > 0.0f) ||
@@ -208,39 +231,55 @@ int convert_stack_data_to_filter(struct stacking_configuration *arg, struct stac
 	}
 	if (arg->filter_included) {
 		filters[nb_filters].filter = seq_filter_included;
-		filters[nb_filters].param = stackargs->seq->selnum;
+		filters[nb_filters].param = seq->selnum;
 		siril_log_message(_("Using selected images filter (%d/%d of the sequence)\n"),
-				stackargs->seq->selnum, stackargs->seq->number);
+				seq->selnum, seq->number);
 		nb_filters++;
 	}
 	if (arg->f_fwhm_p > 0.0f || arg->f_fwhm > 0.0f) {
 		filters[nb_filters].filter = seq_filter_fwhm;
 		filters[nb_filters].param = arg->f_fwhm > 0.f ? arg->f_fwhm :
-				compute_highest_accepted_fwhm(stackargs->seq, layer, arg->f_fwhm_p);
+				compute_highest_accepted_fwhm(seq, layer, arg->f_fwhm_p, arg->f_fwhm_k);
 		siril_log_message(_("Using star FWHM images filter (below %f)\n"),
 					filters[nb_filters].param);
-		       	nb_filters++;
+		nb_filters++;
 	}
 	if (arg->f_wfwhm_p > 0.0f || arg->f_wfwhm > 0.0f) {
 		filters[nb_filters].filter = seq_filter_weighted_fwhm;
 		filters[nb_filters].param = arg->f_wfwhm > 0.f ? arg->f_wfwhm :
-				compute_highest_accepted_weighted_fwhm(stackargs->seq, layer, arg->f_wfwhm_p);
+				compute_highest_accepted_weighted_fwhm(seq, layer, arg->f_wfwhm_p, arg->f_wfwhm_k);
 		siril_log_message(_("Using star weighted FWHM images filter (below %f)\n"),
 					filters[nb_filters].param);
-		       	nb_filters++;
+		nb_filters++;
 	}
 	if (arg->f_round_p > 0.0f || arg->f_round > 0.0f) {
 		filters[nb_filters].filter = seq_filter_roundness;
 		filters[nb_filters].param = arg->f_round > 0.f ? arg->f_round :
-			compute_lowest_accepted_roundness(stackargs->seq, layer, arg->f_round_p);
+			compute_lowest_accepted_roundness(seq, layer, arg->f_round_p, arg->f_round_k);
 		siril_log_message(_("Using star roundness images filter (above %f)\n"),
 				filters[nb_filters].param);
 		nb_filters++;
 	}
+	if (arg->f_bkg_p > 0.0f || arg->f_bkg > 0.0f) {
+		filters[nb_filters].filter = seq_filter_background;
+		filters[nb_filters].param = arg->f_bkg > 0.f ? arg->f_bkg :
+				compute_highest_accepted_background(seq, layer, arg->f_bkg_p, arg->f_bkg_k);
+		siril_log_message(_("Using image background filter (below %f)\n"),
+					filters[nb_filters].param);
+				nb_filters++;
+	}
+	if (arg->f_nbstars_p > 0.0f || arg->f_nbstars > 0.0f) {
+		filters[nb_filters].filter = seq_filter_nbstars;
+		filters[nb_filters].param = arg->f_nbstars > 0.f ? arg->f_nbstars :
+			compute_lowest_accepted_nbstars(seq, layer, arg->f_nbstars_p, arg->f_nbstars_k);
+		siril_log_message(_("Using number of stars filter (above %f)\n"),
+				filters[nb_filters].param);
+		nb_filters++;
+	}
 	if (arg->f_quality_p > 0.0f || arg->f_quality > 0.0f) {
 		filters[nb_filters].filter = seq_filter_quality;
 		filters[nb_filters].param = arg->f_quality > 0.f ? arg->f_quality :
-			compute_lowest_accepted_quality(stackargs->seq, layer, arg->f_quality_p);
+			compute_lowest_accepted_quality(seq, layer, arg->f_quality_p, arg->f_quality_k);
 		siril_log_message(_("Using image quality filter (below %f)\n"),
 				filters[nb_filters].param);
 		nb_filters++;
@@ -248,16 +287,16 @@ int convert_stack_data_to_filter(struct stacking_configuration *arg, struct stac
 	filters[nb_filters].filter = NULL;
 
 	if (nb_filters == 0) {
-		stackargs->filtering_criterion = seq_filter_all;
-		stackargs->filtering_parameter = 0.0;
+		*criterion = seq_filter_all;
+		*param = 0.0;
 	}
 	else if (nb_filters == 1) {
-		stackargs->filtering_criterion = filters[0].filter;
-		stackargs->filtering_parameter = filters[0].param;
+		*criterion = filters[0].filter;
+		*param = filters[0].param;
 	}
 	else {
-		stackargs->filtering_criterion = create_multiple_filter_from_list(filters);
-		stackargs->filtering_parameter = -1.0;
+		*criterion = create_multiple_filter_from_list(filters);
+		*param = -1.0;
 	}
 	return 0;
 }
@@ -271,8 +310,10 @@ int setup_filtered_data(struct stacking_args *args) {
 		siril_log_message(_("Provided filtering options do not allow at least two images to be processed.\n"));
 		return 1;
 	}
-	if (args->image_indices)
+	if (args->image_indices) {
 		free(args->image_indices);
+		args->image_indices = NULL;
+	}
 	return stack_fill_list_of_unfiltered_images(args);
 }
 
@@ -290,7 +331,7 @@ int stack_fill_list_of_unfiltered_images(struct stacking_args *args) {
 		}
 		args->image_indices = newptr;
 	} else {
-		args->image_indices = malloc(args->nb_images_to_stack * sizeof(int));
+		args->image_indices = calloc(args->nb_images_to_stack, sizeof(int));
 		if (!args->image_indices) {
 			PRINT_ALLOC_ERR;
 			return 1;
@@ -323,6 +364,8 @@ static double regdata_fwhm(regdata *reg) { return reg->fwhm; }
 static double regdata_weighted_fwhm(regdata *reg) { return reg->weighted_fwhm; }
 static double regdata_roundness(regdata *reg) { return reg->roundness; }
 static double regdata_quality(regdata *reg) { return reg->quality; }
+static double regdata_background(regdata *reg) { return reg->background_lvl; }
+static double regdata_nbstars(regdata *reg) { return (double)reg->number_of_stars; }
 
 /* from a percentage, find the lowest or highest accepted registration property
  * value for image filtering in sequences. */
@@ -371,23 +414,106 @@ static double generic_compute_accepted_value(sequence *seq, int layer, double pe
 	return threshold;
 }
 
-double compute_highest_accepted_fwhm(sequence *seq, int layer, double percent) {
-	return generic_compute_accepted_value(seq, layer, percent, TRUE, regdata_fwhm);
+/* from a k value (for k-sigma rejection), find the lowest or highest accepted registration property
+ * value for image filtering in sequences. */
+static double generic_compute_accepted_value_with_rejection(sequence *seq, int layer, double k, gboolean lower_is_better, regdata_selector datasel) {
+	double threshold, *val;
+	double factor = (lower_is_better) ? 1. : -1; // if higher is better, we will use opposite values to always reject the upper tail
+	if (layer < 0 || !seq->regparam || !seq->regparam[layer]) {
+		return 0.0;
+	}
+	val = malloc(seq->number * sizeof(double));
+	if (!val) return 0.0;
+
+	// copy values
+	int n = 0;
+	for (int i = 0; i < seq->number; i++) {
+		double data = datasel(&seq->regparam[layer][i]);
+		if (data > 0.0f)
+			val[n++] = data * factor;
+	}
+	if (!n) {
+		free(val);
+		return 0.0;
+	}
+	if (n < seq->number) {
+		siril_log_message(_("Warning: some images don't have information available for best "
+				"images selection, using only available data (%d images on %d).\n"),
+				n, seq->number);
+	}
+
+	//sort values
+	quicksort_d(val, n);
+
+	int j;
+	double m, s, t;
+	do {
+		j = 0;
+		m = gsl_stats_median_from_sorted_data(val, 1, n);
+		s = gsl_stats_sd(val, 1, n);
+		t = m + k * s;
+		for (int i = n; i > 0; i--) {
+			if (val[i - 1] > t)
+				j++;
+			else
+				break;
+		}
+		n -= j;
+	} while (j > 0);
+
+	if (n < 0) {
+		free(val);
+		return 0.0;
+	}
+
+	threshold = factor * val[n - 1]; // we return a positive value
+	free(val);
+	return threshold;
 }
 
-double compute_highest_accepted_weighted_fwhm(sequence *seq, int layer, double percent) {
-	return generic_compute_accepted_value(seq, layer, percent, TRUE, regdata_weighted_fwhm);
+double compute_highest_accepted_fwhm(sequence *seq, int layer, double criterion, gboolean is_ksigma) {
+	if (!is_ksigma)
+		return generic_compute_accepted_value(seq, layer, criterion, TRUE, regdata_fwhm);
+	else
+		return generic_compute_accepted_value_with_rejection(seq, layer, criterion, TRUE, regdata_fwhm);
 }
 
-double compute_lowest_accepted_quality(sequence *seq, int layer, double percent) {
-	return generic_compute_accepted_value(seq, layer, percent, FALSE, regdata_quality);
+double compute_highest_accepted_weighted_fwhm(sequence *seq, int layer, double criterion, gboolean is_ksigma) {
+	if (!is_ksigma)
+		return generic_compute_accepted_value(seq, layer, criterion, TRUE, regdata_weighted_fwhm);
+	else
+		return generic_compute_accepted_value_with_rejection(seq, layer, criterion, TRUE, regdata_weighted_fwhm);
 }
 
-double compute_lowest_accepted_roundness(sequence *seq, int layer, double percent) {
-	return generic_compute_accepted_value(seq, layer, percent, FALSE, regdata_roundness);
+double compute_lowest_accepted_quality(sequence *seq, int layer, double criterion, gboolean is_ksigma) {
+	if (!is_ksigma)
+		return generic_compute_accepted_value(seq, layer, criterion, FALSE, regdata_quality);
+	else
+		return generic_compute_accepted_value_with_rejection(seq, layer, criterion, FALSE, regdata_quality);
 }
 
-char *describe_filter(sequence *seq, seq_image_filter filtering_criterion, double filtering_parameter) {
+double compute_lowest_accepted_roundness(sequence *seq, int layer, double criterion, gboolean is_ksigma) {
+	if (!is_ksigma)
+		return generic_compute_accepted_value(seq, layer, criterion, FALSE, regdata_roundness);
+	else
+		return generic_compute_accepted_value_with_rejection(seq, layer, criterion, FALSE, regdata_roundness);
+}
+
+double compute_highest_accepted_background(sequence *seq, int layer, double criterion, gboolean is_ksigma) {
+	if (!is_ksigma)
+		return generic_compute_accepted_value(seq, layer, criterion, TRUE, regdata_background);
+	else
+		return generic_compute_accepted_value_with_rejection(seq, layer, criterion, TRUE, regdata_background);
+}
+
+double compute_lowest_accepted_nbstars(sequence *seq, int layer, double criterion, gboolean is_ksigma) {
+	if (!is_ksigma)
+		return generic_compute_accepted_value(seq, layer, criterion, FALSE, regdata_nbstars);
+	else
+		return generic_compute_accepted_value_with_rejection(seq, layer, criterion, FALSE, regdata_nbstars);
+}
+
+gchar *describe_filter(sequence *seq, seq_image_filter filtering_criterion, double filtering_parameter) {
 	GString *str = g_string_sized_new(100);
 	int nb_images_to_stack = compute_nb_filtered_images(seq,
 			filtering_criterion, filtering_parameter);
@@ -412,6 +538,14 @@ char *describe_filter(sequence *seq, seq_image_filter filtering_criterion, doubl
 		g_string_printf(str, _("Processing images of the sequence "
 					"with a quality higher or equal than %g (%d)\n"),
 				filtering_parameter, nb_images_to_stack);
+	} else if (filtering_criterion == seq_filter_background) {
+		g_string_printf(str, _("Processing images of the sequence "
+					"with a background lower or equal than %g (%d)\n"),
+				filtering_parameter, nb_images_to_stack);
+	} else if (filtering_criterion == seq_filter_nbstars) {
+		g_string_printf(str, _("Processing images of the sequence "
+					"with a number of stars higher or equal than %g (%d)\n"),
+				filtering_parameter, nb_images_to_stack);
 	} else if (filtering_criterion == seq_filter_output_doesnt_already_exists) {
 		g_string_printf(str, _("Processing images whose output don't already exist (%d)"),
 				nb_images_to_stack);
@@ -424,7 +558,8 @@ char *describe_filter(sequence *seq, seq_image_filter filtering_criterion, doubl
 				descr[strlen(descr)-1] = '\0';	// remove the new line
 				if (f) descr[0] = tolower(descr[0]);
 			}
-			g_string_append(str, descr);
+			if (descr)
+				g_string_append(str, descr);
 			g_string_append(str, ", ");
 			g_free(descr);
 			f++;
@@ -432,6 +567,6 @@ char *describe_filter(sequence *seq, seq_image_filter filtering_criterion, doubl
 		g_string_append_printf(str, _("for a total of images processed of %d)\n"),
 				nb_images_to_stack);
 	}
-	//fprintf(stdout, "FILTERING DECRIPTION: %s", str->str);
+	//fprintf(stdout, "FILTERING DESCRIPTION: %s", str->str);
 	return g_string_free(str, FALSE);
 }
