@@ -8,15 +8,15 @@
 #include <glib/gstdio.h>
 #include <glib/gprintf.h>
 #include <gtk/gtk.h>
-#include <fitsio.h>	// fitsfile
 #include <gsl/gsl_histogram.h>
 #ifdef _OPENMP
 #include <omp.h>
 #endif
 #include <libintl.h>
 
+#include <fitsio.h>	// fitsfile
 
-#include "gui/preferences.h"
+#include "core/settings.h"
 #include "core/atomic.h"
 
 #define _(String) gettext (String)
@@ -29,6 +29,10 @@
 #define DEBUG_TEST 0
 #endif
 
+#if defined (HAVE_FFTW3F_OMP) || defined (HAVE_FFTW3F_THREADS)
+#define HAVE_FFTW3F_MULTITHREAD
+#endif
+
 #define GLADE_FILE "siril3.glade"
 
 /* https://stackoverflow.com/questions/1644868/define-macro-for-debug-printing-in-c */
@@ -51,11 +55,15 @@
        __typeof__ (b) _b = (b); \
      _a < _b ? _a : _b; })
 
-#define SWAP(a,b) { double temp = (a); (a) = (b); (b) = temp; }
+#define SWAPD(a,b) { double temp = (a); (a) = (b); (b) = temp; }
 
 #define SQR(x) ((x)*(x))
 #endif
-#define RADCONV ((3600.0 * 180.0) / M_PI) / 1.0E3
+#define RADCONV (((3600.0 * 180.0) / M_PI) / 1.0E3)
+
+// Used for sanity checking reported sizes from image files.
+// Not a guarantee that the file will fit in memory
+#define MAX_IMAGE_DIM 100000
 
 #define USHRT_MAX_DOUBLE ((double)USHRT_MAX)
 #define SHRT_MAX_DOUBLE ((double)SHRT_MAX)
@@ -75,7 +83,9 @@
 typedef unsigned char BYTE;	// default type for image display data
 typedef unsigned short WORD;	// default type for internal image data
 
-#define MAX_SEQPSF 7	// max number of stars for which seqpsf can be run
+#define MAX_REF_STARS 20	// uchar
+#define MAX_SEQPSF MAX_REF_STARS// max number of stars for which seqpsf can be run
+				// if we use seqpsf for the light curve, we must allow as many as MAX_REF_STARS
 
 #define CMD_HISTORY_SIZE 50	// size of the command line history
 
@@ -159,36 +169,27 @@ typedef enum {
 
 #define MAX_STARS 200000		// maximum length of com.stars
 #define MAX_STARS_FITTED 2000	// maximum number of stars fitted for registration
+#define MIN_STARS_FITTED 100	// minimum number of stars fitted for registration
+#define DEF_BOX_RADIUS 5 // default radius of the box in starfinder_conf
 
 #define INDEX_MAX 65535		// maximum index for images
 
-typedef struct imdata imgdata;
-typedef struct registration_data regdata;
 typedef struct sequ sequence;
-typedef struct single_image single;
-typedef struct wcs_struct wcs_info;
-typedef struct dft_struct dft_info;
 typedef struct ffit fits;
-typedef struct libraw_config libraw;
-typedef struct phot_config phot;
-typedef struct stack_config stackconf;
-typedef struct comp_config compconf;
 typedef struct guiinf guiinfo;
 typedef struct cominf cominfo;
-typedef struct image_stats imstats;
-typedef struct rectangle_struct rectangle;
-typedef struct point_struct point;
-typedef struct pointf_struct pointf;
-typedef struct pointi_struct pointi;
 typedef struct historic_struct historic;
 typedef struct fwhm_struct psf_star;
 typedef struct tilt_struct sensor_tilt;
-typedef struct star_finder_struct star_finder_params;
-typedef struct pref_struct preferences;
-typedef struct save_config_struct save_config;
 
 /* global structures */
 
+typedef enum {
+	BOOL_NOT_SET = -1,
+	BOOL_FALSE = 0,
+	BOOL_TRUE = 1,
+} super_bool;
+
 /* same order as in the combo box 'comboExport' */
 typedef enum {
 	EXPORT_FITS,
@@ -202,7 +203,7 @@ typedef enum {
 } export_format;
 
 typedef enum {
-	LINEAR_DISPLAY,	
+	LINEAR_DISPLAY,
 	LOG_DISPLAY,
 	SQRT_DISPLAY,
 	SQUARED_DISPLAY,
@@ -213,7 +214,7 @@ typedef enum {
 #define DISPLAY_MODE_MAX HISTEQ_DISPLAY
 
 typedef enum {
-	NORMAL_COLOR,	
+	NORMAL_COLOR,
 	RAINBOW_COLOR
 } color_map;
 
@@ -229,63 +230,81 @@ typedef enum {
 	OPEN_IMAGE_CANCEL = 10,
 } open_image_status;
 
-typedef enum {
-	BAYER_BILINEAR,
-	BAYER_VNG,
-	BAYER_AHD,
-	BAYER_AMAZE,
-	BAYER_DCB,
-	BAYER_HPHD,
-	BAYER_IGV,
-	BAYER_LMMSE,
-	BAYER_RCD,
-	BAYER_SUPER_PIXEL,
-	XTRANS
-} interpolation_method;
-
 typedef enum {
 	OPENCV_NEAREST = 0,
 	OPENCV_LINEAR = 1,
 	OPENCV_CUBIC = 2,
 	OPENCV_AREA = 3,
 	OPENCV_LANCZOS4 = 4,
-	OPENCV_INTER_MAX = 7
+	OPENCV_NONE = 5 // this one will use the pixel-wise shift transform w/o opencv
 } opencv_interpolation;
 
 typedef enum {
-	BAYER_FILTER_RGGB,
-	BAYER_FILTER_BGGR,
-	BAYER_FILTER_GBRG,
-	BAYER_FILTER_GRBG,
-	XTRANS_FILTER,
-	BAYER_FILTER_NONE = -1		//case where pattern is undefined or untested
-} sensor_pattern;
-#define BAYER_FILTER_MIN BAYER_FILTER_RGGB
-#define BAYER_FILTER_MAX BAYER_FILTER_GRBG
-
-typedef enum { SEQ_REGULAR, SEQ_SER, SEQ_FITSEQ,
+	SEQ_REGULAR, SEQ_SER, SEQ_FITSEQ,
 #ifdef HAVE_FFMS2
 	SEQ_AVI,
 #endif
 	SEQ_INTERNAL
 } sequence_type;
 
+typedef enum {
+	MULTI_THREADED = 0,	// equivalent to com.max_threads
+	SINGLE_THREADED = 1
+	/* and higher values are the number of threads */
+} threading_type;
+
+typedef enum {
+	RICE_COMP,
+	GZIP1_COMP,
+	GZIP2_COMP,
+	HCOMPRESS_COMP
+} compression_mode;
+
+typedef enum {
+	EXT_NONE,
+	EXT_STARNET,
+	EXT_ASNET,
+	EXT_GNUPLOT // not used for now
+} external_program;
+
 /* image data, exists once for each image */
-struct imdata {
+typedef struct {
 	int filenum;		/* real file index in the sequence, i.e. for mars9.fit = 9 */
 	gboolean incl;		/* selected in the sequence, included for future processings? */
-	GDateTime *date_obs;/* date of the observation, processed and copied from the header */
-};
+	GDateTime *date_obs;	/* date of the observation, processed and copied from the header */
+	int rx, ry;
+} imgdata;
+
+/* this structure is used to characterize the statistics of the image */
+typedef struct {
+	long total,	// number of pixels
+	     ngoodpix;	// number of non-zero pixels
+	double mean, median, sigma, avgDev, mad, sqrtbwmv,
+	       location, scale, min, max, normValue, bgnoise;
+
+	atomic_int* _nb_refs;	// reference counting for data management
+} imstats;
+
+typedef struct {
+	double h00, h01, h02;
+	double h10, h11, h12;
+	double h20, h21, h22;
+	int pair_matched;
+	int Inliers;
+} Homography;
 
 /* registration data, exists once for each image and each layer */
-struct registration_data {
-	float shiftx, shifty;	// we could have a subpixel precision, but is it needed? saved
+typedef struct {
 	psf_star *fwhm_data;	// used in PSF/FWHM registration, not saved
 	float fwhm;		// copy of fwhm->fwhmx, used as quality indicator, saved data
-	float weighted_fwhm; // used to exclude spurious images.
+	float weighted_fwhm;	// used to exclude spurious images.
 	float roundness;	// fwhm->fwhmy / fwhm->fwhmx, 0 when uninit, ]0, 1] when set
 	double quality;
-};
+	float background_lvl;
+	int number_of_stars;
+
+	Homography H;
+} regdata;
 
 /* see explanation about sequence and single image management in io/sequence.c */
 
@@ -295,8 +314,9 @@ struct sequ {
 	int selnum;		// number of selected images in the sequence
 	int fixed;		// fixed length of image index in filename (like %3d)
 	int nb_layers;		// number of layers embedded in each image file, -1 if unknown
-	unsigned int rx;	// first image width
-	unsigned int ry;	// first image height
+	unsigned int rx;	// first image width (or ref if set)
+	unsigned int ry;	// first image height (or ref if set)
+	gboolean is_variable;	// sequence has images of different sizes (imgparam->r[xy])
 	int bitpix;		// image pixel format, from fits
 	int reference_image;	// reference image for registration
 	imgdata *imgparam;	// a structure for each image of the sequence
@@ -311,6 +331,7 @@ struct sequ {
 	int beg;		// imgparam[0]->filenum
 	int end;		// imgparam[number-1]->filenum
 	double exposure;	// exposure of frames (we assume they are all identical)
+	gboolean fz;
 
 	sequence_type type;
 	struct ser_struct *ser_file;
@@ -318,7 +339,7 @@ struct sequ {
 	struct fits_sequence *fitseq_file; // FITS sequence data structure
 #ifdef HAVE_FFMS2
 	struct film_struct *film_file;
-	char *ext;		// extension of video, NULL if not video
+	const char *ext;	// extension of video, NULL if not video
 #endif
 	fits **internal_fits;	// for INTERNAL sequences: images references. Length: number
 	fitsfile **fptr;	// file descriptors for open-mode operations
@@ -333,25 +354,26 @@ struct sequ {
 	int previewW[PREVIEW_NB], previewH[PREVIEW_NB];	// 0 is uninitialized value
 
 	double upscale_at_stacking;// up-scale factor during stacking (see #215)
-	
-	gboolean needs_saving;	// a dirty flag for the sequence, avoid saving it too often
 
-	psf_star **photometry[MAX_SEQPSF];// psf for multiple stars for all images
+	gboolean needs_saving;	// a dirty flag for the sequence, avoid saving it too often
+	gboolean reg_invalidated; // a flag to detect if regframe can be plotted
+
+	psf_star **photometry[MAX_SEQPSF+1];// psf for multiple stars for all images
 	int reference_star;	// reference star for apparent magnitude (index of photometry)
 	double reference_mag;	// reference magnitude for the reference star
 	double photometry_colors[MAX_SEQPSF][3]; // colors for each photometry curve
 };
 
 /* this struct is used to manage data associated with a single image loaded, outside a sequence */
-struct single_image {
+typedef struct {
 	char *filename;		// the name of the file
 	gboolean fileexist;// flag of existing file
 	char *comment;		// comment on how the file got there (user load, result...)
 	int nb_layers;		// number of layers embedded in each image file
 	fits *fit;		// the fits is still gfit, but a reference doesn't hurt
-};
+} single;
 
-struct wcs_struct {
+typedef struct {
 	double equinox;
 	double crpix[2];
 	double crval[2];
@@ -363,13 +385,13 @@ struct wcs_struct {
 	double dec;
 	gboolean pltsolvd;
 	char pltsolvd_comment[FLEN_COMMENT];
-};
+} wcs_info;
 
-struct dft_struct {
+typedef struct {
 	double norm[3];			// Normalization value
 	char type[FLEN_VALUE];		// spectrum, phase
 	char ord[FLEN_VALUE];		// regular, centered
-};
+} dft_info;
 
 typedef enum { DATA_USHORT, DATA_FLOAT, DATA_UNSUPPORTED } data_type;
 
@@ -380,7 +402,7 @@ struct ffit {
 	int orig_bitpix;	// original bitpix of the file
 	/* bitpix can take the following values:
 	 * BYTE_IMG	(8-bit byte pixels, 0 - 255)
-	 * SHORT_IMG	(16 bit signed integer pixels)	
+	 * SHORT_IMG	(16 bit signed integer pixels)
 	 * USHORT_IMG	(16 bit unsigned integer pixels)	(used by Siril, a bit unusual)
 	 * LONG_IMG	(32-bit integer pixels)
 	 * FLOAT_IMG	(32-bit floating point pixels)
@@ -399,23 +421,27 @@ struct ffit {
 	WORD lo;		// MIPS-LO key in FITS file, "Lower visualization cutoff"
 	WORD hi;		// MIPS-HI key in FITS file, "Upper visualization cutoff"
 	double data_max;	// used to check if 32b float is in the [0, 1] range
+	double data_min;	// used to check if 32b float is in the [0, 1] range
 	float pixel_size_x, pixel_size_y;	// XPIXSZ and YPIXSZ keys
 	unsigned int binning_x, binning_y;	// XBINNING and YBINNING keys
-	gboolean unbinned;
 	char row_order[FLEN_VALUE];
-	GDateTime *date, *date_obs;
-	double expstart, expend;
+	GDateTime *date, *date_obs;		// creation and acquisition UTC dates
+	double expstart, expend;		// Julian dates
 	char filter[FLEN_VALUE];		// FILTER key
 	char image_type[FLEN_VALUE];		// IMAGETYP key
 	char object[FLEN_VALUE];		// OBJECT key
 	char instrume[FLEN_VALUE];		// INSTRUME key
 	char telescop[FLEN_VALUE];		// TELESCOP key
 	char observer[FLEN_VALUE];		// OBSERVER key
+	double sitelat;				// SITE LATITUDE key
+	double sitelong;			// SITE LONGITUDE key
+	double siteelev;			// SITE LONGITUDE key
 	char bayer_pattern[FLEN_VALUE];		// BAYERPAT key Bayer Pattern if available
 	int bayer_xoffset, bayer_yoffset;
+	double airmass;                   // relative optical path length through atmosphere.
 	/* data obtained from FITS or RAW files */
-	double focal_length, iso_speed, exposure, aperture, ccd_temp;
-	double livetime;		// total exposure
+	double focal_length, iso_speed, exposure, aperture, ccd_temp, set_temp;
+	double livetime;		// sum of exposures (s)
 	guint stackcnt;			// number of stacked frame
 	double cvf;			// Conversion factor (e-/adu)
 	int key_gain, key_offset;	// Gain, Offset values read in camera headers.
@@ -428,10 +454,11 @@ struct ffit {
 
 	/* data used in the Fourier space */
 	dft_info dft;
-	
+
 	/* data computed or set by Siril */
 	imstats **stats;	// stats of fit for each layer, null if naxes[2] is unknown
 	double mini, maxi;	// min and max of the stats->max[3]
+	float neg_ratio;	// ratio of pixels with a negative value on total number of pixels
 
 	fitsfile *fptr;		// file descriptor. Only used for file read and write.
 
@@ -446,71 +473,17 @@ struct ffit {
 	GSList *history;	// Former HISTORY comments of FITS file
 };
 
-/* This structure is used for all the elements in the box libraw_settings.
- * Don't forget to update conversion.c:initialize_libraw_settings() data when
- * modifying the glade settings */
-struct libraw_config {
-	double mul[3], bright;					// Color & brightness adjustement mul[0] = red, mul[1] = green = 1, mul[2] = blue
-	int auto_mul, use_camera_wb, use_auto_wb;		// White Balance parameters
-	int user_qual;						// Index of the Matrix interpolation set in dcraw, 0: bilinear, 1: VNG, 2: PPG, 3: AHD
-	int user_black;						// black point correction
-	double gamm[3];						// Gamma correction
-};
-
-/* This structure is used for storing all parameters used in photometry module */
-struct phot_config {
-	double gain;	// A/D converter gain in electrons per ADU
-	double inner;	// Inner radius of the annulus used to measure local background.
-	double outer;	// Outer radius of the annulus used to measure local background.
-	double aperture; // flux aperture
-	gboolean force_radius; // force the aperture radius value
-	int minval, maxval;
-};
-
-struct debayer_config {
-	gboolean open_debayer;			// debayer images being opened
-	gboolean use_bayer_header;		// use the pattern given in the file header
-	sensor_pattern bayer_pattern;		// user-defined Bayer pattern
-	interpolation_method bayer_inter;	// interpolation method for non-libraw debayer
-	gboolean top_down;				// debayer top-down orientation
-	int xbayeroff, ybayeroff;			// x and y Bayer offsets
-};
-
-struct stack_config {
-	int method;				// 0=sum, 1=median, 2=average, 3=pixel max, 4=pixel min - Use to save preferences in the init file
-	int normalisation_method;
-	int rej_method;
-	double sigma_low, sigma_high;
-	double linear_low, linear_high;
-	double percentile_low, percentile_high;
-	double esdt_outliers, esdt_significance;
-	enum { RATIO, AMOUNT } mem_mode; // mode of memory management
-	double memory_ratio;			// ratio of available memory to use for stacking (and others)
-	double memory_amount;			// amount of memory in GB to use for stacking (and others)
-};
-
-struct comp_config {
-	gboolean fits_enabled;		// true enabled
-	int fits_method;		// 0=Rice, 1=GZIP1, 2=GZIP2, 3=Hcompress
-	double fits_quantization;	// quantization factor for floating point compression
-	double fits_hcompress_scale;		// scale factor for Hcompress compression
-};
-
-struct rectangle_struct {
-	int x, y, w, h;
-};
-
-struct point_struct {
+typedef struct {
 	double x, y;
-};
+} point;
 
-struct pointf_struct {
+typedef struct {
 	float x, y;
-};
+} pointf;
 
-struct pointi_struct {
+typedef struct {
 	int x, y;
-};
+} pointi;
 
 struct historic_struct {
 	char *filename;
@@ -521,89 +494,29 @@ struct historic_struct {
 	double focal_length;
 };
 
-struct star_finder_struct {
-	int radius;
-	int adj_radius;
-	gboolean adjust;
-	double sigma;
-	double roundness;
-	double focal_length, pixel_size_x;
-};
+typedef struct _GNUPLOT_CTRL_ {
+    /** Pipe to gnuplot process */
+    FILE* gnucmd ;
+	FILE* gnumon ;
 
-struct save_config_struct {
-	gboolean quit;
-	gboolean warn_script;
-};
+    /** Number of currently active plots */
+    int nplots ;
+	/** Current plot window **/
+	gboolean replot; // Add additional plots to the current one in the current window
+    /** Current plotting style */
+    char pstyle[32] ;
 
-/**
- * This is the preference structure.
- * WARNING!!
- * If you update something in this structure you absolutely need to
- * update pref_init in preferences.c
- */
-struct pref_struct {
-	gboolean first_start; // use to display information at first use
-	/* state of window */
-	gboolean remember_windows;
-	rectangle main_w_pos;
-	gint pan_position;
-	gboolean is_extended;
-	gboolean is_maximized;
-
-	gboolean prepro_cfa;	// Use to save type of sensor for cosmetic correction in preprocessing
-	gboolean prepro_equalize_cfa; // Use to save if flat will be equalized in preprocessing
-	gboolean fix_xtrans;	// Use to fix xtrans darks and biases with the AF square
-	rectangle xtrans_af;	// if no xtrans model found, use these values
-	rectangle xtrans_sample; // if no xtrans model found, use these values
-	gchar *prepro_bias_lib;
-	gboolean use_bias_lib;
-	gchar *prepro_bias_synth;
-	gboolean use_bias_synth;
-	gchar *prepro_dark_lib;
-	gboolean use_dark_lib;
-	gchar *prepro_flat_lib;
-	gboolean use_flat_lib;
-
-	save_config save;
-	gboolean show_thumbnails; // show or don't show thumbnails in open dialog box
-	gint thumbnail_size;
-	gboolean check_update;	// check update at startup
-	gboolean script_check_requires; // check the requires command in scripts
-
-	gint combo_theme;	// value of the combobox theme
-	gdouble font_scale;	// font scale
-	gboolean icon_symbolic;	// icon style
-	gchar *combo_lang;	// string value of the combobox lang
-
-	gchar *ext;		// FITS extension used in SIRIL
-
-	gchar *swap_dir;	// swap directory
-	GSList *script_path;	// script path directories
-
-	gdouble focal;		// focal length saved in config file
-	gdouble pitch;		// pixel pitch saved in config file
-
-	libraw raw_set;		// the libraw settings
-	struct debayer_config debayer;	// debayer settings
-	phot phot_set;		// photometry settings
-	gboolean catalog[7];	// Yet 6 catalogs and 1 user catalog
-	int position_compass;	// compass position
-	int wcs_formalism;	// formalism used in FITS header
-
-	stackconf stack;	// stacking option
-	compconf comp;		// compression option
-	gboolean rgb_aladin;	// Add CTYPE3='RGB' in the FITS header
-
-	gboolean force_to_16bit;
-
-	gint selection_guides;	// number of elements of the grid guides (2 for a simple cross, 3 for the 3 thirds rule, etc.)
-
-	gchar *copyright;	// User copyright when saving image as TIFF
-
-};
-/**
- * End of preference structure. Read above if modified.
- */
+    /** Pointer to table of names of temporary files */
+    char** tmp_filename_tbl ;
+    /** Number of temporary files */
+    int ntmp ;
+	GThread* thread;
+	guint source; // Reference for the callback
+	int child_fd_stdin;
+	int child_fd_stderr;
+	GPid child_pid;
+	gboolean running;
+} gnuplot_ctrl ;
 
 /* The rendering of the main image is cached. As it can be much larger than the
  * widget in which it's displayed, it can take a lot of time to transform it
@@ -626,6 +539,15 @@ struct image_view {
 	cairo_surface_t *disp_surface;	// the cache
 };
 
+typedef struct draw_data {
+	cairo_t *cr;	// the context to draw to
+	int vport;	// the viewport index to draw
+	double zoom;	// the current zoom value
+	gboolean neg_view;	// negative view
+	cairo_filter_t filter;	// the type of image filtering to use
+	guint image_width, image_height;	// image size
+	guint window_width, window_height;	// drawing area size
+} draw_data_t;
 
 /* The global data structure of siril gui */
 struct guiinf {
@@ -642,29 +564,42 @@ struct guiinf {
 
 	gboolean translating;		// the image is being translated
 
+	gboolean show_excluded;		// show excluded images in sequences
+
+	int selected_star;		// current selected star in the GtkListStore
+
+	gboolean show_wcs_grid;		// show the equatorial grid over the image
+
+	psf_star *qphot;		// quick photometry result, highlight a star
+
+	void (*draw_extra)(draw_data_t *dd);
+
 	/*********** Color mapping **********/
 	WORD lo, hi;			// the values of the cutoff sliders
 	gboolean cut_over;		// display values over hi as negative
 	sliders_mode sliders;		// lo/hi, minmax, user
-	display_mode rendering_mode;	// pixel value scaling, defaults to LINEAR_DISPLAY
+	display_mode rendering_mode;	// pixel value scaling, defaults to LINEAR_DISPLAY or default_rendering_mode if set in preferences
+	gboolean unlink_channels;	// only for autostretch
+	BYTE remap_index[3][USHRT_MAX];	// abstracted here so it can be used for previews and is easier to change the bit depth
+	BYTE *hd_remap_index[3];	// HD remap indexes for the high precision LUTs.
+	guint hd_remap_max;		// the maximum index value to use for the HD LUT. Default is 2^22
+	gboolean use_hd_remap;		// use high definition LUT for auto-stretch
 
-	gboolean show_excluded;		// show excluded images in sequences
-
-	/* selection rectangle for registration, FWHM, PSF */
+	/* selection rectangle for registration, FWHM, PSF, coords in com.selection */
 	gboolean drawing;		// true if the rectangle is being set (clicked motion)
 	pointi start;			// where the mouse was originally clicked to
 	pointi origin;			// where the selection was originally located
-	gboolean freezeX, freezeY;	// locked axis during modification of a selection 
+	gboolean freezeX, freezeY;	// locked axis during modification of a selection
 	double ratio;			// enforced ratio of the selection (default is 0: none)
+	double rotation;		// selection rotation for dynamic crop
 
 	/* alignment preview data */
-	//guchar *preview_buf[PREVIEW_NB];
 	cairo_surface_t *preview_surface[PREVIEW_NB];
 	GtkWidget *preview_area[PREVIEW_NB];
 	guchar *refimage_regbuffer;	// the graybuf[registration_layer] of the reference image
 	cairo_surface_t *refimage_surface;
-	
-	int filter;			// file extension filter for open/save dialogs
+
+	int file_ext_filter;		// file extension filter for open/save dialogs
 
 	/* history of the command line. This is a circular buffer (cmd_history)
 	 * of size cmd_hist_size, position to be written is cmd_hist_current and
@@ -674,75 +609,68 @@ struct guiinf {
 	int cmd_hist_size;		// allocated size
 	int cmd_hist_current;		// current command index
 	int cmd_hist_display;		// displayed command index
-
 };
 
 /* The global data structure of siril core */
 struct cominf {
-	preferences pref;		// saved variable in preferences
-	gchar *wd;			// working directory, where images and sequences are
-	gchar *initfile;		// the path of the init file
+	gchar *wd;			// current working directory, where images and sequences are
 
-	int reg_settings;		// Use to save registration method in the init file
+	preferences pref;		// some variables are stored in settings
+	gchar *initfile;		// the path of the init file (stores settings)
 
-	gboolean cache_upscaled;	// keep up-scaled files for 'drizzle' (only used by developers)
+	sequence seq;			// currently loaded sequence
+	single *uniq;			// currently loaded image, if outside sequence
 
-	/* history of operations */
-	historic *history;		// the history of all operations
+	gsl_histogram *layers_hist[MAXVPORT]; // current image's histograms
+	gsl_histogram *sat_hist;
+					      // TODO: move in ffit?
+
+	gboolean headless;		// pure console, no GUI
+	gboolean script;		// script being executed, always TRUE when headless is
+	GThread *thread;		// the thread for processing
+	GMutex mutex;			// a mutex we use for this thread
+	gboolean run_thread;		// the main thread loop condition
+	gboolean stop_script;		// abort script execution, not just a command
+	GThread *script_thread;		// reads a script and executes its commands
+	gboolean script_thread_exited;	// boolean set by the script thread when it exits
+
+	int max_images;			// max number of image threads used for parallel execution
+	int max_thread;			// max total number of threads used for parallel execution
+
+	rectangle selection;		// coordinates of the selection rectangle
+
+	psf_star **stars;		// list of stars detected in the current image
+	gboolean star_is_seqdata;	// the only star in stars belongs to seq, don't free it
+	double magOffset;		// offset to reduce the real magnitude, single image
+
+	/* history of operations, for the FITS header and the undo feature */
+	historic *history;		// the history of all operations on the current image
 	int hist_size;			// allocated size
 	int hist_current;		// current index
 	int hist_display;		// displayed index
 
-	sequence seq;			// currently loaded sequence	TODO: *seq
-	single *uniq;			// currently loaded image, if outside sequence
+	/* all fields below are used by some specific features as a temporary storage */
+	GSList *grad_samples;		// list of samples for the background extraction
 
-	gsl_histogram *layers_hist[MAXVPORT]; // current image's histograms
+	GSList *found_object;		// list of objects found in the image from catalogues
 
-	star_finder_params starfinder_conf;	// star finder settings, from GUI or init file
-	psf_star **stars;		// list of stars detected in the current image
-	gboolean star_is_seqdata;	// the only star in stars belongs to seq, don't free it
-	int selected_star;		// current selected star in the GtkListStore
-	double magOffset;		// offset to reduce the real magnitude, single image
-	
-	GSList *grad_samples;
-	GSList *found_object;
-	gboolean show_wcs_grid;
-	psf_star *qphot;		// quick photometry result
-	sensor_tilt *tilt;		// tilt information
+	sensor_tilt *tilt;		// computed tilt information
 
-	int max_thread;			// maximum of thread used for parallel execution
+	external_program child_is_running;	// external_program id to check if there is a child process running
 
-	GThread *thread;		// the thread for processing
-	GMutex mutex;			// a mutex we use for this thread
-	gboolean run_thread;		// the main thread loop condition
+	float* kernel;			// float* to hold kernel for new deconvolution process
+	unsigned kernelsize;		// Holds size of kernel (kernel is square kernelsize * kernelsize)
+	unsigned kernelchannels;	// Holds number of channels for the kernel
+#ifdef _WIN32
+	void* childhandle;		// For Windows, handle of a child process
+#else
+	pid_t childpid;			// For other OSes, PID of a child process
+#endif
+	gnuplot_ctrl **gnuplot_handles; // list of gnuplot handles
+	int num_gnuplot_handles; // how many gnuplot handles are in the list
 
-	gboolean headless;		// pure console, no GUI
-	gboolean script;		// scripts execution
-	gboolean stop_script;		// abort script execution
-	GThread *script_thread;		// reads a script and executes its commands
-	gboolean script_thread_exited;	// boolean set by the script thread when it exits
-
-	rectangle selection;		// coordinates of the selection rectangle
 };
 
-/* this structure is used to characterize the statistics of the image */
-struct image_stats {
-	long total,	// number of pixels
-	     ngoodpix;	// number of non-zero pixels
-	double mean, median, sigma, avgDev, mad, sqrtbwmv,
-	       location, scale, min, max, normValue, bgnoise;
-
-	atomic_int* _nb_refs;	// reference counting for data management
-};
-
-typedef struct Homo {
-	double h00, h01, h02;
-	double h10, h11, h12;
-	double h20, h21, h22;
-	int pair_matched;
-	int Inliers;
-} Homography;
-
 #ifndef MAIN
 extern guiinfo gui;
 extern cominfo com;		// the main data struct
