@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -110,12 +110,12 @@ static void search_for_startup_dir() {
 
 	size = sizeof(sdir) / sizeof(GUserDirectory);
 
-	while (dir == NULL && i < size) {
+	while (!dir && i < size) {
 		dir = g_get_user_special_dir(sdir[i]);
 		i++;
 	}
 	/* Not every platform has a directory for these logical id */
-	if (dir == NULL) {
+	if (!dir) {
 		dir = g_get_home_dir();
 	}
 	if (dir)
