@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -23,7 +23,9 @@
 #include "core/siril.h"
 #include "core/proto.h"
 #include "algos/statistics.h"
+#include "core/arithm.h"
 #include "io/single_image.h"
+#include "gui/callbacks.h"
 #include "gui/image_display.h"
 #include "gui/utils.h"
 #include "gui/progress_and_log.h"
@@ -34,7 +36,7 @@
 #include "asinh.h"
 
 static gboolean asinh_rgb_space = FALSE;
-static double asinh_stretch_value = 0.0, asinh_black_value = 0.0;
+static float asinh_stretch_value = 0.0f, asinh_black_value = 0.0f;
 static gboolean asinh_show_preview;
 
 static void asinh_startup() {
@@ -56,20 +58,21 @@ static void asinh_close(gboolean revert) {
 }
 
 static int asinh_update_preview() {
-	copy_backup_to_gfit();
+	if (asinh_show_preview)
+		copy_backup_to_gfit();
 	asinhlut(&gfit, asinh_stretch_value, asinh_black_value, asinh_rgb_space);
 	return 0;
 }
 
-int asinhlut_ushort(fits *fit, double beta, double offset, gboolean human_luminance) {
+int asinhlut_ushort(fits *fit, float beta, float offset, gboolean human_luminance) {
 	WORD *buf[3] = { fit->pdata[RLAYER], fit->pdata[GLAYER], fit->pdata[BLAYER] };
-
-	double norm = get_normalized_value(fit);
-	double invnorm = 1.0 / norm;
-	double asinh_beta = asinh(beta);
-	double factor_red = human_luminance ? 0.2126 : 0.3333;
-	double factor_green = human_luminance ? 0.7152 : 0.3333;
-	double factor_blue = human_luminance ? 0.0722 : 0.3333;
+	float m_CB = 1.f;
+	float norm = get_normalized_value(fit);
+	float invnorm = 1.0f / norm;
+	float asinh_beta = asinh(beta);
+	float factor_red = human_luminance ? 0.2126f : 0.3333f;
+	float factor_green = human_luminance ? 0.7152f : 0.3333f;
+	float factor_blue = human_luminance ? 0.0722f : 0.3333f;
 
 	size_t i, n = fit->naxes[0] * fit->naxes[1];
 	if (fit->naxes[2] > 1) {
@@ -77,82 +80,102 @@ int asinhlut_ushort(fits *fit, double beta, double offset, gboolean human_lumina
 #pragma omp parallel for num_threads(com.max_thread) schedule(dynamic, fit->rx * 16)
 #endif
 		for (i = 0; i < n; i++) {
-			double r = buf[RLAYER][i] * invnorm;
-			double g = buf[GLAYER][i] * invnorm;
-			double b = buf[BLAYER][i] * invnorm;
-			double rprime = max(0, (r - offset) / (1.0 - offset));
-			double gprime = max(0, (g - offset) / (1.0 - offset));
-			double bprime = max(0, (b - offset) / (1.0 - offset));
+			blend_data data;
+#pragma omp simd
+			for (int i = 0 ; i < 3 ; i++)
+				data.do_channel[i] = TRUE;
+			float r = buf[RLAYER][i] * invnorm;
+			float g = buf[GLAYER][i] * invnorm;
+			float b = buf[BLAYER][i] * invnorm;
+			float rout, gout, bout;
+			float rprime = max(0, (r - offset) / (1.0f - offset));
+			float gprime = max(0, (g - offset) / (1.0f - offset));
+			float bprime = max(0, (b - offset) / (1.0f - offset));
 
-			double x = factor_red * rprime + factor_green * gprime + factor_blue * bprime;
+			float x = factor_red * rprime + factor_green * gprime + factor_blue * bprime;
+			data.tf[RLAYER] = (rprime == 0.f) ? 0.f : (beta == 0.f) ? 1.f : asinhf(beta * rprime) / (rprime * asinh_beta);
+			data.tf[GLAYER] = (gprime == 0.f) ? 0.f : (beta == 0.f) ? 1.f : asinhf(beta * gprime) / (gprime * asinh_beta);
+			data.tf[BLAYER] = (bprime == 0.f) ? 0.f : (beta == 0.f) ? 1.f : asinhf(beta * bprime) / (bprime * asinh_beta);
+			float k = (x == 0.0f) ? 0.0f : (beta == 0.0f) ? 1.0f : asinhf(beta * x) / (x * asinh_beta);
+			data.sf[RLAYER] = min(1.0f, max(0.0f, (rprime * k)));
+			data.sf[GLAYER] = min(1.0f, max(0.0f, (gprime * k)));
+			data.sf[BLAYER] = min(1.0f, max(0.0f, (bprime * k)));
 
-			double k = (x == 0.0) ? 0.0 : (beta == 0.0) ? 1.0 : asinh(beta * x) / (x * asinh_beta);
+			rgbblend(&data, &rout, &gout, &bout, m_CB);
 
-			buf[RLAYER][i] = round_to_WORD(norm * min(1.0, max(0.0,(rprime) * k)));
-			buf[GLAYER][i] = round_to_WORD(norm * min(1.0, max(0.0,(gprime) * k)));
-			buf[BLAYER][i] = round_to_WORD(norm * min(1.0, max(0.0,(bprime) * k)));
+			buf[RLAYER][i] = roundf_to_WORD(norm * min(1.0f, max(0.0f, rout)));
+			buf[GLAYER][i] = roundf_to_WORD(norm * min(1.0f, max(0.0f, gout)));
+			buf[BLAYER][i] = roundf_to_WORD(norm * min(1.0f, max(0.0f, bout)));
 		}
 	} else {
 #ifdef _OPENMP
 #pragma omp parallel for num_threads(com.max_thread) schedule(dynamic, fit->rx * 16)
 #endif
 		for (i = 0; i < n; i++) {
-			double x = buf[RLAYER][i] * invnorm;
-			double xprime = max(0, (x - offset) / (1.0 - offset));
-			double k = (xprime == 0.0) ? 0.0 : (beta == 0.0) ? 1.0 : asinh(beta * xprime) / (xprime * asinh_beta);
-			buf[RLAYER][i] = round_to_WORD(norm * min(1.0, max(0.0,(xprime) * k)));
+			float x = buf[RLAYER][i] * invnorm;
+			float xprime = max(0, (x - offset) / (1.0f - offset));
+			float k = (xprime == 0.0f) ? 0.0f : (beta == 0.0f) ? 1.0f : asinhf(beta * xprime) / (xprime * asinh_beta);
+			buf[RLAYER][i] = roundf_to_WORD(norm * min(1.0f, max(0.0f,(xprime) * k)));
 		}
 	}
 	invalidate_stats_from_fit(fit);
 	return 0;
 }
 
-static int asinhlut_float(fits *fit, double beta, double offset, gboolean human_luminance) {
+static int asinhlut_float(fits *fit, float beta, float offset, gboolean human_luminance) {
 	float *buf[3] = { fit->fpdata[RLAYER], fit->fpdata[GLAYER], fit->fpdata[BLAYER] };
 
-	double asinh_beta = asinh(beta);
-	double factor_red = human_luminance ? 0.2126 : 0.3333;
-	double factor_green = human_luminance ? 0.7152 : 0.3333;
-	double factor_blue = human_luminance ? 0.0722 : 0.3333;
+	float m_CB = 1.f;
+	float asinh_beta = asinhf(beta);
+	float factor_red = human_luminance ? 0.2126f : 0.3333f;
+	float factor_green = human_luminance ? 0.7152f : 0.3333f;
+	float factor_blue = human_luminance ? 0.0722f : 0.3333f;
 
 	size_t i, n = fit->naxes[0] * fit->naxes[1];
 	if (fit->naxes[2] > 1) {
 #ifdef _OPENMP
-#pragma omp parallel for num_threads(com.max_thread) schedule(dynamic, fit->ry * 16)
+#pragma omp parallel for num_threads(com.max_thread) schedule(static)
 #endif
 		for (i = 0; i < n; i++) {
-			double r = buf[RLAYER][i];
-			double g = buf[GLAYER][i];
-			double b = buf[BLAYER][i];
-			double rprime = max(0, (r - offset) / (1.0 - offset));
-			double gprime = max(0, (g - offset) / (1.0 - offset));
-			double bprime = max(0, (b - offset) / (1.0 - offset));
+			blend_data data;
+#pragma omp simd
+			for (int i = 0 ; i < 3 ; i++)
+				data.do_channel[i] = TRUE;
+			float r = buf[RLAYER][i];
+			float g = buf[GLAYER][i];
+			float b = buf[BLAYER][i];
+			float rprime = max(0.0f, (r - offset) / (1.0f - offset));
+			float gprime = max(0.0f, (g - offset) / (1.0f - offset));
+			float bprime = max(0.0f, (b - offset) / (1.0f - offset));
 
-			double x = factor_red * rprime + factor_green * gprime + factor_blue * bprime;
+			float x = factor_red * rprime + factor_green * gprime + factor_blue * bprime;
+			data.tf[RLAYER] = (rprime == 0.f) ? 0.f : (beta == 0.f) ? 1.f : asinhf(beta * rprime) / (rprime * asinh_beta);
+			data.tf[GLAYER] = (gprime == 0.f) ? 0.f : (beta == 0.f) ? 1.f : asinhf(beta * gprime) / (gprime * asinh_beta);
+			data.tf[BLAYER] = (bprime == 0.f) ? 0.f : (beta == 0.f) ? 1.f : asinhf(beta * bprime) / (bprime * asinh_beta);
+			float k = (x == 0.0f) ? 0.0f : (beta == 0.0f) ? 1.0f : asinhf(beta * x) / (x * asinh_beta);
+			data.sf[RLAYER] = min(1.0f, max(0.0f, (rprime * k)));
+			data.sf[GLAYER] = min(1.0f, max(0.0f, (gprime * k)));
+			data.sf[BLAYER] = min(1.0f, max(0.0f, (bprime * k)));
 
-			double k = (x == 0.0) ? 0.0 : (beta == 0.0) ? 1.0 : asinh(beta * x) / (x * asinh_beta);
-
-			buf[RLAYER][i] = min(1.0, max(0.0, (rprime * k)));
-			buf[GLAYER][i] = min(1.0, max(0.0, (gprime * k)));
-			buf[BLAYER][i] = min(1.0, max(0.0, (bprime * k)));
+			rgbblend(&data, &buf[RLAYER][i], &buf[GLAYER][i], &buf[BLAYER][i], m_CB);
 		}
 	} else {
 #ifdef _OPENMP
-#pragma omp parallel for num_threads(com.max_thread) schedule(dynamic, fit->ry * 16)
+#pragma omp parallel for num_threads(com.max_thread) schedule(static)
 #endif
 		for (i = 0; i < n; i++) {
-			double x, k;
+			float x, k;
 			x = buf[RLAYER][i];
-			double xprime = max(0, (x - offset) / (1.0 - offset));
-			k = (xprime == 0.0) ? 0.0 : (beta == 0.0) ? 1.0 : asinh(beta * xprime) / (xprime * asinh_beta);
-			buf[RLAYER][i] = min(1.0, max(0.0,(xprime * k)));
+			float xprime = max(0.0f, (x - offset) / (1.0f - offset));
+			k = (xprime == 0.0f) ? 0.0f : (beta == 0.0f) ? 1.0f : asinhf(beta * xprime) / (xprime * asinh_beta);
+			buf[RLAYER][i] = min(1.0f, max(0.0f,(xprime * k)));
 		}
 	}
 	invalidate_stats_from_fit(fit);
 	return 0;
 }
 
-int asinhlut(fits *fit, double beta, double offset, gboolean human_luminance) {
+int asinhlut(fits *fit, float beta, float offset, gboolean human_luminance) {
 	if (fit->type == DATA_USHORT)
 		return asinhlut_ushort(fit, beta, offset, human_luminance);
 	if (fit->type == DATA_FLOAT)
@@ -162,7 +185,7 @@ int asinhlut(fits *fit, double beta, double offset, gboolean human_luminance) {
 
 
 static void apply_asinh_changes() {
-	gboolean status = (asinh_stretch_value != 1.0) || (asinh_black_value != 0.0) || !asinh_rgb_space;
+	gboolean status = (asinh_stretch_value != 1.0f) || (asinh_black_value != 0.0f) || !asinh_rgb_space;
 	asinh_close(!status);
 }
 
@@ -178,9 +201,12 @@ void on_asinh_dialog_show(GtkWidget *widget, gpointer user_data) {
 	GtkSpinButton *spin_black_p = GTK_SPIN_BUTTON(lookup_widget("black_point_spin_asinh"));
 	GtkToggleButton *toggle_rgb = GTK_TOGGLE_BUTTON(lookup_widget("checkbutton_RGBspace"));
 
+	if (gui.rendering_mode == LINEAR_DISPLAY)
+		setup_stretch_sliders(); // In linear mode, set sliders to 0 / 65535
+
 	asinh_startup();
-	asinh_stretch_value = 0.0;
-	asinh_black_value = 0.0;
+	asinh_stretch_value = 0.0f;
+	asinh_black_value = 0.0f;
 	asinh_rgb_space = TRUE;
 
 	set_notify_block(TRUE);
@@ -280,4 +306,3 @@ void on_asinh_preview_toggled(GtkToggleButton *button, gpointer user_data) {
 	}
 	asinh_show_preview = !asinh_show_preview;
 }
-
