@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -28,6 +28,7 @@
 #include "core/undo.h"
 #include "core/OS_utils.h"
 #include "core/optimize_utils.h"
+#include "core/siril_log.h"
 #include "gui/image_display.h"
 #include "gui/utils.h"
 #include "gui/dialogs.h"
@@ -44,9 +45,11 @@
 #include "opencv/opencv.h"
 
 #include "cosmetic_correction.h"
+static int autoDetect(fits *fit, int layer, const double sig[2], long *icold, long *ihot,
+		double amount, gboolean is_cfa, threading_type threads);
 
 /* see also getMedian3x3 in algos/PSF.c */
-static float getMedian5x5_float(float *buf, const int xx, const int yy, const int w,
+static float getMedian5x5_float(const float *buf, const int xx, const int yy, const int w,
 		const int h, gboolean is_cfa) {
 
 	const int step = is_cfa ? 2 : 1;
@@ -72,6 +75,7 @@ static float getMedian5x5_float(float *buf, const int xx, const int yy, const in
 
 static WORD* getAverage3x3Line(WORD *buf, const int yy, const int w,
 		const int h, gboolean is_cfa) {
+	g_assert(w > 0 && h > 0);
 	int step, radius, x, xx, y;
 	WORD *cpyline;
 
@@ -94,13 +98,14 @@ static WORD* getAverage3x3Line(WORD *buf, const int yy, const int w,
 				}
 			}
 		}
-		cpyline[xx] = round_to_WORD(value / n);
+		cpyline[xx] = n == 0 ? 0 : round_to_WORD(value / n);
 	}
 	return cpyline;
 }
 
-static float* getAverage3x3Line_float(float *buf, const int yy, const int w,
+static float* getAverage3x3Line_float(const float *buf, const int yy, const int w,
 		const int h, gboolean is_cfa) {
+	g_assert(w > 0 && h > 0);
 	int step, radius, x, xx, y;
 	float *cpyline;
 
@@ -123,13 +128,14 @@ static float* getAverage3x3Line_float(float *buf, const int yy, const int w,
 				}
 			}
 		}
-		cpyline[xx] = (value / n);
+		cpyline[xx] = n == 0 ? 0.f : (value / n);
 	}
 	return cpyline;
 }
 
-static float getAverage3x3_float(float *buf, const int xx, const int yy,
+static float getAverage3x3_float(const float *buf, const int xx, const int yy,
 		const int w, const int h, gboolean is_cfa) {
+	g_assert(w > 0 && h > 0);
 
     const int step = is_cfa ? 2 : 1;
     const int radius = step;
@@ -146,11 +152,12 @@ static float getAverage3x3_float(float *buf, const int xx, const int yy,
             }
         }
     }
-    return value / n;
+    return n == 0 ? 0.f : value / n;
 }
 
 static float getAverage3x3_ushort(WORD *buf, const int xx, const int yy,
 		const int w, const int h, gboolean is_cfa) {
+	g_assert(w > 0 && h > 0);
 	int step, radius, x, y;
 	float value = 0.f;
 
@@ -172,7 +179,7 @@ static float getAverage3x3_ushort(WORD *buf, const int xx, const int yy,
 			}
 		}
 	}
-	return value / n;
+	return n == 0 ? 0 :value / n;
 }
 
 /* Gives a list of point p containing deviant pixel coordinates, to be freed by
@@ -181,7 +188,7 @@ static float getAverage3x3_ushort(WORD *buf, const int xx, const int yy,
  * returns NULL. It also returns NULL when no deviant pixel is found.
  * If cold == -1 or hot == -1, this is a flag to not compute cold or hot
  */
-deviant_pixel* find_deviant_pixels(fits *fit, double sig[2], long *icold,
+deviant_pixel* find_deviant_pixels(fits *fit, const double sig[2], long *icold,
 		long *ihot, gboolean eval_only) {
 	int x, y;
 	WORD *buf;
@@ -194,7 +201,7 @@ deviant_pixel* find_deviant_pixels(fits *fit, double sig[2], long *icold,
 	if (sig[0] == -1.0 && sig[1] == -1.0)
 		return NULL;
 
-	stat = statistics(NULL, -1, fit, RLAYER, NULL, STATS_BASIC, FALSE);
+	stat = statistics(NULL, -1, fit, RLAYER, NULL, STATS_BASIC, SINGLE_THREADED);
 	if (!stat) {
 		siril_log_message(_("Error: statistics computation failed.\n"));
 		return NULL;
@@ -233,7 +240,7 @@ deviant_pixel* find_deviant_pixels(fits *fit, double sig[2], long *icold,
 
 	if (eval_only) return NULL;
 
-	/** Second we store deviant pixels in p */
+	/** Second we store deviant pixels in dev */
 	int n = (*icold) + (*ihot);
 	if (n <= 0)
 		return NULL;
@@ -336,8 +343,14 @@ int cosmeticCorrection(fits *fit, deviant_pixel *dev, int size, gboolean is_cfa)
 }
 
 /**** Autodetect *****/
+static int cosmetic_finalize_hook(struct generic_seq_args *args) {
+	int retval = seq_finalize_hook(args);
+	free(args->user);
+	return retval;
+}
+
 int cosmetic_image_hook(struct generic_seq_args *args, int o, int i, fits *fit,
-		rectangle *_) {
+		rectangle *_, int threads) {
 	struct cosmetic_data *c_args = (struct cosmetic_data*) args->user;
 	int chan;
 	/* Count variables, icold and ihot, need to be local in order to be parallelized */
@@ -346,7 +359,7 @@ int cosmetic_image_hook(struct generic_seq_args *args, int o, int i, fits *fit,
 	icold = ihot = 0L;
 	for (chan = 0; chan < fit->naxes[2]; chan++) {
 		int retval = autoDetect(fit, chan, c_args->sigma, &icold, &ihot,
-				c_args->amount, c_args->is_cfa, c_args->multithread);
+				c_args->amount, c_args->is_cfa, c_args->threading);
 		if (retval)
 			return retval;
 	}
@@ -415,7 +428,7 @@ void apply_cosmetic_to_sequence(struct cosmetic_data *cosme_args) {
 	args->nb_filtered_images = cosme_args->seq->selnum;
 	args->compute_mem_limits_hook = cosmetic_mem_limits_hook;
 	args->prepare_hook = seq_prepare_hook;
-	args->finalize_hook = seq_finalize_hook;
+	args->finalize_hook = cosmetic_finalize_hook;
 	args->image_hook = cosmetic_image_hook;
 	args->stop_on_error = FALSE;
 	args->description = _("Cosmetic Correction");
@@ -453,7 +466,7 @@ gpointer autoDetectThreaded(gpointer p) {
 	icold = ihot = 0L;
 	for (chan = 0; chan < args->fit->naxes[2]; chan++) {
 		retval = autoDetect(args->fit, chan, args->sigma, &icold, &ihot,
-				args->amount, args->is_cfa, args->multithread);
+				args->amount, args->is_cfa, args->threading);
 		if (retval)
 			break;
 	}
@@ -484,8 +497,6 @@ int apply_cosme_to_image(fits *fit, GFile *file, int is_cfa) {
 			g_clear_error(&error);
 			siril_log_message(_("File [%s] does not exist\n"), g_file_peek_path(file));
 		}
-
-		g_object_unref(file);
 		return 1;
 	}
 
@@ -539,13 +550,17 @@ int apply_cosme_to_image(fits *fit, GFile *file, int is_cfa) {
 				g_free(line);
 				continue;
 			}
-			point center = { fit->rx / 2.0, fit->ry / 2.0 };
 			dev.type = HOT_PIXEL; // we force it
 			dev.p.y = fit->rx - dev.p.y - 1; /* FITS are stored bottom to top */
-			cvRotateImage(fit, center, 90.0, -1, OPENCV_AREA);
+			if (cvRotateImage(fit, 90)) {
+				retval = 1;
+				break;
+			}
 			cosmeticCorrOneLine(fit, dev, is_cfa);
-			cvRotateImage(fit, center, -90.0, -1, OPENCV_AREA);
-
+			if (cvRotateImage(fit, -90)) {
+				retval = 1;
+				break;
+			}
 			break;
 		default:
 			fprintf(stderr, _("cosmetic correction: "
@@ -554,17 +569,19 @@ int apply_cosme_to_image(fits *fit, GFile *file, int is_cfa) {
 		}
 		g_free(line);
 	}
-
+	g_object_unref(data_input);
 	g_object_unref(input_stream);
 
 	return retval;
 }
 
 int cosme_image_hook(struct generic_seq_args *args, int o, int i, fits *fit,
-		rectangle *_) {
+		rectangle *_, int threads) {
 	struct cosme_data *c_args = (struct cosme_data*) args->user;
 
-	return apply_cosme_to_image(fit, c_args->file, c_args->is_cfa);
+	int retval = apply_cosme_to_image(fit, c_args->file, c_args->is_cfa);
+	g_object_unref(c_args->file);
+	return retval;
 }
 
 static int cosme_finalize_hook(struct generic_seq_args *args) {
@@ -588,7 +605,7 @@ void apply_cosme_to_sequence(struct cosme_data *cosme_args) {
 	args->description = _("Cosmetic Correction");
 	args->has_output = TRUE;
 	args->output_type = get_data_type(args->seq->bitpix);
-	args->new_seq_prefix = cosme_args->prefix;
+	args->new_seq_prefix = strdup(cosme_args->prefix);
 	args->load_new_sequence = TRUE;
 	args->user = cosme_args;
 
@@ -599,18 +616,17 @@ void apply_cosme_to_sequence(struct cosme_data *cosme_args) {
 
 /* this is an autodetect algorithm. Cold and hot pixels
  *  are corrected in the same time */
-int autoDetect(fits *fit, int layer, double sig[2], long *icold, long *ihot,
-		double amount, gboolean is_cfa, gboolean multithread) {
+static int autoDetect(fits *fit, int layer, const double sig[2], long *icold, long *ihot,
+		double amount, gboolean is_cfa, threading_type threading) {
 
 	/* XXX: if cfa, stats are irrelevant. We should compute them taking
 	 * into account the Bayer pattern */
-	imstats *stat = statistics(NULL, -1, fit, layer, NULL, STATS_BASIC | STATS_AVGDEV,
-			multithread);
-
+	imstats *stat = statistics(NULL, -1, fit, layer, NULL, STATS_BASIC | STATS_AVGDEV, threading);
 	if (!stat) {
 		siril_log_message(_("Error: statistics computation failed.\n"));
 		return 1;
 	}
+	int threads = check_threading(&threading);
 	const float bkg = stat->median;
 	const float avgDev = stat->avgDev;
 	free_stats(stat);
@@ -638,21 +654,13 @@ int autoDetect(fits *fit, int layer, double sig[2], long *icold, long *ihot,
 		return 1;
 	}
 
-#ifndef _OPENMP
-	multithread = FALSE;
-#endif
-	if (com.max_thread == 1)
-		multithread = FALSE;
-
 	if (isFloat) {
-		if (multithread) {
+		if (threads > 1) {
 #ifdef _OPENMP
-#pragma omp parallel for num_threads(com.max_thread) if(multithread)
+#pragma omp parallel for num_threads(threads)
 #endif
-			for (int y = 0; y < height; y++) {
-				for (int x = 0; x < width; x++) {
-					temp[y * width + x] = fbuf[y * width + x];
-				}
+			for (size_t i = 0; i < n; i++) {
+				temp[i] = fbuf[i];
 			}
 		} else {
 			// this should be faster in a single-threaded case
@@ -660,12 +668,10 @@ int autoDetect(fits *fit, int layer, double sig[2], long *icold, long *ihot,
 		}
 	} else {
 #ifdef _OPENMP
-#pragma omp parallel for num_threads(com.max_thread) if(multithread)
+#pragma omp parallel for num_threads(threads) if(threads > 1)
 #endif
-		for (int y = 0; y < height; y++) {
-			for (int x = 0; x < width; x++) {
-				temp[y * width + x] = (float) buf[y * width + x];
-			}
+		for (size_t i = 0; i < n; i++) {
+			temp[i] = (float)buf[i];
 		}
 	}
 	const int step = is_cfa ? 2 : 1;
@@ -674,7 +680,7 @@ int autoDetect(fits *fit, int layer, double sig[2], long *icold, long *ihot,
 	long icoldL = *icold;
 	long ihotL = *ihot;
 #ifdef _OPENMP
-#pragma omp parallel num_threads(com.max_thread) if(multithread)
+#pragma omp parallel num_threads(threads) if(threads > 1)
 #endif
 	{
 		float medianin[24];
@@ -784,20 +790,28 @@ void on_button_cosmetic_ok_clicked(GtkButton *button, gpointer user_data) {
 	args->amount = gtk_adjustment_get_value(adjCosmeAmount);
 
 	args->fit = &gfit;
-	args->seqEntry = gtk_entry_get_text(cosmeticSeqEntry);
+	args->seqEntry = strdup(gtk_entry_get_text(cosmeticSeqEntry));
 	set_cursor_waiting(TRUE);
 
 	if (gtk_toggle_button_get_active(seq) && sequence_is_loaded()) {
 		if (args->seqEntry && args->seqEntry[0] == '\0')
-			args->seqEntry = "cc_";
+			args->seqEntry = strdup("cc_");
 		args->seq = &com.seq;
-		args->multithread = FALSE;
+		args->threading = SINGLE_THREADED;
 		gtk_toggle_button_set_active(seq, FALSE);
 		apply_cosmetic_to_sequence(args);
 	} else {
-		args->multithread = TRUE;
+		args->threading = MULTI_THREADED;
 		undo_save_state(&gfit, _("Cosmetic Correction"));
 		start_in_new_thread(autoDetectThreaded, args);
 	}
 }
 
+int denoise_hook_cosmetic(fits *fit) {
+	long icold = 0, ihot = 0;
+	double sig[2] = { 3.0, 3.0 };
+	for (size_t layer = 0; layer < fit->naxes[2] ; layer++) {
+		autoDetect(fit, layer, sig, &icold, &ihot, 1.0, FALSE, MULTI_THREADED);
+	}
+	return 0;
+}
