@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -26,6 +26,7 @@
 #include <fftw3.h>
 
 #include "core/siril.h"
+#include "core/siril_log.h"
 #include "gui/image_display.h"
 #include "gui/image_interactions.h"
 #include "gui/utils.h"
@@ -47,6 +48,8 @@ enum {
 	TYPE_REGULAR
 };
 
+static unsigned strategy = 0;
+
 static void fft_to_spectra(fftwf_complex *frequency_repr, float *as, float *ps,
 		float *maxi, size_t nbdata) {
 	*maxi = 0.0;
@@ -60,7 +63,7 @@ static void fft_to_spectra(fftwf_complex *frequency_repr, float *as, float *ps,
 	}
 }
 
-static void fft_to_freq(fftwf_complex *frequency_repr, float *as, float *ps,
+static void fft_to_freq(fftwf_complex *frequency_repr, const float *as, const float *ps,
 		size_t nbdata) {
 	for (size_t i = 0; i < nbdata; i++) {
 		frequency_repr[i] = as[i] * (cosf(ps[i]) + I * sinf(ps[i]));
@@ -139,7 +142,7 @@ static void centered_float(float *buf, unsigned int width, unsigned int height,
 	free(temp);
 }
 
-static void normalisation_spectra_ushort(unsigned int w, unsigned int h, float *modul, float *phase,
+static void normalisation_spectra_ushort(unsigned int w, unsigned int h, const float *modul, const float *phase,
 		WORD *abuf, WORD *pbuf, float maxi) {
 	for (size_t i = 0; i < h * w; i++) {
 		pbuf[i] = roundf_to_WORD(((phase[i] + (float)M_PI) * USHRT_MAX_SINGLE / (2.f * (float)M_PI)));
@@ -147,7 +150,7 @@ static void normalisation_spectra_ushort(unsigned int w, unsigned int h, float *
 	}
 }
 
-static void normalisation_spectra_float(unsigned int w, unsigned int h, float *modul, float *phase,
+static void normalisation_spectra_float(unsigned int w, unsigned int h, const float *modul, const float *phase,
 		float *abuf, float *pbuf, float maxi) {
 	for (size_t i = 0; i < h * w; i++) {
 		pbuf[i] = (phase[i] + (float)M_PI) / (2.f * (float)M_PI);
@@ -185,6 +188,10 @@ static void FFTD_ushort(fits *fit, fits *x, fits *y, int type_order, int layer)
 		return;
 	}
 
+
+	/* we run the Fourier Transform */
+	fftwf_plan p = fftwf_plan_dft_2d(height, width, spatial_repr, frequency_repr,
+			FFTW_FORWARD, strategy);
 	/* copying image selection into the fftw data */
 #ifdef _OPENMP
 #pragma omp parallel for num_threads(com.max_thread) private(i) schedule(static) if(nbdata > 15000)
@@ -192,10 +199,6 @@ static void FFTD_ushort(fits *fit, fits *x, fits *y, int type_order, int layer)
 	for (i = 0; i < nbdata; i++) {
 		spatial_repr[i] = (float) gbuf[i];
 	}
-
-	/* we run the Fourier Transform */
-	fftwf_plan p = fftwf_plan_dft_2d(height, width, spatial_repr, frequency_repr,
-			FFTW_FORWARD, FFTW_ESTIMATE);
 	fftwf_execute(p);
 
 	/* we compute modulus and phase */
@@ -243,6 +246,10 @@ static void FFTD_float(fits *fit, fits *x, fits *y, int type_order, int layer) {
 		return;
 	}
 
+
+	/* we run the Fourier Transform */
+	fftwf_plan p = fftwf_plan_dft_2d(height, width, spatial_repr, frequency_repr,
+			FFTW_FORWARD, strategy);
 	/* copying image selection into the fftw data */
 #ifdef _OPENMP
 #pragma omp parallel for num_threads(com.max_thread) private(i) schedule(static) if(nbdata > 15000)
@@ -250,10 +257,6 @@ static void FFTD_float(fits *fit, fits *x, fits *y, int type_order, int layer) {
 	for (i = 0; i < nbdata; i++) {
 		spatial_repr[i] = (float) gbuf[i];
 	}
-
-	/* we run the Fourier Transform */
-	fftwf_plan p = fftwf_plan_dft_2d(height, width, spatial_repr, frequency_repr,
-			FFTW_FORWARD, FFTW_ESTIMATE);
 	fftwf_execute(p);
 
 	/* we compute modulus and phase */
@@ -328,10 +331,10 @@ static void FFTI_ushort(fits *fit, fits *xfit, fits *yfit, int type_order, int l
 		return;
 	}
 
-	fft_to_freq(frequency_repr, modul, phase, nbdata);
 
 	fftwf_plan p = fftwf_plan_dft_2d(height, width, frequency_repr, spatial_repr,
-			FFTW_BACKWARD, FFTW_ESTIMATE);
+			FFTW_BACKWARD, strategy);
+	fft_to_freq(frequency_repr, modul, phase, nbdata);
 	fftwf_execute(p);
 
 	for (i = 0; i < nbdata; i++) {
@@ -387,10 +390,10 @@ static void FFTI_float(fits *fit, fits *xfit, fits *yfit, int type_order, int la
 		return;
 	}
 
-	fft_to_freq(frequency_repr, modul, phase, nbdata);
 
 	fftwf_plan p = fftwf_plan_dft_2d(height, width, frequency_repr, spatial_repr,
-			FFTW_BACKWARD, FFTW_ESTIMATE);
+			FFTW_BACKWARD, strategy);
+	fft_to_freq(frequency_repr, modul, phase, nbdata);
 	fftwf_execute(p);
 
 	for (i = 0; i < nbdata; i++) {
@@ -425,7 +428,6 @@ static gboolean end_fourier_transform(gpointer p) {
 	free(args->type);
 	free(args);
 	set_cursor_waiting(FALSE);
-	
 
 	return FALSE;
 }
@@ -437,7 +439,38 @@ gpointer fourier_transform(gpointer p) {
 	int chan;
 	struct timeval t_start, t_end;
 	fits *tmp = NULL, *tmp1 = NULL, *tmp2 = NULL;
+#ifdef HAVE_FFTW3F_MULTITHREAD
+// Change these lines to use fftw functions if double support is ever required
+	int n = com.pref.fftw_conf.multithreaded ? com.max_thread : 1;
+	fftwf_plan_with_nthreads(n);
+	fprintf(stdout, "fftwf initialized with %d threads\n", n);
+#endif
 
+// The following switch is no longer used: the overhead generated by expensive
+// planning strategies is not worth it for one-off FFTs
+
+/*	switch (com.pref.fftw_conf.strategy) {
+		case 1:
+			strategy = FFTW_MEASURE;
+			break;
+		case 2:
+			strategy = FFTW_PATIENT;
+			break;
+		case 3:
+			strategy = FFTW_EXHAUSTIVE;
+			break;
+		default:
+			strategy = FFTW_ESTIMATE;
+	}*/
+	strategy = FFTW_ESTIMATE;
+	set_wisdom_file();
+    if (fftwf_import_wisdom_from_filename(com.pref.fftw_conf.wisdom_file) == 1) {
+        siril_log_message(_("Siril FFT wisdom imported successfully...\n"));
+	} else if (fftwf_import_system_wisdom() == 1) {
+        siril_log_message(_("System FFT wisdom imported successfully...\n"));
+	} else {
+        siril_log_message(_("No FFT wisdom found to import...\n"));
+	}
 	data_type type = args->fit->type;
 
 	siril_log_color_message(_("Fourier Transform: processing...\n"), "green");
@@ -516,6 +549,12 @@ gpointer fourier_transform(gpointer p) {
 	}
 
 end:
+	if (fftwf_export_wisdom_to_filename(com.pref.fftw_conf.wisdom_file) == 1) {
+        siril_log_message(_("Siril FFT wisdom updated successfully...\n"));
+	} else {
+        siril_log_message(_("Siril FFT wisdom update failed...\n"));
+	}
+
 	invalidate_stats_from_fit(args->fit);
 	if (tmp)  { clearfits(tmp);  free(tmp);  }
 	if (tmp1) { clearfits(tmp1); free(tmp1); }
@@ -574,10 +613,8 @@ void on_button_fft_apply_clicked(GtkButton *button, gpointer user_data) {
 		phase = gtk_entry_get_text(entry_phase);
 	} else {
 		type = strdup("ffti");
-		mag = gtk_file_chooser_get_filename(
-				GTK_FILE_CHOOSER(lookup_widget("filechooser_mag")));
-		phase = gtk_file_chooser_get_filename(
-				GTK_FILE_CHOOSER(lookup_widget("filechooser_phase")));
+		mag = siril_file_chooser_get_filename(GTK_FILE_CHOOSER(lookup_widget("filechooser_mag")));
+		phase = siril_file_chooser_get_filename(GTK_FILE_CHOOSER(lookup_widget("filechooser_phase")));
 
 		if (mag == NULL || phase == NULL) {
 			char *msg = siril_log_message(_("Select magnitude and phase before !\n"));
