@@ -35,6 +35,7 @@
 #include "dialog_preview.h"
 
 static gboolean preview_allocated = FALSE; // flag needed when user load image before preview was displayed.
+static gboolean callback_is_called = TRUE;
 
 struct _updta_preview_data {
 	GtkFileChooser *file_chooser;
@@ -73,8 +74,9 @@ static gboolean end_update_preview_cb(gpointer p) {
 	fileChooserPreview *preview = args->preview;
 	siril_debug_print("preview idle\n");
 
-	if (!preview_allocated || !preview || !(GTK_IS_IMAGE((preview->image)))) {
+	if (!preview_allocated || !preview || !(GTK_IS_IMAGE(preview->image))) {
 		set_cursor_waiting(FALSE);
+		callback_is_called = TRUE;
 		return FALSE;
 	}
 
@@ -83,6 +85,7 @@ static gboolean end_update_preview_cb(gpointer p) {
 	if (!args->file_info) {
 		g_free(name_str);
 		set_cursor_waiting(FALSE);
+		callback_is_called = TRUE;
 		return FALSE;
 	}
 	type = g_file_info_get_file_type(args->file_info);
@@ -138,6 +141,7 @@ static gboolean end_update_preview_cb(gpointer p) {
 	free(args);
 	args = NULL;
 	set_cursor_waiting(FALSE);
+	callback_is_called = TRUE;
 	return FALSE;
 }
 
@@ -149,6 +153,8 @@ static gpointer update_preview(gpointer p) {
 	image_type im_type;
 	gboolean libheif_is_ok = FALSE;
 
+	callback_is_called = FALSE;
+
 	struct _updta_preview_data *args = (struct _updta_preview_data *) p;
 
 	args->description = NULL;
@@ -273,3 +279,7 @@ void siril_file_chooser_add_preview(GtkFileChooser *dialog, fileChooserPreview *
 		g_signal_connect(dialog, "update-preview", G_CALLBACK(update_preview_cb), (gpointer)preview);
 	}
 }
+
+gboolean is_callback_called() {
+	return callback_is_called;
+}
