@@ -1152,6 +1152,8 @@ void on_button_histo_apply_clicked(GtkButton *button, gpointer user_data) {
 		histo_recompute();
 		// partial cleanup
 		if (invocation == HISTO_STRETCH) {
+			siril_log_message(_("Applying MTF with values %f, %f, %f\n"),
+				_shadows, _midtones, _highlights);
 			siril_debug_print("Applying histogram (mid=%.3f, lo=%.3f, hi=%.3f)\n",
 				_midtones, _shadows, _highlights);
 			undo_save_state(get_preview_gfit_backup(),
@@ -1338,6 +1340,8 @@ void updateGHTcontrols() {
 }
 
 void toggle_histogram_window_visibility(int _invocation) {
+	siril_close_preview_dialogs();
+
 	invocation = _invocation;
 	for (int i=0;i<3;i++) {
 		do_channel[i] = TRUE;
@@ -1514,39 +1518,6 @@ gboolean on_drawingarea_histograms_button_release_event(GtkWidget *widget,
 	return FALSE;
 }
 
-void on_histoMidEntry_activate(GtkEntry *entry, gpointer user_data) {
-	float mid = g_ascii_strtod(gtk_entry_get_text(entry), NULL);
-	if (mid <= _shadows) mid = _shadows;
-	if (mid >= _highlights) mid = _highlights;
-	_midtones = mid;
-	set_cursor_waiting(TRUE);
-	update_histo_mtf();
-	histo_update_preview();
-	gchar *str = g_strdup_printf("%8.7f", mid);
-	gtk_entry_set_text(entry, str);
-	g_free(str);
-	set_cursor_waiting(FALSE);
-}
-
-gboolean on_histoMidEntry_focus_out_event(GtkWidget *widget, GdkEvent *event,
-		gpointer user_data) {
-
-	GtkEntry *entry = GTK_ENTRY(lookup_widget("histoMidEntry"));
-	float mid = g_ascii_strtod(gtk_entry_get_text(entry), NULL);
-	if (mid <= _shadows) mid = _shadows;
-	if (mid >= _highlights) mid = _highlights;
-	_midtones = mid;
-	set_cursor_waiting(TRUE);
-	update_histo_mtf();
-	histo_update_preview();
-	gchar *str = g_strdup_printf("%8.7f", mid);
-	gtk_entry_set_text(entry, str);
-	g_free(str);
-	set_cursor_waiting(FALSE);
-
-	return FALSE;
-}
-
 void on_spin_ghtD_value_changed(GtkSpinButton *button, gpointer user_data) {
 	_D = (float) expm1(gtk_spin_button_get_value(button));
 	update_histo_mtf();
@@ -1558,6 +1529,11 @@ void on_spin_ghtD_value_changed(GtkSpinButton *button, gpointer user_data) {
 
 void on_spin_ghtB_value_changed(GtkSpinButton *button, gpointer user_data) {
 	_B = (float) gtk_spin_button_get_value(button);
+	if (fabsf(_B) < 1.e-3f) {
+		_B = 0.f;
+		gtk_spin_button_set_value(button, 0.f);
+	}
+
 	update_histo_mtf();
 	update_image *param = malloc(sizeof(update_image));
 	param->update_preview_fn = histo_update_preview;
@@ -1739,7 +1715,6 @@ gboolean on_histoShadEntry_focus_out_event(GtkWidget *widget, GdkEvent *event,
 	GtkEntry *entry = GTK_ENTRY(lookup_widget("histoShadEntry"));
 	float lo = g_ascii_strtod(gtk_entry_get_text(entry), NULL);
 	if (lo <= 0.f) lo = 0.f;
-	if (lo >= _highlights) lo = _highlights;
 	_shadows = lo;
 	set_cursor_waiting(TRUE);
 	update_histo_mtf();
@@ -1755,7 +1730,6 @@ gboolean on_histoShadEntry_focus_out_event(GtkWidget *widget, GdkEvent *event,
 void on_histoShadEntry_activate(GtkEntry *entry, gpointer user_data) {
 	float lo = g_ascii_strtod(gtk_entry_get_text(entry), NULL);
 	if (lo <= 0.f) lo = 0.f;
-	if (lo >= _highlights) lo = _highlights;
 	_shadows = lo;
 	set_cursor_waiting(TRUE);
 	update_histo_mtf();
@@ -1766,11 +1740,46 @@ void on_histoShadEntry_activate(GtkEntry *entry, gpointer user_data) {
 	set_cursor_waiting(FALSE);
 }
 
+void on_histoMidEntry_activate(GtkEntry *entry, gpointer user_data) {
+	float mid = g_ascii_strtod(gtk_entry_get_text(entry), NULL);
+	_midtones = mid;
+	set_cursor_waiting(TRUE);
+	update_histo_mtf();
+	update_image *param = malloc(sizeof(update_image));
+	param->update_preview_fn = histo_update_preview;
+	param->show_preview = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget("HistoCheckPreview")));
+	notify_update((gpointer) param);
+	gchar *str = g_strdup_printf("%8.7f", mid);
+	gtk_entry_set_text(entry, str);
+	g_free(str);
+	set_cursor_waiting(FALSE);
+}
+
+gboolean on_histoMidEntry_focus_out_event(GtkWidget *widget, GdkEvent *event,
+		gpointer user_data) {
+
+	GtkEntry *entry = GTK_ENTRY(lookup_widget("histoMidEntry"));
+	float mid = g_ascii_strtod(gtk_entry_get_text(entry), NULL);
+	_midtones = mid;
+	set_cursor_waiting(TRUE);
+	update_histo_mtf();
+	update_image *param = malloc(sizeof(update_image));
+	param->update_preview_fn = histo_update_preview;
+	param->show_preview = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget("HistoCheckPreview")));
+	notify_update((gpointer) param);
+	gchar *str = g_strdup_printf("%8.7f", mid);
+	gtk_entry_set_text(entry, str);
+	g_free(str);
+	set_cursor_waiting(FALSE);
+
+	return FALSE;
+}
+
 gboolean on_histoHighEntry_focus_out_event(GtkWidget *widget, GdkEvent *event,
 		gpointer user_data) {
 	GtkEntry *entry = GTK_ENTRY(lookup_widget("histoHighEntry"));
 	float hi = g_ascii_strtod(gtk_entry_get_text(entry), NULL);
-	if (hi <= _shadows) hi = _shadows;
+//	if (hi <= _shadows) hi = _shadows;
 	if (hi >= 1.f) hi = 1.f;
 	_highlights = hi;
 	set_cursor_waiting(TRUE);
@@ -1786,7 +1795,7 @@ gboolean on_histoHighEntry_focus_out_event(GtkWidget *widget, GdkEvent *event,
 
 void on_histoHighEntry_activate(GtkEntry *entry, gpointer user_data) {
 	float hi = g_ascii_strtod(gtk_entry_get_text(entry), NULL);
-	if (hi <= _shadows) hi = _shadows;
+//	if (hi <= _shadows) hi = _shadows;
 	if (hi >= 1.f) hi = 1.f;
 	_highlights = hi;
 	set_cursor_waiting(TRUE);
