@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -24,6 +24,7 @@
 #include "core/siril.h"
 #include "core/proto.h"
 #include "core/siril_app_dirs.h"
+#include "core/siril_log.h"
 #include "core/processing.h"
 #include "algos/astrometry_solver.h"
 #include "algos/colors.h"
@@ -33,14 +34,18 @@
 #include "algos/siril_wcs.h"
 #include "algos/sorting.h"
 #include "algos/annotate.h"
+#include "filters/mtf.h"
 #include "io/single_image.h"
 #include "io/sequence.h"
 #include "gui/image_interactions.h"
 #include "gui/registration_preview.h"
 #include "gui/callbacks.h"
 #include "gui/utils.h"
+#include "livestacking/livestacking.h"
 #include "histogram.h"
 #include "registration/matching/degtorad.h"
+#include "registration/registration.h"
+#include "opencv/opencv.h"
 #include "git-version.h"
 
 #ifdef HAVE_WCSLIB
@@ -51,13 +56,17 @@
 #include "image_display.h"
 
 /* remap index data, an index for each layer */
-static BYTE *remap_index;
 static float last_pente;
 static display_mode last_mode;
 
 /* STF (auto-stretch) data */
 static gboolean stf_computed = FALSE; // Flag to know if STF parameters are available
-static float stf_shadows, stf_highlights, stf_m;
+struct mtf_params stf[3];
+
+/* widgets for draw_reg_data*/
+GtkComboBox *seqcombo;
+GtkToggleButton *drawframe;
+static GtkWidget *rotation_dlg = NULL;
 
 static void invalidate_image_render_cache(int vport);
 
@@ -132,7 +141,33 @@ static void remaprgb(void) {
 	invalidate_image_render_cache(RGB_VPORT);
 }
 
-static int make_index_for_current_display();
+void allocate_hd_remap_indices() {
+	gui.hd_remap_max = 1 << (guint) com.pref.hd_bitdepth;
+	for (unsigned i=0; i < 3; i++) {
+		if (gui.hd_remap_index[i] != NULL)
+			free(gui.hd_remap_index[i]);
+		gui.hd_remap_index[i] = (BYTE*) calloc(gui.hd_remap_max + 1, sizeof(BYTE));
+		if (gui.hd_remap_index[i] == NULL) {
+			siril_log_color_message(_("Error: memory allocaton failure when instantiating HD LUTs. Reverting to standard 16 bit LUTs.\n"), "red");
+			gui.use_hd_remap = FALSE;
+			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("autohd_item")), FALSE);
+			hd_remap_indices_cleanup();
+		}
+	}
+}
+
+void hd_remap_indices_cleanup() {
+	for (unsigned i=0 ; i < 3; i++) {
+		if (gui.hd_remap_index[i] != NULL) {
+			free(gui.hd_remap_index[i]);
+			gui.hd_remap_index[i] = NULL;
+		}
+	}
+}
+
+static int make_index_for_current_display(int vport);
+
+static int make_hd_index_for_current_display(int vport);
 
 static int make_index_for_rainbow(BYTE index[][3]);
 
@@ -170,62 +205,41 @@ static void remap(int vport) {
 	g_variant_unref(state);
 
 	if (gui.rendering_mode == HISTEQ_DISPLAY) {
-		if (gfit.type == DATA_USHORT) {
-			double hist_sum, nb_pixels;
-			size_t i, hist_nb_bins;
-			gsl_histogram *histo;
+		double hist_sum, nb_pixels;
+		size_t i, hist_nb_bins;
+		gsl_histogram *histo;
 
-			compute_histo_for_gfit();
-			histo = com.layers_hist[vport];
-			hist_nb_bins = gsl_histogram_bins(histo);
-			nb_pixels = (double)(gfit.rx * gfit.ry);
+		compute_histo_for_gfit();
+		histo = com.layers_hist[vport];
+		hist_nb_bins = gsl_histogram_bins(histo);
+		nb_pixels = (double)(gfit.rx * gfit.ry);
 
-			// build the remap_index
-			if (!remap_index)
-				remap_index = malloc(USHRT_MAX + 1);
-
-			remap_index[0] = 0;
-			hist_sum = gsl_histogram_get(histo, 0);
-			for (i = 1; i < hist_nb_bins; i++) {
-				hist_sum += gsl_histogram_get(histo, i);
-				remap_index[i] = round_to_BYTE(
-						(hist_sum / nb_pixels) * UCHAR_MAX_DOUBLE);
-			}
-
-			last_mode = gui.rendering_mode;
+		// build the remap_index
+		index = gui.remap_index[0];
+		index[0] = 0;
+		hist_sum = gsl_histogram_get(histo, 0);
+		for (i = 1; i < hist_nb_bins; i++) {
+			hist_sum += gsl_histogram_get(histo, i);
+			index[i] = round_to_BYTE(
+					(hist_sum / nb_pixels) * UCHAR_MAX_DOUBLE);
 		}
-		else if (gfit.type == DATA_FLOAT) {
-			double hist_sum, nb_pixels;
-			size_t i, hist_nb_bins;
-			gsl_histogram *histo;
 
-			compute_histo_for_gfit();
-			histo = com.layers_hist[vport];
-			hist_nb_bins = gsl_histogram_bins(histo);
-			nb_pixels = (double)(gfit.rx * gfit.ry);
-
-			// build the remap_index
-			if (!remap_index)
-				remap_index = malloc(USHRT_MAX + 1);
-
-			remap_index[0] = 0;
-			hist_sum = gsl_histogram_get(histo, 0);
-			for (i = 1; i < hist_nb_bins; i++) {
-				hist_sum += gsl_histogram_get(histo, i);
-				remap_index[i] = round_to_BYTE(
-						(hist_sum / nb_pixels) * UCHAR_MAX_DOUBLE);
-			}
-
-			last_mode = gui.rendering_mode;
-		}
+		last_mode = gui.rendering_mode;
+		histo = com.layers_hist[vport];
 		set_viewer_mode_widgets_sensitive(FALSE);
 	} else {
 		// for all other modes and ushort data, the index can be reused
 		if (gui.rendering_mode == STF_DISPLAY && !stf_computed) {
-			stf_m = findMidtonesBalance(&gfit, &stf_shadows, &stf_highlights);
+			if (gui.unlink_channels)
+				find_unlinked_midtones_balance_default(&gfit, stf);
+			else find_linked_midtones_balance_default(&gfit, stf);
 			stf_computed = TRUE;
 		}
-		make_index_for_current_display();
+		if (gui.rendering_mode == STF_DISPLAY && gui.use_hd_remap && gfit.type == DATA_FLOAT) {
+			make_hd_index_for_current_display(vport);
+		}
+		else
+			make_index_for_current_display(vport);
 		set_viewer_mode_widgets_sensitive(gui.rendering_mode != STF_DISPLAY);
 	}
 
@@ -240,9 +254,16 @@ static void remap(int vport) {
 
 	if (color == RAINBOW_COLOR)
 		make_index_for_rainbow(rainbow_index);
-	index = remap_index;
+	int target_index = gui.rendering_mode == STF_DISPLAY && gui.unlink_channels ? vport : 0;
 
-	gboolean special_mode = (gui.rendering_mode == HISTEQ_DISPLAY || gui.rendering_mode == STF_DISPLAY);
+	gboolean hd_mode = (gui.rendering_mode == STF_DISPLAY && gui.use_hd_remap && gfit.type == DATA_FLOAT);
+	if (hd_mode) {
+		index = gui.hd_remap_index[target_index];
+	}
+	else
+		index = gui.remap_index[target_index];
+
+	gboolean special_mode = (gui.rendering_mode == HISTEQ_DISPLAY || (gui.rendering_mode == STF_DISPLAY && !(gui.use_hd_remap && gfit.type == DATA_FLOAT)));
 #ifdef _OPENMP
 #pragma omp parallel for num_threads(com.max_thread) private(y) schedule(static)
 #endif
@@ -254,7 +275,10 @@ static void remap(int vport) {
 			guint src_index = y * gfit.rx + x;
 			BYTE dst_pixel_value = 0;
 			if (gfit.type == DATA_USHORT) {
-				if (special_mode) // special case, no lo & hi
+				if (hd_mode) {
+					dst_pixel_value = index[src[src_index] * gui.hd_remap_max / USHRT_MAX]; // Works as long as hd_remap_max is power of 2
+				}
+				else if (special_mode) // special case, no lo & hi
 					dst_pixel_value = index[src[src_index]];
 				else if (gui.cut_over && src[src_index] > gui.hi)	// cut
 					dst_pixel_value = 0;
@@ -262,7 +286,9 @@ static void remap(int vport) {
 					dst_pixel_value = index[src[src_index] - gui.lo < 0 ? 0 : src[src_index] - gui.lo];
 				}
 			} else if (gfit.type == DATA_FLOAT) {
-				if (special_mode) // special case, no lo & hi
+				if (hd_mode)
+					dst_pixel_value = index[float_to_max_range(fsrc[src_index], gui.hd_remap_max)];
+				else if (special_mode) // special case, no lo & hi
 					dst_pixel_value = index[roundf_to_WORD(fsrc[src_index] * USHRT_MAX_SINGLE)];
 				else if (gui.cut_over && roundf_to_WORD(fsrc[src_index] * USHRT_MAX_SINGLE) > gui.hi)	// cut
 					dst_pixel_value = 0;
@@ -296,7 +322,44 @@ static void remap(int vport) {
 	test_and_allocate_reference_image(vport);
 }
 
-static int make_index_for_current_display() {
+static int make_hd_index_for_current_display(int vport) {
+	float slope;
+	int i;
+	BYTE *index;
+	float pxl;
+	// Check if the bit depth matches the LUT size, if not we need to realloc
+	if (gui.hd_remap_max != 1 << com.pref.hd_bitdepth) {
+		gui.hd_remap_max = 1 << com.pref.hd_bitdepth;
+		allocate_hd_remap_indices();
+	}
+
+	/* initialization of data required to build the remap_index
+	 *
+	 * The HD remap curve is only used with STF rendering mode
+	 * as it is the only mode that results in a slope steep enough
+	 * to cause noticeable quantization of levels */
+	slope = UCHAR_MAX_SINGLE;
+	/************* Building the HD remap_index **************/
+	siril_debug_print("Rebuilding HD remap_index\n");
+	int target_index = gui.rendering_mode == STF_DISPLAY && gui.use_hd_remap && gui.unlink_channels ? vport : 0;
+	index = gui.hd_remap_index[target_index];
+
+	for (i = 0; i <= gui.hd_remap_max; i++) {
+		pxl = (float) i / gui.hd_remap_max;
+		index[i] = roundf_to_BYTE((MTFp(pxl, stf[target_index])) * slope);
+		// check for maximum overflow, given that df/di > 0. Should not happen with round_to_BYTE
+		if (index[i] == UCHAR_MAX)
+			break;
+	}
+	if (i != gui.hd_remap_max + 1) {
+		/* no more computation needed, just fill with max value */
+		for (++i; i <= gui.hd_remap_max; i++)
+			index[i] = UCHAR_MAX;
+	}
+	return 0;
+}
+
+static int make_index_for_current_display(int vport) {
 	float slope, delta = gui.hi - gui.lo;
 	int i;
 	BYTE *index;
@@ -326,20 +389,14 @@ static int make_index_for_current_display() {
 	}
 	if ((gui.rendering_mode != HISTEQ_DISPLAY && gui.rendering_mode != STF_DISPLAY) &&
 			slope == last_pente && gui.rendering_mode == last_mode) {
-		siril_debug_print("Re-using previous remap_index\n");
+		siril_debug_print("Re-using previous gui.remap_index\n");
 		return 0;
 	}
-	siril_debug_print("Rebuilding remap_index\n");
 
 	/************* Building the remap_index **************/
-	if (!remap_index) {
-		remap_index = malloc(USHRT_MAX + 1);
-		if (!remap_index) {
-			PRINT_ALLOC_ERR;
-			return 1;
-		}
-	}
-	index = remap_index;
+	siril_debug_print("Rebuilding gui.remap_index\n");
+	int target_index = gui.rendering_mode == STF_DISPLAY && gui.unlink_channels ? vport : 0;
+	index = gui.remap_index[target_index];
 
 	for (i = 0; i <= USHRT_MAX; i++) {
 		switch (gui.rendering_mode) {
@@ -369,7 +426,7 @@ static int make_index_for_current_display() {
 				pxl = (gfit.orig_bitpix == BYTE_IMG ?
 						(float) i / UCHAR_MAX_SINGLE :
 						(float) i / USHRT_MAX_SINGLE);
-				index[i] = roundf_to_BYTE((MTF(pxl, stf_m, stf_shadows, stf_highlights)) * slope);
+				index[i] = roundf_to_BYTE((MTFp(pxl, stf[target_index])) * slope);
 				break;
 			default:
 				return 1;
@@ -414,16 +471,6 @@ static int make_index_for_rainbow(BYTE index[][3]) {
  * v v v below:          R E D R A W I N G     W I D G E T S           v v v *
  *****************************************************************************/
 
-typedef struct draw_data {
-	cairo_t *cr;	// the context to draw to
-	int vport;	// the viewport index to draw
-	double zoom;	// the current zoom value
-	gboolean neg_view;	// negative view
-	cairo_filter_t filter;	// the type of image filtering to use
-	guint image_width, image_height;	// image size
-	guint window_width, window_height;	// drawing area size
-} draw_data_t;
-
 typedef struct label_point_struct {
 	double x, y, ra, dec, angle;
 	gboolean isRA;
@@ -483,10 +530,13 @@ static void draw_empty_image(const draw_data_t* dd) {
 #ifdef SIRIL_UNSTABLE
 
 	msg = g_strdup_printf(_("<big>Unstable Development Version</big>\n\n"
+			    "<small>%c%s</small>\n"
 				"<small>commit <tt>%s</tt></small>\n"
 				"<small>Please test bugs against "
 				"latest git master branch\n"
 				"before reporting them.</small>"),
+			toupper(PACKAGE_STRING[0]),
+			(char *)PACKAGE_STRING + 1,
 			SIRIL_GIT_VERSION_ABBREV);
 	pango_layout_set_markup(layout, msg, -1);
 	g_free(msg);
@@ -546,15 +596,21 @@ static void draw_vport(const draw_data_t* dd) {
 		view->view_width = dd->window_width;
 		view->view_height = dd->window_height;
 		cairo_t *cached_cr = cairo_create(view->disp_surface);
-
-		cairo_transform(cached_cr, &gui.display_matrix);
+		cairo_matrix_t y_reflection_matrix, flipped_matrix;
+		cairo_matrix_init_identity(&y_reflection_matrix);
+		if (livestacking_is_started() && !g_strcmp0(gfit.row_order, "TOP-DOWN")) {
+			y_reflection_matrix.yy = -1.0;
+			y_reflection_matrix.y0 = gfit.ry;
+		}
+		cairo_matrix_multiply(&flipped_matrix, &y_reflection_matrix, &gui.display_matrix);
+		cairo_transform(cached_cr, &flipped_matrix);
 		cairo_set_source_surface(cached_cr, view->full_surface, 0, 0);
 		cairo_pattern_set_filter(cairo_get_source(cached_cr), dd->filter);
 		cairo_paint(cached_cr);
 		cairo_destroy(cached_cr);
 
-		siril_debug_print("@@@\t\t\tcache surface created (%d x %d)\t\t\t@@@\n",
-				view->view_width, view->view_height);
+//		siril_debug_print("@@@\t\t\tcache surface created (%d x %d)\t\t\t@@@\n",
+//				view->view_width, view->view_height);
 	}
 	cairo_set_source_surface(dd->cr, view->disp_surface, 0, 0);
 	cairo_paint(dd->cr);
@@ -571,24 +627,67 @@ static void draw_main_image(const draw_data_t* dd) {
 	}
 }
 
+gboolean get_context_rotation_matrix(double rotation, cairo_matrix_t *transform, gboolean invert) {
+	if (rotation == 0.) return FALSE;
+	double dx = (double)com.selection.x + (double)com.selection.w * 0.5;
+	double dy = (double)com.selection.y + (double)com.selection.h * 0.5;
+	cairo_matrix_init_translate(transform, dx, dy);
+	cairo_matrix_rotate(transform, rotation * DEGTORAD);
+	cairo_matrix_translate(transform, -dx, -dy);
+	if (invert) return (cairo_matrix_invert(transform) == CAIRO_STATUS_SUCCESS);
+	return TRUE;
+}
+
+static void rotate_context(cairo_t *cr, double rotation) {
+	cairo_matrix_t transform;
+	if (!get_context_rotation_matrix(rotation, &transform, FALSE)) return;
+	cairo_transform(cr, &transform);
+}
+
 static void draw_selection(const draw_data_t* dd) {
 	if (com.selection.w > 0 && com.selection.h > 0) {
+		if ((com.selection.x + com.selection.w > gfit.rx) ||
+		(com.selection.y + com.selection.h > gfit.ry)) {
+			rectangle area = {0, 0, gfit.rx, gfit.ry};
+			memcpy(&com.selection, &area, sizeof(rectangle));
+		}
+		if (!rotation_dlg) rotation_dlg = lookup_widget("rotation_dialog");
 		cairo_t *cr = dd->cr;
 		static double dash_format[] = { 4.0, 2.0 };
 		cairo_set_line_width(cr, 1.5 / dd->zoom);
 		cairo_set_dash(cr, dash_format, 2, 0);
 		cairo_set_source_rgb(cr, 0.8, 1.0, 0.8);
+		cairo_save(cr); // save the original transform
+		if (gtk_widget_is_visible(rotation_dlg)) {
+			double dashes2[]={5.0, 5.0};
+			cairo_set_dash(cr, dashes2, 2, 0);
+			cairo_set_line_width(cr, 0.5 / dd->zoom);
+			cairo_rectangle(cr, (double) com.selection.x, (double) com.selection.y,
+						(double) com.selection.w, (double) com.selection.h);
+			cairo_stroke(cr);
+			cairo_set_line_width(cr, 3. / dd->zoom);
+			cairo_set_source_rgb(cr, 0.8, 0.0, 0.0);
+			rotate_context(cr, -gui.rotation); // cairo is positive CW while opencv is positive CCW
+
+			// draw a circle at top left corner to visualize rots larger than 90
+			double size = 10. / dd->zoom;
+			cairo_set_dash(cr, NULL, 0, 0);
+			cairo_arc(cr, com.selection.x, com.selection.y, size * 0.5, 0., 2. * M_PI);
+			cairo_stroke_preserve(cr);
+			cairo_fill(cr);
+			cairo_set_dash(cr, dash_format, 2, 0);
+		}
 		cairo_rectangle(cr, (double) com.selection.x, (double) com.selection.y,
 						(double) com.selection.w, (double) com.selection.h);
 		cairo_stroke(cr);
 
 		// display a grid when the selection is being made / modified, when it is big enough
-		if (com.pref.selection_guides > 1 && gui.drawing && com.selection.w > 40 / dd->zoom && com.selection.h > 40 / dd->zoom) {
+		if (com.pref.gui.selection_guides > 1 && gui.drawing && com.selection.w > 40 / dd->zoom && com.selection.h > 40 / dd->zoom) {
 			cairo_set_line_width(cr, 0.4 / dd->zoom);
 			cairo_set_dash(cr, NULL, 0, 0);
-			for (int i = 1; i < com.pref.selection_guides; i++) {
-				int x = com.selection.x + com.selection.w * i / com.pref.selection_guides;
-				int y = com.selection.y + com.selection.h * i / com.pref.selection_guides;
+			for (int i = 1; i < com.pref.gui.selection_guides; i++) {
+				int x = com.selection.x + com.selection.w * i / com.pref.gui.selection_guides;
+				int y = com.selection.y + com.selection.h * i / com.pref.gui.selection_guides;
 				cairo_move_to(cr, x, com.selection.y);
 				cairo_line_to(cr, x, com.selection.y + com.selection.h);
 				cairo_move_to(cr, com.selection.x, y);
@@ -598,7 +697,7 @@ static void draw_selection(const draw_data_t* dd) {
 		}
 
 		// display a mini cross when the selection is being dragged
-		if (gui.freezeX && gui.freezeY) {
+		if ((gui.freezeX && gui.freezeY) || gtk_widget_is_visible(rotation_dlg)) {
 			cairo_set_line_width(cr, 1.0 / dd->zoom);
 			point selection_center = { com.selection.x + (double)com.selection.w / 2.,
 				com.selection.y + (double)com.selection.h / 2. };
@@ -608,6 +707,7 @@ static void draw_selection(const draw_data_t* dd) {
 			cairo_line_to(cr, selection_center.x + 5 / dd->zoom, selection_center.y);
 			cairo_stroke(cr);
 		}
+		cairo_restore(cr); // restore the original transform
 	}
 }
 
@@ -623,7 +723,7 @@ static void draw_stars(const draw_data_t* dd) {
 
 		while (com.stars[i]) {
 			double size = com.stars[i]->fwhmx * 2.0;
-			if (i == com.selected_star) {
+			if (i == gui.selected_star) {
 				// We draw horizontal and vertical lines to show the star
 				cairo_set_line_width(cr, 2.0 / dd->zoom);
 				cairo_set_source_rgba(cr, 0.0, 0.4, 1.0, 0.6);
@@ -638,22 +738,40 @@ static void draw_stars(const draw_data_t* dd) {
 				cairo_set_source_rgba(cr, 1.0, 0.4, 0.0, 0.9);
 				cairo_set_line_width(cr, 1.5 / dd->zoom);
 			}
-			cairo_arc(cr, com.stars[i]->xpos, com.stars[i]->ypos, size, 0., 2. * M_PI);
+			if (com.stars[i]->has_saturated) {
+				cairo_set_source_rgba(cr, 0.75, 0.22, 1.0, 0.9);
+				cairo_set_line_width(cr, 3.0 / dd->zoom);
+			}
+			cairo_save(cr); // save the original transform
+			cairo_translate(cr, com.stars[i]->xpos, com.stars[i]->ypos);
+			cairo_rotate(cr, M_PI * 0.5 + com.stars[i]->angle * M_PI / 180.);
+			cairo_scale(cr, com.stars[i]->fwhmy / com.stars[i]->fwhmx, 1);
+			cairo_arc(cr, 0., 0., size, 0.,2 * M_PI);
+			cairo_restore(cr); // restore the original transform
 			cairo_stroke(cr);
+			/* to keep  for debugging boxes adjustements */
+			// if (com.stars[i]->R > 0)
+			// 	cairo_rectangle(cr, com.stars[i]->xpos - (double)com.stars[i]->R, com.stars[i]->ypos - (double)com.stars[i]->R, (double)com.stars[i]->R * 2 + 1, (double)com.stars[i]->R * 2 + 1);
+			// cairo_stroke(cr);
+			if (com.stars[i]->has_saturated) {
+				cairo_set_source_rgba(cr, 1.0, 0.4, 0.0, 0.9);
+				cairo_set_line_width(cr, 1.5 / dd->zoom);
+			}
+
 			i++;
 		}
 	}
 
 	/* quick photometry */
-	if (!com.script && com.qphot && mouse_status == MOUSE_ACTION_PHOTOMETRY) {
-		double size = (!com.pref.phot_set.force_radius && com.qphot) ? com.qphot->fwhmx * 2.0 : com.pref.phot_set.aperture;
+	if (!com.script && gui.qphot && mouse_status == MOUSE_ACTION_PHOTOMETRY) {
+		double size = (!com.pref.phot_set.force_radius && gui.qphot) ? gui.qphot->fwhmx * 2.0 : com.pref.phot_set.aperture;
 
 		cairo_set_dash(cr, NULL, 0, 0);
 		cairo_set_source_rgba(cr, 1.0, 0.4, 0.0, 0.9);
 		cairo_set_line_width(cr, 1.5 / dd->zoom);
 
 		/* fwhm * 2: first circle */
-		cairo_arc(cr, com.qphot->xpos, com.qphot->ypos, size, 0., 2. * M_PI);
+		cairo_arc(cr, gui.qphot->xpos, gui.qphot->ypos, size, 0., 2. * M_PI);
 		cairo_stroke(cr);
 
 		/* sky annulus */
@@ -663,29 +781,35 @@ static void draw_stars(const draw_data_t* dd) {
 			cairo_set_source_rgba(cr, 0.5, 1.0, 0.3, 0.9);
 		}
 
-		cairo_arc(cr, com.qphot->xpos, com.qphot->ypos, com.pref.phot_set.inner, 0., 2. * M_PI);
+		cairo_arc(cr, gui.qphot->xpos, gui.qphot->ypos, com.pref.phot_set.inner, 0., 2. * M_PI);
 		cairo_stroke(cr);
-		cairo_arc(cr, com.qphot->xpos, com.qphot->ypos, com.pref.phot_set.outer, 0., 2. * M_PI);
+		cairo_arc(cr, gui.qphot->xpos, gui.qphot->ypos, com.pref.phot_set.outer, 0., 2. * M_PI);
 		cairo_stroke(cr);
 		cairo_select_font_face(cr, "Purisa", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD);
 		cairo_set_font_size(cr, 40);
-		cairo_move_to(cr, com.qphot->xpos + com.pref.phot_set.outer + 5, com.qphot->ypos);
+		cairo_move_to(cr, gui.qphot->xpos + com.pref.phot_set.outer + 5, gui.qphot->ypos);
 		cairo_stroke(cr);
 	}
 
+	/* draw seqpsf stars */
 	if (sequence_is_loaded() && com.seq.current >= 0) {
-		/* draw seqpsf stars */
 		for (i = 0; i < MAX_SEQPSF && com.seq.photometry[i]; i++) {
-			cairo_set_dash(cr, NULL, 0, 0);
-			cairo_set_source_rgba(cr, com.seq.photometry_colors[i][0],
-					com.seq.photometry_colors[i][1],
-					com.seq.photometry_colors[i][2], 1.0);
-			cairo_set_line_width(cr, 2.0 / dd->zoom);
 			psf_star *the_psf = com.seq.photometry[i][com.seq.current];
 			if (the_psf) {
-				double size = (!com.pref.phot_set.force_radius && com.qphot) ? com.qphot->fwhmx * 2.0 : com.pref.phot_set.aperture;
+				double size = (!com.pref.phot_set.force_radius && the_psf->fwhmx > 0.0) ?
+					the_psf->fwhmx * 2.0 : com.pref.phot_set.aperture;
+				cairo_set_dash(cr, NULL, 0, 0);
+				// make the aperture slightly brighter
+				cairo_set_source_rgba(cr, min(com.seq.photometry_colors[i][0] + 0.2, 1.0),
+						min(com.seq.photometry_colors[i][1] + 0.2, 1.0),
+						min(com.seq.photometry_colors[i][2] + 0.2, 1.0), 1.0);
+				cairo_set_line_width(cr, 2.0 / dd->zoom);
 				cairo_arc(cr, the_psf->xpos, the_psf->ypos, size, 0., 2. * M_PI);
 				cairo_stroke(cr);
+
+				cairo_set_source_rgba(cr, com.seq.photometry_colors[i][0],
+						com.seq.photometry_colors[i][1],
+						com.seq.photometry_colors[i][2], 1.0);
 				cairo_arc(cr, the_psf->xpos, the_psf->ypos, com.pref.phot_set.inner, 0.,
 						2. * M_PI);
 				cairo_stroke(cr);
@@ -697,13 +821,10 @@ static void draw_stars(const draw_data_t* dd) {
 				cairo_move_to(cr, the_psf->xpos + com.pref.phot_set.outer + 5, the_psf->ypos);
 				if (i == 0) {
 					cairo_show_text(cr, "V");
-				}
-				else {
-					gchar *tmp;
-					tmp = g_strdup_printf("%d", i);
+				} else {
+					char tmp[16];
+					sprintf(tmp, "%d", i);
 					cairo_show_text(cr, tmp);
-
-					g_free(tmp);
 				}
 				cairo_stroke(cr);
 			}
@@ -748,6 +869,7 @@ static void draw_stars(const draw_data_t* dd) {
 				cairo_set_font_size(cr, 12.0 / dd->zoom);
 				cairo_move_to(cr, textX, textY);
 				cairo_show_text(cr, text);
+				cairo_stroke(cr);
 				g_free(text);
 			}
 		}
@@ -756,13 +878,15 @@ static void draw_stars(const draw_data_t* dd) {
 
 static void draw_brg_boxes(const draw_data_t* dd) {
 	GSList *list;
+	GdkRGBA gdk_color;
 	for (list = com.grad_samples; list; list = list->next) {
 		background_sample *sample = (background_sample *)list->data;
 		if (sample && background_sample_is_valid(sample)) {
 			int radius = (int) (background_sample_get_size(sample) / 2);
 			point position = background_sample_get_position(sample);
 			cairo_set_line_width(dd->cr, 1.5 / dd->zoom);
-			cairo_set_source_rgba(dd->cr, 1.0,  0.2, 0.3, 1.0);
+			gdk_rgba_parse(&gdk_color, com.pref.gui.config_colors.color_bkg_samples);
+		    	cairo_set_source_rgba(dd->cr, gdk_color.red, gdk_color.green, gdk_color.blue, gdk_color.alpha);
 			cairo_rectangle(dd->cr, position.x - radius - 1, position.y - radius,
 					radius * 2, radius * 2);
 			cairo_stroke(dd->cr);
@@ -772,7 +896,7 @@ static void draw_brg_boxes(const draw_data_t* dd) {
 
 #ifdef HAVE_WCSLIB
 static void draw_compass(const draw_data_t* dd) {
-	int pos = com.pref.position_compass;
+	int pos = com.pref.gui.position_compass;
 	if (!pos) return; // User chose None
 	fits *fit = &gfit;
 	cairo_t *cr = dd->cr;
@@ -787,7 +911,7 @@ static void draw_compass(const draw_data_t* dd) {
 	double len = (double) fit->ry / 20.;
 	wcs2pix(fit, ra0, dec0 + 0.1, &xN, &yN);
 	wcs2pix(fit, ra0 + 0.1, dec0, &xE, &yE);
-	if (fabs(dec0 - 90.) < len * get_wcs_image_resolution(fit)) return; //If within one arrow length of the North Pole, do not plot
+	if (90. - fabs(dec0) < 2.78e-3) return;// center is less than 10"off from a pole
 	double angleN = -atan2(yN - ypos, xN - xpos);
 	double angleE = -atan2(yE - ypos, xE - xpos);
 
@@ -830,10 +954,11 @@ static void draw_compass(const draw_data_t* dd) {
 	cairo_move_to(cr, (len / 2) * 2.0, -0.1 * len);
 	cairo_rotate(cr, -angleE);
 	cairo_show_text(cr, "E");
+	cairo_stroke(cr);
 	cairo_restore(cr); // restore the original transform
 }
 
-static label_point *new_label_point(double height, double *pix1, double *pix2, double *world, gboolean isRA, int border) {
+static label_point *new_label_point(double height, const double *pix1, const double *pix2, const double *world, gboolean isRA, int border) {
 	label_point *pt = g_new(label_point, 1);
 
 	pt->x = pix1[0];
@@ -847,12 +972,11 @@ static label_point *new_label_point(double height, double *pix1, double *pix2, d
 	return pt;
 }
 
-static int has_pole(fits *fit, double width, double height) {
-	double x, y;
-	wcs2pix(fit, 0., 90., &x, &y);
-	if ((x >= 0.) && (x <= width) && (y >= 0.) && (y <= height)) return 1;
-	wcs2pix(fit, 0., -90., &x, &y);
-	if ((x >= 0.) && (x <= width) && (y >= 0.) && (y <= height)) return -1;
+static int has_pole(fits *fit) {
+	if (!wcs2pix(fit, 0., 90., NULL, NULL))
+		return 1;
+	if (!wcs2pix(fit, 0., -90., NULL, NULL))
+		return -1;
 	return 0;
 }
 
@@ -892,7 +1016,7 @@ static double ra_values[] = { 45, 30, 15, 10, 7.5, 5, 3.75, 2.5, 1.5, 1.25, 1, 3
 
 static void draw_wcs_grid(const draw_data_t* dd) {
 #ifdef HAVE_WCSLIB
-	if (!com.show_wcs_grid) return;
+	if (!gui.show_wcs_grid) return;
 	fits *fit = &gfit;
 	if (!has_wcs(fit)) return;
 	cairo_t *cr = dd->cr;
@@ -919,9 +1043,9 @@ static void draw_wcs_grid(const draw_data_t* dd) {
 
 	/* Compute borders in pixel for tags*/
 	double pixbox[5][2] = { { 0., 0. }, { width, 0. }, { width, height }, { 0., height }, { 0., 0. } };
-	double pixval[4] = { 0., width, height, 0. }; // bottom, right, top, left with ref bottom left
+	const double pixval[4] = { 0., width, height, 0. }; // bottom, right, top, left with ref bottom left
 	int pixtype[4] = { 1, 0, 1, 0 }; // y, x, y, x
-	int polesign = has_pole(fit, width, height);
+	int polesign = has_pole(fit);
 
 	/* calculate DEC step size */
 	if (range > 16.0) {
@@ -978,9 +1102,9 @@ static void draw_wcs_grid(const draw_data_t* dd) {
 				cairo_line_to(cr, x2, y2);
 				cairo_stroke(cr);
 			}
-				// check crossing
+			// check crossing
 			if (!(((xa >= 0) && (ya >= 0) && (xa < width) && (ya < height))
-				&& ((xb >= 0) && (yb >= 0) && (xb < width) && (yb < height)))) {
+						&& ((xb >= 0) && (yb >= 0) && (xb < width) && (yb < height)))) {
 				for (int k = 0; k < 4; k ++) {
 					if (get_line_intersection(xa, ya, xb, yb, pixbox[k][0], pixbox[k][1], pixbox[k+1][0], pixbox[k+1][1], NULL, NULL)) {
 						world[0] = di;
@@ -1099,6 +1223,7 @@ static void draw_wcs_grid(const draw_data_t* dd) {
 				cairo_move_to(cr, dx, dy);
 				cairo_text_extents(cr, tag, &te2);
 				cairo_show_text(cr, tag);
+				cairo_stroke(cr);
 				cairo_restore(cr); // restore the orginal transform
 			} else {
 				g_free(tag);
@@ -1122,39 +1247,54 @@ static gdouble y_circle(gdouble y, gdouble radius) {
 
 static void draw_annotates(const draw_data_t* dd) {
 	if (!com.found_object) return;
-	fits *fit = &gfit;
-	double width = (double) fit->rx;
-	double height = (double) fit->ry;
+	gdouble resolution = get_wcs_image_resolution(&gfit);
+	if (resolution <= 0) return;
+	double width = (double) gfit.rx;
+	double height = (double) gfit.ry;
 	cairo_t *cr = dd->cr;
 	cairo_set_dash(cr, NULL, 0, 0);
 
-	if (dd->neg_view) {
-		cairo_set_source_rgba(cr, 0.5, 0.0, 0.7, 0.9);
-	} else {
-		cairo_set_source_rgba(cr, 0.5, 1.0, 0.3, 0.9);
-	}
 	cairo_set_line_width(cr, 1.0 / dd->zoom);
 	cairo_rectangle(cr, 0., 0., width, height); // to clip the grid
 	cairo_clip(cr);
-	GSList *list;
-	for (list = com.found_object; list; list = list->next) {
+
+	for (GSList *list = com.found_object; list; list = list->next) {
 		CatalogObjects *object = (CatalogObjects *)list->data;
 		gdouble radius = get_catalogue_object_radius(object);
-		gdouble world_x = get_catalogue_object_ra(object);
-		gdouble world_y = get_catalogue_object_dec(object);
+		gdouble ra = get_catalogue_object_ra(object);
+		gdouble dec = get_catalogue_object_dec(object);
 		gchar *code = get_catalogue_object_code(object);
-		gdouble resolution = get_wcs_image_resolution(&gfit);
-		gdouble x, y;
-		gdouble size = 18 * (com.pref.font_scale / 100.0);
+		guint catalog = get_catalogue_object_cat(object);
+		GdkRGBA gdk_color;
 
-		if (resolution <= 0) return;
+		switch (catalog) {
+		case USER_DSO_CAT_INDEX:
+			cairo_set_source_rgba(cr, 1.0, 0.5, 0.0, 0.9);
+			break;
+		case USER_SSO_CAT_INDEX:
+			cairo_set_source_rgba(cr, 1.0, 1.0, 0.0, 0.9);
+			break;
+		case USER_TEMP_CAT_INDEX:
+			cairo_set_source_rgba(cr, 1.0, 0.0, 0.0, 0.9);
+			break;
+		default:
+		case 0:
+			gdk_rgba_parse(&gdk_color, com.pref.gui.config_colors.color_std_annotations);
+			if (dd->neg_view) {
+				cairo_set_source_rgba(cr, 1.0 - gdk_color.red, 1.0 - gdk_color.green, 1.0 - gdk_color.blue, gdk_color.alpha);
+			} else {
+				cairo_set_source_rgba(cr, gdk_color.red, gdk_color.green, gdk_color.blue, gdk_color.alpha);
+			}
+			break;
+		}
 
 		radius = radius / resolution / 60.0;
+		// radius now in pixels
 
-		wcs2pix(&gfit, world_x, world_y, &x, &y);
-		y = height - y;
-
-		if (x > 0 && x < width && y > 0 && y < height) {
+		double fx, fy;
+		if (!wcs2pix(&gfit, ra, dec, &fx, &fy)) {
+			double x, y;
+			fits_to_display(fx, fy, &x, &y, gfit.ry);
 			point offset = {10, -10};
 			if (radius < 0) {
 				// objects we don't have an accurate location (LdN, Sh2)
@@ -1181,11 +1321,28 @@ static void draw_annotates(const draw_data_t* dd) {
 				cairo_stroke(cr);
 			}
 			if (code) {
+				gchar *name = code, *name2;
+				name2 = strstr(code, "\\n");
+				if (name2) {
+					name = g_strndup(code, name2-code);
+					name2+=2;
+				}
+
+				gdouble size = 18 * (com.pref.gui.font_scale / 100.0);
 				cairo_select_font_face(cr, "Liberation Sans", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
 				cairo_set_font_size(cr, size / dd->zoom);
 				cairo_move_to(cr, x + offset.x, y + offset.y);
-				cairo_show_text(cr, code);
+				cairo_show_text(cr, name);
 				cairo_stroke(cr);
+				if (name2) {
+					// subtitle, draw it below
+					cairo_move_to(cr, x + offset.x + 5 / dd->zoom, y + offset.y + (size + 4) / dd->zoom);
+					size = 16 * (com.pref.gui.font_scale / 100.0);
+					cairo_set_font_size(cr, size / dd->zoom);
+					cairo_show_text(cr, name2);
+					cairo_stroke(cr);
+					g_free(name);
+				}
 			}
 		}
 	}
@@ -1237,19 +1394,93 @@ static void draw_analysis(const draw_data_t* dd) {
 		str = g_strdup_printf("%.2f", com.tilt->fwhm_centre);
 		cairo_move_to(cr, gfit.rx / 2.0, (gfit.ry / 2.0) + size);
 		cairo_show_text(cr, str);
+		cairo_stroke(cr);
 		g_free(str);
 	}
 }
 
+static void draw_regframe(const draw_data_t* dd) {
+	if (com.script || com.headless) return;
+	if (!sequence_is_loaded()) return;
+	if (com.seq.current == RESULT_IMAGE) return;
+	if (!drawframe) {
+		drawframe = GTK_TOGGLE_BUTTON(lookup_widget("drawframe_check"));
+		seqcombo = GTK_COMBO_BOX(lookup_widget("seqlist_dialog_combo"));
+	}
+	if (!gtk_toggle_button_get_active(drawframe)) return;
+	int activelayer = gtk_combo_box_get_active(seqcombo);
+	if (!layer_has_registration(&com.seq, activelayer)) return;
+	if (com.seq.reg_invalidated) return;
+	transformation_type min, max;
+	guess_transform_from_seq(&com.seq, activelayer, &min, &max, FALSE);
+	if (max <= IDENTITY_TRANSFORMATION) return;
+
+	if (guess_transform_from_H(com.seq.regparam[activelayer][com.seq.reference_image].H) == NULL_TRANSFORMATION ||
+			guess_transform_from_H(com.seq.regparam[activelayer][com.seq.current].H) == NULL_TRANSFORMATION)
+		return; // reference or current image H matrix is null matrix
+
+	regframe framing = { 0 };
+	framing.pt[0].x = 0.;
+	framing.pt[0].y = 0.;
+	framing.pt[1].x = (double)com.seq.imgparam[com.seq.reference_image].rx;
+	framing.pt[1].y = 0.;
+	framing.pt[2].x = (double)com.seq.imgparam[com.seq.reference_image].rx;
+	framing.pt[2].y = (double)com.seq.imgparam[com.seq.reference_image].ry;
+	framing.pt[3].x = 0.;
+	framing.pt[3].y = (double)com.seq.imgparam[com.seq.reference_image].ry;
+	double cogx = 0., cogy = 0., cx, cy;
+	for (int i = 0; i < 4; i++) {
+		cvTransfPoint(&framing.pt[i].x, &framing.pt[i].y, com.seq.regparam[activelayer][com.seq.reference_image].H, com.seq.regparam[activelayer][com.seq.current].H);
+		cogx += framing.pt[i].x;
+		cogy += framing.pt[i].y;
+	}
+	cogx *= 0.25;
+	cogy *= 0.25;
+	cx = (com.seq.is_variable) ? (double)com.seq.imgparam[com.seq.current].rx * 0.5 : (double)com.seq.rx * 0.5;
+	cy = (com.seq.is_variable) ? (double)com.seq.imgparam[com.seq.current].ry * 0.5 : (double)com.seq.ry * 0.5;
+
+	cairo_t *cr = dd->cr;
+	double size = 10. / dd->zoom;
+	cairo_set_dash(cr, NULL, 0, 0);
+	cairo_set_source_rgb(cr, 1., 0., 0.);
+
+
+	cairo_set_line_width(cr, 2.0 / dd->zoom);
+	// reference origin
+	cairo_arc(cr, framing.pt[0].x, framing.pt[0].y, size * 0.5, 0., 2. * M_PI);
+	cairo_stroke_preserve(cr);
+	cairo_fill(cr);
+	// reference frame
+	cairo_move_to(cr, framing.pt[0].x, framing.pt[0].y);
+	cairo_line_to(cr, framing.pt[1].x, framing.pt[1].y);
+	cairo_line_to(cr, framing.pt[2].x, framing.pt[2].y);
+	cairo_line_to(cr, framing.pt[3].x, framing.pt[3].y);
+	cairo_line_to(cr, framing.pt[0].x, framing.pt[0].y);
+	cairo_stroke(cr);
+
+	// reference center
+	cairo_arc(cr, cogx, cogy, size * 0.5, 0., 2. * M_PI);
+	cairo_stroke(cr);
+	// current center
+	cairo_set_source_rgb(cr, 0., 1., 0.);
+	cairo_move_to(cr, cx - size * 0.5, cy);
+	cairo_rel_line_to(cr, size, 0.);
+	cairo_stroke(cr);
+	cairo_move_to(cr, cx, cy - size * 0.5);
+	cairo_rel_line_to(cr, 0., size);
+	cairo_stroke(cr);
+}
+
 void initialize_image_display() {
 	int i;
+	siril_debug_print("HD AutoStretch bitdepth: %d\n", com.pref.hd_bitdepth);
+	gui.hd_remap_max = 1 << (guint) com.pref.hd_bitdepth;
 	for (i = 0; i < MAXGRAYVPORT; i++) {
-		remap_index = NULL;
+		memset(gui.remap_index[i], 0, sizeof(gui.remap_index[i]));
 		last_pente = 0.f;
 		last_mode = HISTEQ_DISPLAY;
 		// only HISTEQ mode always computes the index, it's a good initializer here
 	}
-
 	cairo_matrix_init_identity(&gui.display_matrix);
 }
 
@@ -1314,8 +1545,7 @@ void adjust_vport_size_to_image() {
 
 void redraw(remap_type doremap) {
 	if (com.script) return;
-	siril_debug_print("redraw %d\n", doremap);
-
+//	siril_debug_print("redraw %d\n", doremap);
 	switch (doremap) {
 		case REDRAW_OVERLAY:
 			break;
@@ -1329,6 +1559,8 @@ void redraw(remap_type doremap) {
 			}
 			if (gfit.naxis == 3)
 				remaprgb();
+			/* redraw the 9-panel mosaic dialog if needed */
+			redraw_aberration_inspector();
 			break;
 		default:
 			siril_debug_print("UNKNOWN REMAP\n\n");
@@ -1427,6 +1659,13 @@ gboolean redraw_drawingarea(GtkWidget *widget, cairo_t *cr, gpointer data) {
 	/* background removal gradient selection boxes */
 	draw_brg_boxes(&dd);
 
+	/* registration framing*/
+	draw_regframe(&dd);
+
+	/* allow custom rendering */
+	if (gui.draw_extra)
+		gui.draw_extra(&dd);
+
 	return FALSE;
 }
 
@@ -1436,7 +1675,7 @@ point get_center_of_vport() {
 	guint window_width = gtk_widget_get_allocated_width(widget);
 	guint window_height = gtk_widget_get_allocated_height(widget);
 
-	point center = { window_width / 2, window_height / 2 };
+	point center = { window_width / 2., window_height / 2. };
 
 	return center;
 }
