@@ -9,6 +9,9 @@ typedef enum {
 	REMAP_ALL       // the image data changed, remap and render all
 } remap_type;
 
+void allocate_hd_remap_indices();
+void hd_remap_indices_cleanup();
+
 void initialize_image_display();
 
 void redraw(remap_type doremap);	// redraw the image, possibly with a remap
@@ -19,5 +22,7 @@ double get_zoom_val();	// for image_interactions
 point get_center_of_vport();
 void add_image_and_label_to_cairo(cairo_t *cr, int vport);
 
+gboolean get_context_rotation_matrix(double rotation, cairo_matrix_t *transform, gboolean invert); //computes rotation matrix about center of com.selection
+
 #endif
 
