@@ -42,6 +42,10 @@
 
 mouse_status_enum mouse_status;
 
+static gboolean double_middle_click_zooms_to_fit = FALSE;
+// Tracks whether double middle click will zoom to fit or zoom to 1:1, for the
+// toggle
+
 // caching widgets
 static GtkWidget *rotation_dlg = NULL;
 
@@ -350,18 +354,52 @@ gboolean on_drawingarea_button_press_event(GtkWidget *widget,
 	//		event->x, event->y, zoomed.x, zoomed.y, inside ? "" : " not");
 
 	if (inside) {
-		if (event->state & get_primary()) {
-			if (event->button == GDK_BUTTON_PRIMARY) {
-				// viewport translation
-				gui.translating = TRUE;
-				gui.start.x = (int)(event->x);
-				gui.start.y = (int)(event->y);
-				return TRUE;
-			}
+		/* Double middle-click toggles between zoom to fit and zoom 1:1 centred on the click */
+		if (event->button == GDK_BUTTON_MIDDLE && event->type == GDK_DOUBLE_BUTTON_PRESS) {
+			update_zoom_fit_button();
+			switch (com.pref.gui.mmb_action) {
+				case MMB_ZOOM_FIT:
+					gui.zoom_value = ZOOM_FIT;
+					reset_display_offset();
+					break;
+				case MMB_ZOOM_100:
+					gui.zoom_value = ZOOM_NONE;
+					double x = evpos.x;
+					double y = evpos.y;
+					cairo_matrix_transform_point(&gui.display_matrix, &evpos.x, &evpos.y);
+					gui.display_offset.x = evpos.x - x;
+					gui.display_offset.y = evpos.y - y;
+					break;
+				case MMB_ZOOM_TOGGLE:
+					if (double_middle_click_zooms_to_fit) {
+						gui.zoom_value = ZOOM_FIT;
+						reset_display_offset();
+					} else {
+						gui.zoom_value = ZOOM_NONE;
+						double x = evpos.x;
+						double y = evpos.y;
+						cairo_matrix_transform_point(&gui.display_matrix, &evpos.x, &evpos.y);
+						gui.display_offset.x = evpos.x - x;
+						gui.display_offset.y = evpos.y - y;
+					}
+					break;
+ 			}
+			update_zoom_label();
+			redraw(REDRAW_IMAGE);
+			double_middle_click_zooms_to_fit = !double_middle_click_zooms_to_fit;
+		}
+
+		/* Ctrl click or middle click to drag */
+		else if (((event->state & get_primary()) && (event->button == GDK_BUTTON_PRIMARY)) || (event->button == GDK_BUTTON_MIDDLE && !(event->state & get_primary()))) {
+			// viewport translation
+			gui.translating = TRUE;
+			gui.start.x = (int)(event->x);
+			gui.start.y = (int)(event->y);
+			return TRUE;
 		}
 
 		/* else, click on gray image */
-		if (event->button == GDK_BUTTON_PRIMARY) {	// left click
+		else if (event->button == GDK_BUTTON_PRIMARY) {	// left click
 			if (mouse_status == MOUSE_ACTION_SELECT_REG_AREA) {
 				if (gui.drawing) {
 					gui.drawing = FALSE;
@@ -547,8 +585,12 @@ gboolean on_drawingarea_button_release_event(GtkWidget *widget,
 			mouse_status = MOUSE_ACTION_SELECT_REG_AREA;
 			redraw(REDRAW_OVERLAY);
 		}
-	} else if (event->button == GDK_BUTTON_MIDDLE) {	// middle click
-		if (inside) {
+	} else if (event->button == GDK_BUTTON_MIDDLE) {	// middle click, stop translating the image
+		if (gui.translating) {
+			gui.translating = FALSE;
+		}
+		/* Ctrl middle-click to set the photometry box */
+		else if (inside && event->state & get_primary()) {
 			double dX = 1.5 * com.pref.phot_set.outer;
 			double dY = dX;
 			double w = 3 * com.pref.phot_set.outer;
@@ -566,7 +608,6 @@ gboolean on_drawingarea_button_release_event(GtkWidget *widget,
 				new_selection_zone();
 			}
 		}
-
 	} else if (event->button == GDK_BUTTON_SECONDARY) {	// right click
 		if (mouse_status != MOUSE_ACTION_DRAW_SAMPLES && mouse_status != MOUSE_ACTION_PHOTOMETRY) {
 			do_popup_graymenu(widget, NULL);
@@ -879,26 +920,27 @@ gboolean on_drawingarea_scroll_event(GtkWidget *widget, GdkEventScroll *event, g
 	if (!single_image_is_loaded() && !sequence_is_loaded())
 		return FALSE;
 
-	if (event->state & get_primary()) {
-		point delta;
+	point delta;
 
-		switch (event->direction) {
-		case GDK_SCROLL_SMOOTH:	// what's that?
+	// The handler is written to act on either horizontal or vertical scroll events
+	// However the handler will only ever be called for one or the other depending
+	// on the configuration.
+	switch (event->direction) {
+		case GDK_SCROLL_SMOOTH:
 			gdk_event_get_scroll_deltas((GdkEvent*) event, &delta.x, &delta.y);
 			if (delta.y < 0) {
-				return update_zoom(event->x, event->y, ZOOM_IN);
-			}
-			if (delta.y > 0) {
-				return update_zoom(event->x, event->y, ZOOM_OUT);
+				return update_zoom(event->x, event->y, ((ZOOM_IN - 1.0) * fabs(delta.y)) + 1.0);
+			} else if (delta.y > 0) {
+				return update_zoom(event->x, event->y, 1.0 / (((ZOOM_IN - 1.0) * delta.y) + 1.0));
 			}
 			break;
 		case GDK_SCROLL_DOWN:
-			return update_zoom(event->x, event->y, ZOOM_OUT);
-		case GDK_SCROLL_UP:
 			return update_zoom(event->x, event->y, ZOOM_IN);
+			// Fallthrough intentional
+		case GDK_SCROLL_UP:
+			return update_zoom(event->x, event->y, ZOOM_OUT);
 		default:
-			handled = FALSE;
-		}
+			break;
 	}
-	return handled;
+	return FALSE;
 }
