@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2017 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -129,7 +129,7 @@ static gboolean siril_message_dialog_idle(gpointer p) {
 	return FALSE;
 }
 
-void queue_message_dialog(GtkMessageType type, char *title, char *text) {
+static void queue_message_dialog(GtkMessageType type, const char *title, const char *text) {
 	if (com.headless || com.script)
 		return;	// show_dialog usually follows a siril_log_message() call
 	struct message_data *data = malloc(sizeof(struct message_data));
@@ -139,6 +139,15 @@ void queue_message_dialog(GtkMessageType type, char *title, char *text) {
 	gdk_threads_add_idle(siril_message_dialog_idle, data);
 }
 
+/* the interface that does not use GTK types */
+void queue_error_message_dialog(const char *title, const char *text) {
+	queue_message_dialog(GTK_MESSAGE_ERROR, title, text);
+}
+
+void queue_warning_message_dialog(const char *title, const char *text) {
+	queue_message_dialog(GTK_MESSAGE_WARNING, title, text);
+}
+
 void siril_data_dialog(GtkMessageType type, char *title, char *text, gchar *data) {
 	if (com.headless || com.script)
 		return;	// show_dialog usually follows a siril_log_message() call
