@@ -5,6 +5,7 @@
 struct savedial_data {
 	GtkEntry *entry;
 	gint bitspersamples;
+	gboolean tiff_compression;
 	char *description;
 	char *copyright;
 	gboolean embeded_icc;
@@ -20,7 +21,7 @@ enum {
 };
 
 void on_header_save_as_button_clicked();
-void on_header_snapshot_button_clicked();
+void on_header_snapshot_button_clicked(gboolean clipboard);
 void on_header_save_button_clicked();
 
 #endif /* SRC_GUI_SAVE_DIALOG_H_ */
