@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -36,6 +36,7 @@
 #include "core/processing.h"
 #include "core/OS_utils.h"
 #include "core/siril_app_dirs.h"
+#include "core/siril_log.h"
 #include "gui/utils.h"
 #include "gui/message_dialog.h"
 #include "gui/progress_and_log.h"
@@ -57,6 +58,9 @@ static GSList *initialize_script_paths(){
 	g_free(execpath);
 #else
 	list = g_slist_prepend(list, g_build_filename(siril_get_system_data_dir(), "scripts", NULL));
+	if (g_getenv("XDG_CONFIG_HOME") != NULL) {
+		list = g_slist_prepend(list, g_build_filename(g_get_home_dir(), getenv("XDG_CONFIG_HOME"), "scripts", NULL));
+	}
 	list = g_slist_prepend(list, g_build_filename(g_get_home_dir(), ".siril", "scripts", NULL));
 	list = g_slist_prepend(list, g_build_filename(g_get_home_dir(), "siril", "scripts", NULL));
 #endif
@@ -102,7 +106,7 @@ static GSList *search_script(const char *path) {
 
 	dir = g_dir_open(path, 0, &error);
 	if (!dir) {
-		fprintf(stderr, "scripts: %s\n", error->message);
+		g_print("scripts: %s\n", error->message);
 		g_clear_error(&error);
 		return NULL;
 	}
@@ -120,27 +124,20 @@ static GSList *search_script(const char *path) {
 }
 
 static void on_script_execution(GtkMenuItem *menuitem, gpointer user_data) {
-	GError *error = NULL;
-
 	if (get_thread_run()) {
 		PRINT_ANOTHER_THREAD_RUNNING;
 		return;
 	}
 
-	if (com.pref.save.warn_script) {
+	if (com.pref.gui.warn_script_run) {
 		gboolean dont_show_again;
 		gboolean confirm = siril_confirm_dialog_and_remember(
 				_("Please read me before using scripts"), CONFIRM_RUN_SCRIPTS, _("Run Script"), &dont_show_again);
-		com.pref.save.warn_script = !dont_show_again;
-		/* We do not use set_GUI_misc because some button state can be in an unsaved state if the
-		 * preference dialog is opened
-		 */
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget("miscAskScript")), com.pref.save.warn_script);
-		/* update config file */
-		writeinitfile();
-		if (!confirm) {
+		if (!confirm)
 			return;
-		}
+
+		com.pref.gui.warn_script_run = !dont_show_again;
+		writeinitfile();
 	}
 
 	if (com.script_thread)
@@ -151,10 +148,8 @@ static void on_script_execution(GtkMenuItem *menuitem, gpointer user_data) {
 
 	gchar *script_file = g_strdup_printf("%s%s", (gchar *) user_data, SCRIPT_EXT);
 	GFile *file = g_file_new_for_path(script_file);
-
-    GFileInfo *info;
-
-	info = g_file_query_info(file, G_FILE_ATTRIBUTE_STANDARD_SIZE,
+	GError *error = NULL;
+	GFileInfo *info = g_file_query_info(file, G_FILE_ATTRIBUTE_STANDARD_SIZE,
 			G_FILE_QUERY_INFO_NONE, NULL, &error);
 	if (info) {
 		GInputStream *input_stream = (GInputStream*) g_file_read(file, NULL, &error);
@@ -169,9 +164,7 @@ static void on_script_execution(GtkMenuItem *menuitem, gpointer user_data) {
 			g_object_unref(file);
 			return;
 		}
-		/* ensure that everything is closed */
-		process_close(0);
-		/* Then, run script */
+		/* Run the script */
 		siril_log_message(_("Starting script %s\n"), script_file);
 		com.script_thread = g_thread_new("script", execute_script, input_stream);
 	}
@@ -182,27 +175,19 @@ static void on_script_execution(GtkMenuItem *menuitem, gpointer user_data) {
 
 int initialize_script_menu() {
 	static GtkWidget *menuscript = NULL;
-	GSList *list, *script, *s;
-	GtkWidget *menu;
+	GSList *list, *script_paths, *s;
 	gint nb_item = 0;
 
-	if (!menuscript) {
+	if (!menuscript)
 		menuscript = lookup_widget("header_scripts_button");
-	}
-	
-	script = set_list_to_preferences_dialog(com.pref.script_path);
 
-	menu = gtk_menu_new();
-	gtk_widget_hide(menuscript);
+	script_paths = set_list_to_preferences_dialog(com.pref.gui.script_path);
 
-	for (s = script; s; s = s->next) {
+	GtkWidget *menu = gtk_menu_new();
+
+	for (s = script_paths; s; s = s->next) {
 		list = search_script(s->data);
 		if (list) {
-			GSList *l;
-			if (!gtk_widget_get_visible(menuscript)) {
-				gtk_widget_show(menuscript);
-				gtk_menu_button_set_popup(GTK_MENU_BUTTON(menuscript), menu);
-			}
 			/* write separator but not for the first one */
 			if (nb_item != 0) {
 				GtkWidget *separator = gtk_separator_menu_item_new();
@@ -210,8 +195,9 @@ int initialize_script_menu() {
 				gtk_widget_show(separator);
 			}
 			siril_log_color_message(_("Searching scripts in: \"%s\"...\n"), "green", s->data);
-			for (l = list; l; l = l->next) {
-				nb_item ++;
+
+			for (GSList *l = list; l; l = l->next) {
+				nb_item++;
 				/* write an item per script file */
 				GtkWidget *menu_item;
 
@@ -228,8 +214,13 @@ int initialize_script_menu() {
 		}
 	}
 
-	writeinitfile();
-
+	if (!nb_item) {
+		gtk_widget_hide(menuscript);
+		return 0;
+	}
+	gtk_menu_button_set_popup(GTK_MENU_BUTTON(menuscript), menu);
+	if (!gtk_widget_get_visible(menuscript))
+		gtk_widget_show(menuscript);
 	return 0;
 }
 
@@ -241,8 +232,8 @@ int refresh_scripts(gboolean update_list, gchar **error) {
 		err = siril_log_color_message(_("Cannot refresh the scripts if the list is empty.\n"), "red");
 		retval = 1;
 	} else {
-		g_slist_free_full(com.pref.script_path, g_free);
-		com.pref.script_path = list;
+		g_slist_free_full(com.pref.gui.script_path, g_free);
+		com.pref.gui.script_path = list;
 		retval = initialize_script_menu();
 	}
 	if (error) {
@@ -292,19 +283,19 @@ GSList *set_list_to_preferences_dialog(GSList *list) {
 
 /* Get Scripts menu */
 
-#define GET_SCRIPTS_URL "https://free-astro.org/index.php?title=Siril:scripts"
+#define GET_SCRIPTS_URL "https://siril.readthedocs.io"
 
 void siril_get_on_script_pages() {
 	gboolean ret;
 	const char *locale;
-	const char *supported_languages[] = { "fr", NULL }; // en is NULL: default language
+	const char *supported_languages[] = { "de", "fr", "it", "ru", NULL };  // en is NULL: default language
 	gchar *lang = NULL;
 	int i = 0;
 
-	if (!g_strcmp0(com.pref.combo_lang, "")) {
+	if (!com.pref.lang || !g_strcmp0(com.pref.lang, "")) {
 		locale = setlocale(LC_MESSAGES, NULL);
 	} else {
-		locale = com.pref.combo_lang;
+		locale = com.pref.lang;
 	}
 
 	if (locale) {
@@ -316,7 +307,11 @@ void siril_get_on_script_pages() {
 			i++;
 		}
 	}
-	gchar *url = g_build_path("/", GET_SCRIPTS_URL, lang, NULL);
+	if (!lang) {
+		lang = g_strdup_printf("en"); // Last gasp fallback in case there is an error with the locale
+	}
+
+	gchar *url = g_build_path("/", GET_SCRIPTS_URL, lang, "/stable/Scripts.html#getting-more-scripts", NULL);
 
 #if GTK_CHECK_VERSION(3, 22, 0)
 	GtkWidget* win = lookup_widget("control_window");
