@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -100,24 +100,22 @@ GtkWidget* popover_new_with_image(GtkWidget *widget, const gchar *text, GdkPixbu
 
 	popover = gtk_popover_new(widget);
 	label = gtk_label_new(NULL);
-	gtk_label_set_line_wrap_mode(GTK_LABEL(label), PANGO_WRAP_WORD_CHAR);
+	gtk_label_set_line_wrap_mode(GTK_LABEL(label), PANGO_WRAP_WORD);
 	box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);
 
 	if (pixbuf) {
 		float ratio = 1.0 * gdk_pixbuf_get_height(pixbuf) / gdk_pixbuf_get_width(pixbuf);
 		int width = 128, height = 128 * ratio;
-		GdkPixbuf *new_pixbuf = gdk_pixbuf_scale_simple(pixbuf, width, height,
-				GDK_INTERP_BILINEAR);
+		GdkPixbuf *new_pixbuf = gdk_pixbuf_scale_simple(pixbuf, width, height, GDK_INTERP_BILINEAR);
 		image = gtk_image_new_from_pixbuf(new_pixbuf);
 		g_object_unref(new_pixbuf);
 	} else {
-		image = gtk_image_new_from_icon_name("dialog-information-symbolic",
-					GTK_ICON_SIZE_DIALOG);
+		image = gtk_image_new_from_icon_name("dialog-information-symbolic", GTK_ICON_SIZE_DIALOG);
 	}
 
 	gtk_label_set_markup(GTK_LABEL(label), text);
 	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
-	gtk_label_set_max_width_chars(GTK_LABEL(label), 80);
+	gtk_label_set_max_width_chars(GTK_LABEL(label), 100);
 
 	gtk_box_pack_start(GTK_BOX(box), image, FALSE, FALSE, 10);
 	gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 10);
@@ -171,13 +169,14 @@ void set_GUI_DiskSpace(gint64 space, const gchar *label) {
 	gboolean set_class = FALSE;
 
 	if (space > 0) {
-		if (space < 1073741824) { // we want to warn user of space is less than 1GiB
+		if (space < 1073741824) { // we want to warn user if space is less than 1GiB
 			set_class = TRUE;
 		}
 		gchar *mem = g_format_size_full(space, G_FORMAT_SIZE_IEC_UNITS);
 		str = g_strdup_printf(_("Disk Space: %s"), mem);
 		g_free(mem);
 	} else {
+		set_class = TRUE;
 		str = g_strdup(_("Disk Space: N/A"));
 	}
 	set_label_text_from_main_thread(label, str,
@@ -198,7 +197,7 @@ void unset_suggested(GtkWidget *widget) {
 
 /* Managing GUI calls *
  * We try to separate core concerns from GUI (GTK+) concerns to keep the core
- * code efficient and clean, and avoir calling GTK+ functions in threads other
+ * code efficient and clean, and avoid calling GTK+ functions in threads other
  * than the main GTK+ thread.
  * GTK+ functions calls should be grouped in idle functions: these are
  * functions called by the GTK+ main thread whenever it's idle. The problem is
@@ -248,3 +247,7 @@ void execute_idle_and_wait_for_it(gboolean (* idle)(gpointer), gpointer arg) {
 	g_cond_clear(&data.cond);
 	siril_debug_print("idle %p wait is over\n", idle);
 }
+
+int select_vport(int vport) {
+	return vport == RGB_VPORT ? GREEN_VPORT : vport;
+}
