@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -26,7 +26,7 @@
 
 #include <glib.h>
 
-gboolean test_if_symlink_is_ok();
+gboolean test_if_symlink_is_ok(gboolean verbose);
 gpointer symlink_thread_worker(gpointer p);
 int symlink_uniq_file(gchar *src_filename, gchar *dest_filename, gboolean allow_symlink);
 
