@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -38,7 +38,8 @@
 #include <libswresample/swresample.h>
 
 #include "io/mp4_output.h"
-#include "core/proto.h"	// computePente
+#include "core/proto.h"
+#include "core/siril_log.h"
 #include "gui/progress_and_log.h"
 
 //#define STREAM_DURATION   10.0
@@ -79,7 +80,7 @@ static int write_frame(AVFormatContext *fmt_ctx, const AVRational *time_base, AV
 }
 
 /* Add an output stream. */
-static int add_stream(struct mp4_struct *ost, AVCodec **codec,
+static int add_stream(struct mp4_struct *ost, const AVCodec **codec,
 		enum AVCodecID codec_id, int w, int h, int fps)
 {
 	AVCodecContext *c;
@@ -202,7 +203,7 @@ static AVFrame *alloc_picture(enum AVPixelFormat pix_fmt, int width, int height)
 	return picture;
 }
 
-static int open_video(AVCodec *codec, struct mp4_struct *ost, AVDictionary *opt_arg, int nb_layers)
+static int open_video(const AVCodec *codec, struct mp4_struct *ost, AVDictionary *opt_arg, int nb_layers)
 {
 	AVCodecContext *c = ost->enc;
 	AVDictionary *opt = NULL;
@@ -271,7 +272,7 @@ static int fill_rgb_image(AVFrame *pict, int frame_index,
 
 	BYTE map[USHRT_MAX + 1];
 	int i;
-	
+
 	float slope = (fit->orig_bitpix == BYTE_IMG) ? 1.0f : UCHAR_MAX_SINGLE / USHRT_MAX_SINGLE;
 
 	for (i = 0; i <= USHRT_MAX; i++) {
@@ -284,7 +285,7 @@ static int fill_rgb_image(AVFrame *pict, int frame_index,
 		for (++i; i <= USHRT_MAX; i++)
 			map[i] = UCHAR_MAX;
 	}
-	
+
 	/* doing the WORD to BYTE conversion, bottom-up */
 	if (fit->naxes[2] == 1) {
 		int x, y;
@@ -443,7 +444,7 @@ static void close_stream(struct mp4_struct *ost)
 struct mp4_struct *mp4_create(const char *filename, int dst_w, int dst_h, int fps, int nb_layers, int quality, int src_w, int src_h, export_format type)
 {
 	struct mp4_struct *video_st;
-	AVCodec *video_codec;
+	const AVCodec *video_codec;
 	int ret;
 	AVDictionary *opt = NULL;
 
@@ -469,21 +470,23 @@ struct mp4_struct *mp4_create(const char *filename, int dst_w, int dst_h, int fp
 		}
 	}
 
-	video_st->fmt = video_st->oc->oformat;
+	video_st->fmt = (AVOutputFormat *)video_st->oc->oformat;
 	video_st->quality = quality;
 	/* disable unwanted features, is this the correct way? */
-	video_st->fmt->audio_codec = AV_CODEC_ID_NONE;
+	// video_st->fmt->audio_codec = AV_CODEC_ID_NONE;
+	enum AVCodecID codecid;
 	switch(type) {
 	case EXPORT_WEBM_VP9:
-		video_st->fmt->video_codec = AV_CODEC_ID_VP9;
+		codecid = AV_CODEC_ID_VP9;
 		break;
 	case EXPORT_MP4:
-		video_st->fmt->video_codec = AV_CODEC_ID_H264;
+		codecid = AV_CODEC_ID_H264;
 		break;
 	case EXPORT_MP4_H265:
-		video_st->fmt->video_codec = AV_CODEC_ID_H265;
+		codecid = AV_CODEC_ID_H265;
 		break;
 	default:
+		free(video_st);
 		fprintf(stderr, "mp4_create: unknown type, should not happen\n");
 		return NULL;
 	}
@@ -492,7 +495,7 @@ struct mp4_struct *mp4_create(const char *filename, int dst_w, int dst_h, int fp
 
 	/* Add the video stream and initialize the codecs. */
 	if (video_st->fmt->video_codec != AV_CODEC_ID_NONE) {
-		if (add_stream(video_st, &video_codec, video_st->fmt->video_codec, dst_w, dst_h, fps)) {
+		if (add_stream(video_st, &video_codec, codecid, dst_w, dst_h, fps)) {
 			avformat_free_context(video_st->oc);
 			free(video_st);
 			siril_log_message("Could not add the video stream in the output film, aborting\n");
@@ -500,6 +503,7 @@ struct mp4_struct *mp4_create(const char *filename, int dst_w, int dst_h, int fp
 		}
 	} else {
 		avformat_free_context(video_st->oc);
+		free(video_st);
 		siril_log_message("Error setting video codec for output file.");
 		return NULL;
 	}
