@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -32,6 +32,7 @@
 
 #include "core/siril.h"
 #include "core/OS_utils.h"
+#include "core/siril_log.h"
 #include "algos/statistics.h"
 #include "io/fits_sequence.h"
 #include "io/ser.h"
@@ -39,6 +40,7 @@
 #include "core/proto.h"
 #include "gui/utils.h"
 #include "gui/progress_and_log.h"
+#include "registration/registration.h"
 #ifdef HAVE_FFMS2
 #include "io/films.h"
 #endif
@@ -48,15 +50,16 @@
  * version 1 introduced roundness in regdata, 0.9.9
  * version 2 allowed regdata to be stored for CFA SER sequences, 0.9.11
  * version 3 introduced new weighted fwhm criteria, 0.99.0
+ * version 4 introduced variable size sequences, extended registration data (incl. H), 1.1.0
  */
-#define CURRENT_SEQFILE_VERSION 3	// to increment on format change
+#define CURRENT_SEQFILE_VERSION 4	// to increment on format change
 
 /* File format (lines starting with # are comments, lines that are (for all
  * something) need to be in all in sequence of this only type of line):
  *
  * S sequence_name beg number selnum fixed reference_image [version]
  * L nb_layers
- * (for all images) I filenum incl [stats+] <- stats added at some point, removed in 0.9.9
+ * (for all images) I filenum incl [width,height] [stats+] <- stats added at some point, removed in 0.9.9
  * (for all layers (x)) Rx regparam+
  * TS | TA | TF (type for ser or film (avi) or fits)
  * U up-scale_ratio
@@ -107,13 +110,13 @@ sequence * readseqfile(const char *name){
 				 * Such sequences don't exist anymore. */
 				assert(line[2] != '"');
 				if (line[2] == '\'')	/* new format, quoted string */
-					scanformat = "'%511[^']' %d %d %d %d %d %d";
-				else scanformat = "%511s %d %d %d %d %d %d";
+					scanformat = "'%511[^']' %d %d %d %d %d %d %d %d";
+				else scanformat = "%511s %d %d %d %d %d %d %d %d";
 
 				if(sscanf(line+2, scanformat,
 							filename, &seq->beg, &seq->number,
 							&seq->selnum, &seq->fixed,
-							&seq->reference_image, &version) < 6 ||
+							&seq->reference_image, &version, &seq->is_variable, &seq->fz) < 6 ||
 						allocated != 0){
 					fprintf(stderr,"readseqfile: sequence file format error: %s\n",line);
 					goto error;
@@ -175,27 +178,42 @@ sequence * readseqfile(const char *name){
 					fprintf(stderr, "readseqfile: sequence file format error, missing S line\n");
 					goto error;
 				}
-				allocate_stats(&stats);
-				nb_tokens = sscanf(line + 2,
-						"%d %d %lg %lg %lg %lg %lg %lg %lg %lg %lg %lg",
-						&(seq->imgparam[i].filenum),
-						&(seq->imgparam[i].incl),
-						&(stats->mean),
-						&(stats->median),
-						&(stats->sigma),
-						&(stats->avgDev),
-						&(stats->mad),
-						&(stats->sqrtbwmv),
-						&(stats->location),
-						&(stats->scale),
-						&(stats->min),
-						&(stats->max));
-				if (nb_tokens == 12) {
-					add_stats_to_seq(seq, i, 0, stats);
-					free_stats(stats);	// we unreference it here
+
+				if (version <= 3) {
+					allocate_stats(&stats);
+					nb_tokens = sscanf(line + 2,
+							"%d %d %lg %lg %lg %lg %lg %lg %lg %lg %lg %lg",
+							&(seq->imgparam[i].filenum),
+							&(seq->imgparam[i].incl),
+							&(stats->mean),
+							&(stats->median),
+							&(stats->sigma),
+							&(stats->avgDev),
+							&(stats->mad),
+							&(stats->sqrtbwmv),
+							&(stats->location),
+							&(stats->scale),
+							&(stats->min),
+							&(stats->max));
+					if (nb_tokens == 12) {
+						add_stats_to_seq(seq, i, 0, stats);
+						free_stats(stats);	// we unreference it here
+					} else {
+						free_stats(stats);
+						if (nb_tokens != 2) {
+							fprintf(stderr, "readseqfile: sequence file format error: %s\n", line);
+							goto error;
+						}
+					}
 				} else {
-					free_stats(stats);
-					if (nb_tokens != 2) {
+					// v4: width and height for variable sequences
+					nb_tokens = sscanf(line + 2, "%d %d %d,%d",
+							&(seq->imgparam[i].filenum),
+							&(seq->imgparam[i].incl),
+							&(seq->imgparam[i].rx),
+							&(seq->imgparam[i].ry));
+					if ((nb_tokens != 4 && seq->is_variable) ||
+							(nb_tokens != 2 && !seq->is_variable)) {
 						fprintf(stderr, "readseqfile: sequence file format error: %s\n", line);
 						goto error;
 					}
@@ -268,10 +286,10 @@ sequence * readseqfile(const char *name){
 					goto error;
 				}
 				if (version < 1) {
-					float rot_centre_x, rot_centre_y, angle;
+					float rot_centre_x, rot_centre_y, angle, shiftx, shifty;
 					nb_tokens = sscanf(line+3, "%f %f %g %g %g %g %lg",
-							&(regparam[i].shiftx),
-							&(regparam[i].shifty),
+							&shiftx,
+							&shifty,
 							&rot_centre_x, &rot_centre_y,
 							&angle,
 							&(regparam[i].fwhm),
@@ -286,22 +304,26 @@ sequence * readseqfile(const char *name){
 							goto error;
 						}
 					}
+					regparam[i].H = H_from_translation(shiftx, shifty);
 				} else if (version <= 2) { // include version 1
 					// version 2 with roundness instead of weird things
+					float shiftx, shifty;
 					if (sscanf(line+3, "%f %f %g %g %lg",
-								&(regparam[i].shiftx),
-								&(regparam[i].shifty),
+								&shiftx,
+								&shifty,
 								&(regparam[i].fwhm),
 								&(regparam[i].roundness),
 								&(regparam[i].quality)) != 5) {
 						fprintf(stderr,"readseqfile: sequence file format error: %s\n",line);
 						goto error;
 					}
-				} else {
+					regparam[i].H = H_from_translation(shiftx, shifty);
+				} else if (version == 3) {
 					// version 3 with weighted_fwhm
+					float shiftx, shifty;
 					if (sscanf(line+3, "%f %f %g %g %g %lg",
-								&(regparam[i].shiftx),
-								&(regparam[i].shifty),
+								&shiftx,
+								&shifty,
 								&(regparam[i].fwhm),
 								&(regparam[i].weighted_fwhm),
 								&(regparam[i].roundness),
@@ -309,6 +331,29 @@ sequence * readseqfile(const char *name){
 						fprintf(stderr,"readseqfile: sequence file format error: %s\n",line);
 						goto error;
 					}
+					regparam[i].H = H_from_translation(shiftx, shifty);
+				}
+				else {
+					// version 4 without shifts and with homography matrix
+					if (sscanf(line+3, "%g %g %g %lg %g %d H %lg %lg %lg %lg %lg %lg %lg %lg %lg",
+								&(regparam[i].fwhm),
+								&(regparam[i].weighted_fwhm),
+								&(regparam[i].roundness),
+								&(regparam[i].quality),
+								&(regparam[i].background_lvl),
+								&(regparam[i].number_of_stars),
+								&(regparam[i].H.h00),
+								&(regparam[i].H.h01),
+								&(regparam[i].H.h02),
+								&(regparam[i].H.h10),
+								&(regparam[i].H.h11),
+								&(regparam[i].H.h12),
+								&(regparam[i].H.h20),
+								&(regparam[i].H.h21),
+								&(regparam[i].H.h22)) != 15) {
+						fprintf(stderr,"readseqfile: sequence file format error: %s\n",line);
+						goto error;
+					}
 				}
 				++i;
 				break;
@@ -346,15 +391,16 @@ sequence * readseqfile(const char *name){
 				else if (line[1] == 'F') {
 					seq->type = SEQ_FITSEQ;
 #ifdef HAVE_FFMS2
-					seq->ext = com.pref.ext + 1;
+					seq->ext = get_com_ext(seq->fz) + 1;
 #endif
 					if (seq->fitseq_file) break;
-					seq->fitseq_file = malloc(sizeof(struct ser_struct));
+					seq->fitseq_file = malloc(sizeof(struct fits_sequence));
 					fitseq_init_struct(seq->fitseq_file);
 					GString *fileString = g_string_new(filename);
-					g_string_append(fileString, com.pref.ext);
+					g_string_append(fileString, get_com_ext(seq->fz));
 					seq->fitseq_file->filename = g_string_free(fileString, FALSE);
 					if (fitseq_open(seq->fitseq_file->filename, seq->fitseq_file)) {
+						g_free(seq->fitseq_file->filename);
 						free(seq->fitseq_file);
 						seq->fitseq_file = NULL;
 						goto error;
@@ -408,7 +454,7 @@ sequence * readseqfile(const char *name){
 					}
 					else {
 						film_display_info(seq->film_file);
-						seq->ext = strdup(get_filename_ext(seq->film_file->filename));
+						seq->ext = get_filename_ext(seq->film_file->filename);
 						g_free(filmname);
 					}
 				}
@@ -538,6 +584,7 @@ int writeseqfile(sequence *seq){
 	int i, layer;
 
 	if (!seq->seqname || seq->seqname[0] == '\0') return 1;
+	if (!seq->imgparam) return 1;
 	filename = malloc(strlen(seq->seqname)+5);
 	sprintf(filename, "%s.seq", seq->seqname);
 	seqfile = g_fopen(filename, "w+t");
@@ -550,10 +597,11 @@ int writeseqfile(sequence *seq){
 	fprintf(stdout, "Writing sequence file %s\n", filename);
 	free(filename);
 
-	fprintf(seqfile,"#Siril sequence file. Contains list of files (images), selection, and registration data\n");
-	fprintf(seqfile,"#S 'sequence_name' start_index nb_images nb_selected fixed_len reference_image version\n");
-	fprintf(seqfile,"S '%s' %d %d %d %d %d %d\n",
-			seq->seqname, seq->beg, seq->number, seq->selnum, seq->fixed, seq->reference_image, CURRENT_SEQFILE_VERSION);
+	fprintf(seqfile,"#Siril sequence file. Contains list of images, selection, registration data and statistics\n");
+	fprintf(seqfile,"#S 'sequence_name' start_index nb_images nb_selected fixed_len reference_image version variable_size fz_flag\n");
+	fprintf(seqfile,"S '%s' %d %d %d %d %d %d %d %d\n",
+			seq->seqname, seq->beg, seq->number, seq->selnum, seq->fixed,
+			seq->reference_image, CURRENT_SEQFILE_VERSION, seq->is_variable, seq->fz);
 	if (seq->type != SEQ_REGULAR) {
 		char type;
 		switch (seq->type) {
@@ -576,23 +624,40 @@ int writeseqfile(sequence *seq){
 	fprintf(seqfile, "L %d\n", seq->nb_layers);
 
 	for(i=0; i < seq->number; ++i){
-		fprintf(seqfile,"I %d %d\n",
-				seq->imgparam[i].filenum,
-				seq->imgparam[i].incl);
+		if (seq->is_variable) {
+			fprintf(seqfile,"I %d %d %d,%d\n",
+					seq->imgparam[i].filenum,
+					seq->imgparam[i].incl,
+					seq->imgparam[i].rx,
+					seq->imgparam[i].ry);
+		} else {
+			fprintf(seqfile,"I %d %d\n",
+					seq->imgparam[i].filenum,
+					seq->imgparam[i].incl);
+		}
 	}
 
 	for (layer = 0; layer < seq->nb_layers; layer++) {
-		if (seq->regparam[layer]) {
+		if (seq->regparam && seq->regparam[layer]) {
 			for (i=0; i < seq->number; ++i) {
-				fprintf(seqfile, "R%c %f %f %g %g %g %g\n",
+				fprintf(seqfile, "R%c %g %g %g %g %g %d H %g %g %g %g %g %g %g %g %g\n",
 						seq->cfa_opened_monochrome ? '*' : '0' + layer,
-						seq->regparam[layer][i].shiftx,
-						seq->regparam[layer][i].shifty,
 						seq->regparam[layer][i].fwhm,
 						seq->regparam[layer][i].weighted_fwhm,
 						seq->regparam[layer][i].roundness,
-						seq->regparam[layer][i].quality
-				       );
+						seq->regparam[layer][i].quality,
+						seq->regparam[layer][i].background_lvl,
+						seq->regparam[layer][i].number_of_stars,
+						seq->regparam[layer][i].H.h00,
+						seq->regparam[layer][i].H.h01,
+						seq->regparam[layer][i].H.h02,
+						seq->regparam[layer][i].H.h10,
+						seq->regparam[layer][i].H.h11,
+						seq->regparam[layer][i].H.h12,
+						seq->regparam[layer][i].H.h20,
+						seq->regparam[layer][i].H.h21,
+						seq->regparam[layer][i].H.h22
+					);
 			}
 		}
 		if (seq->stats && seq->stats[layer]) {
@@ -622,15 +687,24 @@ int writeseqfile(sequence *seq){
 	for (layer = 0; layer < 3; layer++) {
 		if (seq->regparam_bkp && seq->regparam_bkp[layer]) {
 			for (i=0; i < seq->number; ++i) {
-				fprintf(seqfile, "R%c %f %f %g %g %g %g\n",
+				fprintf(seqfile, "R%c %g %g %g %g %g %d H %g %g %g %g %g %g %g %g %g\n",
 						seq->cfa_opened_monochrome ? '0' + layer : '*',
-						seq->regparam_bkp[layer][i].shiftx,
-						seq->regparam_bkp[layer][i].shifty,
 						seq->regparam_bkp[layer][i].fwhm,
 						seq->regparam_bkp[layer][i].weighted_fwhm,
 						seq->regparam_bkp[layer][i].roundness,
-						seq->regparam_bkp[layer][i].quality
-				       );
+						seq->regparam_bkp[layer][i].quality,
+						seq->regparam_bkp[layer][i].background_lvl,
+						seq->regparam_bkp[layer][i].number_of_stars,
+						seq->regparam_bkp[layer][i].H.h00,
+						seq->regparam_bkp[layer][i].H.h01,
+						seq->regparam_bkp[layer][i].H.h02,
+						seq->regparam_bkp[layer][i].H.h10,
+						seq->regparam_bkp[layer][i].H.h11,
+						seq->regparam_bkp[layer][i].H.h12,
+						seq->regparam_bkp[layer][i].H.h20,
+						seq->regparam_bkp[layer][i].H.h21,
+						seq->regparam_bkp[layer][i].H.h22
+					);
 			}
 		}
 		if (seq->stats_bkp && seq->stats_bkp[layer]) {
@@ -653,7 +727,6 @@ int writeseqfile(sequence *seq){
 						seq->stats_bkp[layer][i]->max,
 						seq->stats_bkp[layer][i]->normValue,
 						seq->stats_bkp[layer][i]->bgnoise);
-
 			}
 		}
 	}
