@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 #include "core/siril.h"
 #include "core/proto.h"
 #include "core/siril_date.h"
+#include "core/siril_log.h"
 #include "sequence.h"
 #include "ser.h"
 #include "stacking/stacking.h"
@@ -31,6 +32,7 @@
 #include "gui/message_dialog.h"
 #include "gui/progress_and_log.h"
 #include "io/image_format_fits.h"
+#include "io/Astro-TIFF.h"
 #ifdef HAVE_FFMS2
 #include "io/films.h"
 #endif
@@ -58,6 +60,8 @@ struct exportseq_args {
 	export_format output;
 	gboolean normalize;
 
+	gboolean tiff_compression; // compression of TIFF files
+
 	int film_fps;		// has to be int for avi or ffmpeg
 	int film_quality;	// [1, 5], for mp4 and webm
 
@@ -252,7 +256,7 @@ static gpointer export_sequence(gpointer ptr) {
 	}
 
 	if (output_bitpix == FLOAT_IMG)
-		output_bitpix = com.pref.force_to_16bit ? USHORT_IMG : FLOAT_IMG;
+		output_bitpix = com.pref.force_16bit ? USHORT_IMG : FLOAT_IMG;
 
 	nb_frames = compute_nb_filtered_images(args->seq,
 			args->filtering_criterion, args->filtering_parameter);
@@ -272,6 +276,8 @@ static gpointer export_sequence(gpointer ptr) {
 		stackargs.reglayer = reglayer;
 		stackargs.use_32bit_output = (output_bitpix == FLOAT_IMG);
 		stackargs.ref_image = args->seq->reference_image;
+		stackargs.equalizeRGB = FALSE;
+		stackargs.lite_norm = FALSE;
 
 		// build image indices used by normalization
 		if (stack_fill_list_of_unfiltered_images(&stackargs))
@@ -319,7 +325,7 @@ static gpointer export_sequence(gpointer ptr) {
 			goto free_and_reset_progress_bar;
 		}
 		gchar *tmpmsg = g_strdup_printf(_("Processing image %s"), filename);
-		set_progress_bar_data(tmpmsg, (double)cur_nb / (double)nb_frames);
+		set_progress_bar_data(tmpmsg, (double)cur_nb / (nb_frames == 0 ? 1 : (double)nb_frames));
 		g_free(tmpmsg);
 
 		/* we read the full frame */
@@ -398,8 +404,10 @@ static gpointer export_sequence(gpointer ptr) {
 		int shiftx, shifty;
 		/* load registration data for current image */
 		if (reglayer != -1 && args->seq->regparam[reglayer]) {
-			shiftx = roundf_to_int(args->seq->regparam[reglayer][i].shiftx);
-			shifty = roundf_to_int(args->seq->regparam[reglayer][i].shifty);
+			double dx, dy;
+			translation_from_H(args->seq->regparam[reglayer][i].H, &dx, &dy);
+			shiftx = round_to_int(dx);
+			shifty = round_to_int(dy);
 		} else {
 			shiftx = 0;
 			shifty = 0;
@@ -466,7 +474,9 @@ static gpointer export_sequence(gpointer ptr) {
 #ifdef HAVE_LIBTIFF
 			case EXPORT_TIFF:
 				snprintf(dest, 255, "%s%05d", args->basename, i + 1);
-				retval = savetif(dest, destfit, 16, NULL, com.pref.copyright, TRUE);
+				gchar *astro_tiff = AstroTiff_build_header(destfit);
+				retval = savetif(dest, destfit, 16, astro_tiff, com.pref.copyright, args->tiff_compression, TRUE, TRUE);
+				g_free(astro_tiff);
 				break;
 #endif
 			case EXPORT_SER:
@@ -488,6 +498,10 @@ static gpointer export_sequence(gpointer ptr) {
 				retval = mp4_add_frame(mp4_file, destfit);
 				break;
 #endif
+			default:
+				fprintf(stderr, "Case not handled, should not happen.\n");
+				seqwriter_release_memory();
+				goto free_and_reset_progress_bar;
 		}
 		if (retval) {
 			seqwriter_release_memory();
@@ -579,7 +593,6 @@ void on_buttonExportSeq_clicked(GtkButton *button, gpointer user_data) {
 	GtkEntry *entry = GTK_ENTRY(lookup_widget("entryExportSeq"));
 	const char *bname = gtk_entry_get_text(entry);
 	gboolean normalize = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget("exportNormalize")));
-	struct exportseq_args *args;
 
 	if (bname[0] == '\0') {
 		widget_set_class(GTK_WIDGET(entry), "warning", "");
@@ -587,7 +600,17 @@ void on_buttonExportSeq_clicked(GtkButton *button, gpointer user_data) {
 	}
 	if (selected == -1) return;
 
-	args = malloc(sizeof(struct exportseq_args));
+	// checking regdata is absent, or if present, is only shift
+	if (!test_regdata_is_valid_and_shift(&com.seq, get_registration_layer(&com.seq))) {
+		int confirm = siril_confirm_dialog(_("Registration data found"),
+			_("Export has detected registration data with more than simple shifts.\n"
+			"Normally, you should apply existing registration before exporting."),
+			_("Export anyway"));
+		if (!confirm)
+			return;
+	}
+
+	struct exportseq_args *args = malloc(sizeof(struct exportseq_args));
 	args->seq = &com.seq;
 	get_sequence_filtering_from_gui(&args->filtering_criterion, &args->filtering_parameter);
 	args->basename = g_str_to_ascii(bname, NULL);
@@ -604,7 +627,7 @@ void on_buttonExportSeq_clicked(GtkButton *button, gpointer user_data) {
 		if (args->film_fps <= 0) args->film_fps = 1;
 	}
 	if (args->output == EXPORT_MP4 || args->output == EXPORT_MP4_H265 || args->output == EXPORT_WEBM_VP9) {
-		GtkAdjustment *adjQual = GTK_ADJUSTMENT(gtk_builder_get_object(gui.builder,"adjustment3"));
+		GtkAdjustment *adjQual = GTK_ADJUSTMENT(gtk_builder_get_object(gui.builder,"adjustmentqualscale"));
 		GtkToggleButton *checkResize = GTK_TOGGLE_BUTTON(lookup_widget("checkAviResize"));
 		args->film_quality = (int)gtk_adjustment_get_value(adjQual);
 		args->resample = gtk_toggle_button_get_active(checkResize);
@@ -626,6 +649,11 @@ void on_buttonExportSeq_clicked(GtkButton *button, gpointer user_data) {
 	else if (args->output == EXPORT_FITS || args->output == EXPORT_TIFF) {
 		// add a trailing '_' for multiple-files sequences
 		args->basename = format_basename(args->basename, TRUE);
+		if (args->output == EXPORT_TIFF) {
+#ifdef HAVE_LIBTIFF
+			args->tiff_compression = get_tiff_compression();
+#endif
+		}
 	}
 	// Display a useful warning because I always forget to remove selection
 	if (args->crop) {
