@@ -731,6 +731,19 @@ int ser_metadata_as_fits(struct ser_struct *ser_file, fits *fit) {
 	return SER_OK;
 }
 
+static const gchar *flip_bayer_pattern(const gchar *old_pattern) {
+	if (!strcmp(old_pattern, "RGGB"))
+		return "GBRG";
+	else if (!strcmp(old_pattern, "GBRG"))
+		return "RGGB";
+	else if (!strcmp(old_pattern, "BGGR"))
+		return "GRBG";
+	else if (!strcmp(old_pattern, "GRBG"))
+		return "BGGR";
+	else
+		return NULL;
+}
+
 /* reads a frame on an already opened SER sequence.
  * frame number starts at 0 */
 int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit, gboolean force_float, gboolean open_debayer) {
@@ -809,9 +822,6 @@ int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit, gboolea
 		pattern = filter_pattern[com.pref.debayer.bayer_pattern];
 		type_ser = get_cfa_pattern_index_from_string(pattern) + 8;
 	}
-	if (pattern) {
-		strncpy(fit->bayer_pattern, pattern, 70); // fixed char* length FLEN == 71, leave 1 char for the NULL
-	}
 
 	switch (type_ser) {
 	case SER_MONO:
@@ -910,8 +920,13 @@ int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit, gboolea
 		fit_replace_buffer(fit, newbuf, DATA_FLOAT);
 	}
 
-	fits_flip_top_to_bottom(fit);
-	fit->top_down = FALSE;
+	if (pattern) {
+		pattern = flip_bayer_pattern(pattern);
+		// No need to inform the user as the FITS header details for a sequence frame are not accessible
+		strncpy(fit->bayer_pattern, pattern, 70); // fixed char* length FLEN == 71, leave 1 char for the NULL
+	}
+	fits_flip_top_to_bottom(fit); // convert top-down ser to bottom-up fits roworder
+	fit->top_down = TRUE;
 	snprintf(fit->row_order, FLEN_VALUE, "BOTTOM-UP");
 	return SER_OK;
 }
@@ -1173,6 +1188,12 @@ static int ser_write_frame_from_fit_internal(struct ser_struct *ser_file, fits *
 	BYTE *data8 = NULL;	// for 8-bit files
 	WORD *data16 = NULL;	// for 16-bit files
 
+	// return bottom-up fits to top-down ser row_order (not if the image is already top-down)
+	if (fit->top_down) {
+		snprintf(fit->bayer_pattern, FLEN_VALUE, "%s", flip_bayer_pattern(fit->bayer_pattern));
+	}
+	fits_flip_top_to_bottom(fit);
+
 	if (!ser_file || ser_file->file == NULL || !fit)
 		return SER_GENERIC_ERROR;
 	if (ser_file->number_of_planes == 0) {
