@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -25,11 +25,13 @@
 
 #include "core/siril.h"
 #include "core/OS_utils.h"
+#include "core/siril_log.h"
 #include "algos/statistics.h"
 #include "algos/annotate.h"
 #include "algos/ccd-inspector.h"
 #include "algos/background_extraction.h"
 #include "algos/astrometry_solver.h"
+#include "algos/demosaicing.h"
 #include "gui/image_interactions.h"
 #include "gui/image_display.h"
 #include "gui/utils.h"
@@ -50,6 +52,7 @@
 #include "core/undo.h"
 #include "core/processing.h"
 #include "compositing/compositing.h"
+#include "registration/registration.h"
 
 /* Closes and frees resources attached to the single image opened in gfit.
  * If a sequence is loaded and one of its images is displayed, nothing is done.
@@ -71,11 +74,13 @@ static gboolean free_image_data_idle(gpointer p) {
 	delete_selected_area();
 	reset_plot(); // clear existing plot if any
 	siril_close_preview_dialogs();
+	/* It is better to close all other dialog. Indeed, some dialog are not compatible with all images */
 	display_filename();
 	update_zoom_label();
 	update_display_fwhm();
 	adjust_sellabel();
 	update_MenuItem();
+	reset_3stars();
 	close_tab();	// close Green and Blue tabs
 
 	GtkComboBox *binning = GTK_COMBO_BOX(gtk_builder_get_object(gui.builder, "combobinning"));
@@ -87,16 +92,16 @@ static gboolean free_image_data_idle(gpointer p) {
 	g_signal_handlers_block_by_func(pitchX_entry, on_pitchX_entry_changed, NULL);
 	g_signal_handlers_block_by_func(pitchY_entry, on_pitchY_entry_changed, NULL);
 	g_signal_handlers_block_by_func(binning, on_combobinning_changed, NULL);
-	clear_stars_list();
+	clear_stars_list(TRUE);
 	clear_sampling_setting_box();	// clear focal and pixel pitch info
 	free_background_sample_list(com.grad_samples);
 	com.grad_samples = NULL;
-	g_slist_free_full(com.found_object, (GDestroyNotify)free_catalogue_object);
+	g_slist_free(com.found_object);
 	com.found_object = NULL;
 	reset_display_offset();
 	reset_zoom_default();
-	free(com.qphot);
-	com.qphot = NULL;
+	free(gui.qphot);
+	gui.qphot = NULL;
 	clear_sensor_tilt();
 	g_signal_handlers_unblock_by_func(focal_entry, on_focal_entry_changed, NULL);
 	g_signal_handlers_unblock_by_func(pitchX_entry, on_pitchX_entry_changed, NULL);
@@ -148,9 +153,8 @@ void free_image_data() {
 	 * shouldn't it be used here instead of gfit? */
 	if (!single_image_is_loaded() && sequence_is_loaded())
 		save_stats_from_fit(&gfit, &com.seq, com.seq.current);
-	clearfits(&gfit);
 
-	clear_histograms();
+	invalidate_gfit_histogram();
 
 	if (com.uniq) {
 		free(com.uniq->filename);
@@ -162,11 +166,12 @@ void free_image_data() {
 
 	if (!com.headless)
 		free_image_data_gui();
+
+	clearfits(&gfit);
 }
 
 static gboolean end_read_single_image(gpointer p) {
 	set_GUI_CAMERA();
-	set_GUI_photometry();
 	return FALSE;
 }
 
@@ -191,7 +196,7 @@ int read_single_image(const char *filename, fits *dest, char **realname_out,
 
 	retval = stat_file(filename, &imagetype, &realname);
 	if (retval) {
-		siril_log_message(_("Error opening image %s: file not found or not supported.\n"), filename);
+		siril_log_color_message(_("Error opening image %s: file not found or not supported.\n"), "red", filename);
 		free(realname);
 		return 1;
 	}
@@ -201,7 +206,7 @@ int read_single_image(const char *filename, fits *dest, char **realname_out,
 			retval = read_single_sequence(realname, imagetype);
 			single_sequence = TRUE;
 		} else {
-			siril_log_message(_("Cannot open a sequence from here\n"));
+			siril_log_color_message(_("Cannot open a sequence from here\n"), "red");
 			free(realname);
 			return 1;
 		}
@@ -214,12 +219,13 @@ int read_single_image(const char *filename, fits *dest, char **realname_out,
 		*is_sequence = single_sequence;
 	}
 	if (retval && retval != OPEN_IMAGE_CANCEL)
-		siril_log_message(_("Opening %s failed.\n"), realname);
+		siril_log_color_message(_("Opening %s failed.\n"), "red", realname);
 	if (realname_out)
 		*realname_out = realname;
 	else
 		free(realname);
-	gui.filter = (int) imagetype;
+	gui.file_ext_filter = (int) imagetype;
+	update_gain_from_gfit();
 	siril_add_idle(end_read_single_image, NULL);
 	return retval;
 }
@@ -249,20 +255,30 @@ int create_uniq_from_gfit(char *filename, gboolean exists) {
  * image, gfit.
  */
 int open_single_image(const char* filename) {
-	int retval;
-	char *realname;
+	int retval = 0;
+	char *realname = NULL;
 	gboolean is_single_sequence;
 
-	/* first, close everything */
-	close_sequence(FALSE);	// closing a sequence if loaded
-	close_single_image();	// close the previous image and free resources
+	/* Check we aren't running a processing thread otherwise it will clobber gfit
+	 * when it finishes and cause a segfault.
+	 */
+	if ((retval = get_thread_run())) {
+		siril_log_message(_("Cannot open another file while the processing thread is still operating on the current one!\n"));
+	}
 
-	/* open the new file */
-	retval = read_single_image(filename, &gfit, &realname, TRUE, &is_single_sequence, TRUE, FALSE);
+	/* first, close everything */
+	if (!retval) {
+		close_sequence(FALSE);	// closing a sequence if loaded
+		close_single_image();	// close the previous image and free resources
+
+		/* open the new file */
+		retval = read_single_image(filename, &gfit, &realname, TRUE, &is_single_sequence, TRUE, FALSE);
+	}
 	if (retval) {
 		siril_message_dialog(GTK_MESSAGE_ERROR, _("Error opening file"),
 				_("There was an error when opening this image. "
 						"See the log for more information."));
+		free(realname);
 		return 1;
 	}
 
@@ -283,6 +299,8 @@ int open_single_image(const char* filename) {
 				execute_idle_and_wait_for_it(end_open_single_image, NULL);
 			else end_open_single_image(NULL);
 		}
+	} else {
+		free(realname);
 	}
 	return retval;
 }
@@ -314,6 +332,8 @@ void open_single_image_from_gfit() {
 	update_MenuItem();
 
 	close_tab();
+	init_right_tab();
+
 	update_gfit_histogram_if_needed();
 	redraw(REMAP_ALL);
 }
@@ -329,7 +349,7 @@ int image_find_minmax(fits *fit) {
 	for (layer = 0; layer < fit->naxes[2]; ++layer) {
 		// calling statistics() saves stats in the fit already, we don't need
 		// to use the returned handle
-		free_stats(statistics(NULL, -1, fit, layer, NULL, STATS_MINMAX, TRUE));
+		free_stats(statistics(NULL, -1, fit, layer, NULL, STATS_MINMAX, SINGLE_THREADED));
 		if (!fit->stats || !fit->stats[layer])
 			return -1;
 		fit->maxi = max(fit->maxi, fit->stats[layer]->max);
@@ -338,27 +358,6 @@ int image_find_minmax(fits *fit) {
 	return 0;
 }
 
-static int fit_get_minmax(fits *fit, int layer) {
-	// calling statistics() saves stats in the fit already, we don't need
-	// to use the returned handle
-	free_stats(statistics(NULL, -1, fit, layer, NULL, STATS_MINMAX, FALSE));
-	if (!fit->stats[layer])
-		return -1;
-	return 0;
-}
-
-double fit_get_max(fits *fit, int layer) {
-	if (fit_get_minmax(fit, layer))
-		return -1.0;
-	return fit->stats[layer]->max;
-}
-
-double fit_get_min(fits *fit, int layer) {
-	if (fit_get_minmax(fit, layer))
-		return -1.0;
-	return fit->stats[layer]->min;
-}
-
 static void fit_lohi_to_layers(fits *fit, double lo, double hi) {
 	if (fit->type == DATA_USHORT) {
 		gui.lo = (WORD)lo;
@@ -389,18 +388,59 @@ void init_layers_hi_and_lo_values(sliders_mode force_minmax) {
 	}
 }
 
+int single_image_is_loaded() {
+	return (com.uniq != NULL && com.uniq->nb_layers > 0);
+}
+
+/**************** updating the single image *******************/
+
 /* was level_adjust, to call when gfit changed and need min/max to be recomputed. */
+/* deprecated, use notify_gfit_modified() instead */
 void adjust_cutoff_from_updated_gfit() {
 	invalidate_stats_from_fit(&gfit);
 	invalidate_gfit_histogram();
 	if (!com.script) {
-		compute_histo_for_gfit();
+		update_gfit_histogram_if_needed();
 		init_layers_hi_and_lo_values(gui.sliders);
 		set_cutoff_sliders_values();
 	}
 }
 
-int single_image_is_loaded() {
-	return (com.uniq != NULL && com.uniq->nb_layers > 0);
+/* generic idle function for end of operation on gfit */
+static gboolean end_gfit_operation() {
+	// this function should not contain anything required by the execution
+	// of the operation because it won't be run in headless
+
+	siril_debug_print("end of gfit operation - idle function\n");
+	stop_processing_thread();
+
+	update_gfit_histogram_if_needed();
+
+	/* update bit depth selector */
+	set_precision_switch();
+
+	/* update display of gfit name (useful if it changes) */
+	adjust_sellabel();
+	display_filename();
+
+	// compute new min and max if needed for display and update sliders
+	init_layers_hi_and_lo_values(gui.sliders);
+	set_cutoff_sliders_values();
+
+	redraw(REMAP_ALL);	// queues a redraw if !com.script
+	redraw_previews();	// queues redraws if !com.script
+
+	set_cursor_waiting(FALSE); // called from current thread if !com.script, idle else
+	return FALSE;
+}
+
+/* to be called after each operation that modifies the content of gfit, at the
+ * end of a processing operation, not for previews */
+void notify_gfit_modified() {
+	siril_debug_print("end of gfit operation\n");
+	invalidate_stats_from_fit(&gfit);
+	invalidate_gfit_histogram();
+
+	siril_add_idle(end_gfit_operation, NULL);
 }
 
