@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -22,7 +22,10 @@
 #include "core/proto.h"
 #include "core/processing.h"
 #include "core/OS_utils.h"
-
+#include "core/initfile.h"
+#include "core/siril_log.h"
+#include "algos/siril_wcs.h"
+#include "algos/statistics.h"
 #include "gui/utils.h"
 #include "gui/dialogs.h"
 #include "gui/dialog_preview.h"
@@ -63,9 +66,12 @@ typedef struct {
 static const _pm_op_func functions[] = {
     { "abs",   "abs ( x )",                             N_("Absolute value of x.")                    },
     { "acos",  "acos ( x )",                            N_("Arc cosine of x.")                        },
+    { "acosh", "acosh ( x )",                           N_("Hyperbolic arc cosine of x.")             },
     { "asin",  "asin ( x )",                            N_("Arc sine of x.")                          },
+    { "asinh", "asinh ( x )",                           N_("Hyperbolic arc sine of x.")               },
     { "atan",  "atan ( x )",                            N_("Arc tangent of x.")                       },
     { "atan2", "atan2 ( y, x )",                        N_("Arc tangent of y/x.")                     },
+    { "atanh", "atanh ( x )",                           N_("Hyperbolic arc tangent of x.")            },
     { "ceil",  "ceil ( x )",                            N_("Round x upwards to the nearest integer.") },
     { "cos",   "cos ( x )",                             N_("Cosine of x.")                            },
     { "cosh",  "cosh ( x )",                            N_("Hyperbolic cosine of x.")                 },
@@ -73,8 +79,8 @@ static const _pm_op_func functions[] = {
     { "exp",   "exp ( x )",                             N_("Exponential function.")                   },
     { "fac",   "fac( x )",                              N_("Factorial function.")                     },
     { "iif",   "iif( cond, expr_true, expr_false )",    N_("Conditional function (or inline if function).\n"
-										"\nReturns <i>expr_true</i> if <i>cond</i> evaluates to nonzero."
-										"\nReturns <i>expr_false</i> if <i>cond</i> evaluates to zero.") },
+                                        "\nReturns <i>expr_true</i> if <i>cond</i> evaluates to nonzero."
+                                        "\nReturns <i>expr_false</i> if <i>cond</i> evaluates to zero.") },
     { "floor", "floor ( x )",                           N_("Highest integer less than or equal to x.")},
     { "ln",    "ln ( x )",                              N_("Natural logarithm of x.")                 },
     { "log",   "log ( x )",                             N_("Base-10 logarithm of x.")                 },
@@ -82,10 +88,15 @@ static const _pm_op_func functions[] = {
     { "log2",  "log2 ( x )",                            N_("Base-2 logarithm of x.")                  },
     { "max",   "max ( x, y )",                          N_("Maximum function.")                       },
     { "min",   "min ( x, y )",                          N_("Minimum function.")                       },
+    { "mtf",   "mtf ( m, x )",                          N_("Midtones Transfer Function (MTF) of x for a midtones balance parameter m in the [0, 1] range.") },
     { "ncr",   "ncr ( x, y )",                          N_("Combinations function.")                  },
     { "npr",   "npr ( x, y )",                          N_("Permutations function.")                  },
     { "pi",    "pi",                                    N_("The constant \u03c0=3.141592...")         },
     { "pow",   "pow ( x, y )",                          N_("Exponentiation function.")                },
+    { "sign",  "sign ( x )",                            N_("Sign of x:\n"
+                                                                                "\n\t +1 if x &gt; 0"
+                                                                                "\n\t −1 if x &lt; 0"
+                                                                                "\n\t  0 if x = 0.")  },
     { "sin",   "sin ( x )",                             N_("Sine of x.")                              },
     { "sinh",  "sinh ( x )",                            N_("Hyperbolic sine of x.")                   },
     { "sqrt",  "sqrt ( x )",                            N_("Square root of x.")                       },
@@ -94,6 +105,22 @@ static const _pm_op_func functions[] = {
     { "trunc", "trunc ( x )",                           N_("Truncated integer part of x.")            }
 };
 
+static const _pm_op_func image_functions[] = {
+	{ "adev",     "adev ( Image )",                    N_("Average absolute deviation of the image.")  },
+	{ "bwmv",     "bwmv ( Image )",                    N_("Biweight midvariance of the image.")        },
+	{ "height",   "height ( Image )",                  N_("Height in pixels of the specified image.")  },
+	{ "mad",      "mad ( Image )",                     N_("Median absolute deviation of the image.")   },
+	{ "max",      "max ( Image )",                     N_("Pixel maximum of the image.")               },
+	{ "mdev",     "mdev ( Image )",                    N_("Median absolute deviation of the image.")   },
+	{ "mean",     "mean ( Image )",                    N_("Mean of the image.")                        },
+	{ "med",      "med ( Image )",                     N_("Median of the image.")                      },
+	{ "median",   "median ( Image )",                  N_("Median of the image.")                      },
+	{ "min",      "min ( Image )",                     N_("Pixel minimum of the image.")               },
+	{ "noise",    "noise ( Image )",                   N_("Estimation of Gaussian noise in the image.")},
+	{ "sdev",     "sdev ( Image )",                    N_("Standard deviation of the image.")          },
+	{ "width",    "width ( Image )",                   N_("Width in pixels of the specified image.")   }
+};
+
 static const _pm_op_func operators[] = {
     { "~",   "~x",                             N_("Pixel Inversion operator.")                        },
     { "-",   "-x",                             N_("Unary Minus operator (sign change).")              },
@@ -107,8 +134,8 @@ static const _pm_op_func operators[] = {
     { "-",   "x - y",                          N_("Subtraction operator.")                            },
     { "<",   "x &lt; y",                       N_("Less Than relational operator.")                   },
     { "<=",  "x &lt;= y",                      N_("Less Than Or Equal relational operator.")          },
-    { ">",   "x > y",                          N_("Greater Than relational operator.")                },
-    { ">=",  "x >= y",                         N_("Greater Than Or Equal relational operator.")       },
+    { ">",   "x &gt; y",                       N_("Greater Than relational operator.")                },
+    { ">=",  "x &gt;= y",                      N_("Greater Than Or Equal relational operator.")       },
     { "==",  "x == y",                         N_("Equal To relational operator.")                    },
     { "!=",  "x != y",                         N_("Not Equal To relational operator.")                },
     { "&&",  "x &amp;&amp; y",                 N_("Logical AND operator.")                            },
@@ -116,6 +143,7 @@ static const _pm_op_func operators[] = {
 };
 
 #define MAX_FUNCTIONS G_N_ELEMENTS(functions)
+#define MAX_IMAGE_FUNCTIONS G_N_ELEMENTS(image_functions)
 #define MAX_OPERATORS G_N_ELEMENTS(operators)
 
 
@@ -134,16 +162,20 @@ enum {
 };
 
 static fits var_fit[MAX_IMAGES] = { 0 };
+static gboolean var_fit_mask[MAX_IMAGES] = { 0 };
 
 static GtkListStore *pixel_math_list_store = NULL;
 static GtkListStore *pixel_math_list_store_functions = NULL;
 static GtkListStore *pixel_math_list_store_operators = NULL;
+static GtkListStore *pixel_math_list_store_presets = NULL;
 static GtkTreeView *pixel_math_tree_view = NULL;
 static GtkTreeView *pixel_math_treeview_functions = NULL;
 static GtkTreeView *pixel_math_treeview_operators = NULL;
+static GtkTreeView *pixel_math_treeview_presets = NULL;
 static GtkTreeModel *pixel_math_tree_model = NULL;
 static GtkTreeModel *pixel_math_tree_model_functions = NULL;
 static GtkTreeModel *pixel_math_tree_model_operators = NULL;
+static GtkTreeModel *pixel_math_tree_model_presets = NULL;
 static GtkLabel *pixel_math_status_bar = NULL;
 static GtkLabel *pixel_math_status_bar2 = NULL;
 static GtkEntry *pixel_math_entry_r = NULL;
@@ -162,22 +194,32 @@ static void init_widgets() {
 		pixel_math_entry_b = GTK_ENTRY(lookup_widget("pixel_math_entry_b"));
 		pixel_math_treeview_functions = GTK_TREE_VIEW(gtk_builder_get_object(gui.builder, "pixel_math_treeview_functions"));
 		pixel_math_treeview_operators = GTK_TREE_VIEW(gtk_builder_get_object(gui.builder, "pixel_math_treeview_operators"));
+		pixel_math_treeview_presets = GTK_TREE_VIEW(gtk_builder_get_object(gui.builder, "pixel_math_treeview_presets"));
 		pixel_math_tree_model_functions = gtk_tree_view_get_model(pixel_math_treeview_functions);
 		pixel_math_tree_model_operators = gtk_tree_view_get_model(pixel_math_treeview_operators);
+		pixel_math_tree_model_presets = gtk_tree_view_get_model(pixel_math_treeview_presets);
 		pixel_math_list_store_functions = GTK_LIST_STORE(gtk_builder_get_object(gui.builder, "pixel_math_liststore_functions"));
 		pixel_math_list_store_operators = GTK_LIST_STORE(gtk_builder_get_object(gui.builder, "pixel_math_liststore_operators"));
+		pixel_math_list_store_presets = GTK_LIST_STORE(gtk_builder_get_object(gui.builder, "pixel_math_liststore_presets"));
 
 #if GTK_CHECK_VERSION(3, 22, 0)
 		gtk_scrolled_window_set_propagate_natural_width(GTK_SCROLLED_WINDOW(lookup_widget("pixel_math_scrolled_functions")), TRUE);
 		gtk_scrolled_window_set_propagate_natural_width(GTK_SCROLLED_WINDOW(lookup_widget("pixel_math_scrolled_operators")), TRUE);
 #endif
 
+		/* Due to a glade bug, this property is often removed, lets code it */
+		GtkTreeSelection *selection = GTK_TREE_SELECTION(gtk_builder_get_object(gui.builder, "pixel_math_selection"));
+		gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
+		selection = GTK_TREE_SELECTION(gtk_builder_get_object(gui.builder, "pixel_math_presets_selection"));
+		gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
+
 	}
 	g_assert(pixel_math_tree_view);
 	g_assert(pixel_math_tree_model);
 	g_assert(pixel_math_list_store);
 	g_assert(pixel_math_list_store_functions);
 	g_assert(pixel_math_list_store_operators);
+	g_assert(pixel_math_list_store_presets);
 	g_assert(pixel_math_status_bar);
 	g_assert(pixel_math_status_bar2);
 	g_assert(pixel_math_entry_r);
@@ -187,9 +229,27 @@ static void init_widgets() {
 	g_assert(pixel_math_tree_model_functions);
 	g_assert(pixel_math_treeview_operators);
 	g_assert(pixel_math_tree_model_operators);
+	g_assert(pixel_math_treeview_presets);
+	g_assert(pixel_math_tree_model_presets);
 
 }
 
+static int FnCompare_functions(const void *v1, const void *v2) {
+	const _pm_op_func *i1 = v1;
+	const _pm_op_func *i2 = v2;
+
+	return (g_strcmp0(i1->name, i2->name));
+}
+
+static _pm_op_func *concat_functions() {
+	_pm_op_func *all_functions = malloc((MAX_FUNCTIONS + MAX_IMAGE_FUNCTIONS) * sizeof(_pm_op_func));
+	memcpy(all_functions, functions, MAX_FUNCTIONS * sizeof(_pm_op_func));
+	memcpy(all_functions + MAX_FUNCTIONS, image_functions, MAX_IMAGE_FUNCTIONS * sizeof(_pm_op_func));
+
+	qsort(all_functions, MAX_FUNCTIONS + MAX_IMAGE_FUNCTIONS, sizeof(_pm_op_func), FnCompare_functions);
+
+	return all_functions;
+}
 
 static gchar* get_pixel_math_expression1() {
 	init_widgets();
@@ -322,6 +382,12 @@ static int get_pixel_math_operators_number_of_rows(){
 	else return 0;
 }
 
+static int get_pixel_math_presets_number_of_rows(){
+	if (GTK_IS_TREE_MODEL(pixel_math_list_store_presets))
+		return gtk_tree_model_iter_n_children(GTK_TREE_MODEL(pixel_math_list_store_presets), NULL);
+	else return 0;
+}
+
 static const gchar *get_pixel_math_var_name(int i) {
 	GtkTreeIter iter;
 	GValue value = G_VALUE_INIT;
@@ -367,20 +433,206 @@ static const gchar *get_operator_name(int i) {
 	return NULL;
 }
 
+static const gchar *get_preset_expr(int i) {
+	GtkTreeIter iter;
+	GValue value = G_VALUE_INIT;
+
+	init_widgets();
+
+	GtkTreePath *path = gtk_tree_path_new_from_indices(i, -1);
+	if (gtk_tree_model_get_iter(pixel_math_tree_model_presets, &iter, path)) {
+		gtk_tree_model_get_value(pixel_math_tree_model_presets, &iter, 0, &value);
+		return g_value_get_string(&value);
+	}
+
+	return NULL;
+}
+
 static gboolean is_pm_use_rgb_button_checked() {
 	return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget("pm_use_rgb_button")));
 }
 
+static gboolean is_pm_rescale_checked() {
+	return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget("rescale_pm_button")));
+}
+
+static float get_min_rescale_value() {
+	return (float) gtk_spin_button_get_value(GTK_SPIN_BUTTON(lookup_widget("spin_pm_low")));
+}
+
+static float get_max_rescale_value() {
+	return (float) gtk_spin_button_get_value(GTK_SPIN_BUTTON(lookup_widget("spin_pm_high")));
+}
+
+static void update_metadata(fits *fit) {
+	fits **f = malloc((MAX_IMAGES + 1) * sizeof(fits *));
+	int j = 0;
+	for (int i = 0; i < MAX_IMAGES ; i++)
+		if (var_fit[i].rx > 0 && var_fit_mask[i])
+			f[j++] = &var_fit[i];
+	f[j] = NULL;
+
+	if (!f[0])
+		// if no fit used (only constants),
+		// we copy the metadata from first image of the list
+		copy_fits_metadata(var_fit, fit);
+	else
+		merge_fits_headers_to_result2(fit, f);
+	load_WCS_from_memory(fit);
+	free(f);
+}
+
+static gchar *parse_image_functions(gpointer p, int idx, int c) {
+	struct pixel_math_data *args = (struct pixel_math_data *)p;
+
+	gchar *expression;
+	gchar **image = args->varname;
+	int nb_images = args->nb_rows;
+	switch(idx) {
+	default:
+	case 1:
+		expression = args->expression1;
+		break;
+	case 2:
+		expression = args->expression2;
+		break;
+	case 3:
+		expression = args->expression3;
+		break;
+	}
+	if (!expression) return expression;
+	for (int i = 0; i < MAX_IMAGE_FUNCTIONS; i++) {
+		/* First we need to find each occurrences of functions
+		 * Then if args->varname[i] is inside function parameters
+		 * Then apply statistics on args->varname[i]
+		 * Then transform it to numerical expression
+		 */
+		const gchar *function = image_functions[i].name;
+		int len = strlen(function);
+		int total_len = strlen(expression);
+
+		for (int pos = 0; pos < total_len - len; pos++) {
+			// Match word at current position
+			int found = 1;
+			for (int j = 0; j < len; j++) {
+				// If word is not matched
+				if (expression[pos + j] != function[j]) {
+					found = 0;
+					break;
+				}
+			}
+
+			// If word have been found then print found message
+			if (found == 1) {
+				if (expression[pos + len] == '(') {
+					for (int j = 0; j < nb_images; j++) {
+						int len2 = strlen(image[j]);
+						if (expression[pos + len + len2 + 1] == ')') {
+							// possible match
+							gchar *str = malloc(len2 + 1);
+							g_strlcpy(str, expression + pos + len + 1, len2 + 1);
+							if (!g_strcmp0(str, image[j])) {
+								gchar *replace = NULL;
+								double median = 0.0, mean = 0.0, min = 0.0,
+										max = 0.0, noise = 0.0, adev = 0.0,
+										bwmv = 0.0, mad = 0.0, sdev = 0.0;
+								double w = 0.0, h = 0.0;
+								imstats *stats = statistics(NULL, -1, &var_fit[j], c, NULL, STATS_MAIN, MULTI_THREADED);
+								if (!stats) return expression;
+
+								median = stats->median;
+								mean = stats->mean;
+								min = stats->min;
+								max = stats->max;
+								noise = stats->bgnoise;
+								adev = stats->avgDev;
+								bwmv = stats->sqrtbwmv * stats->sqrtbwmv;
+								mad = stats->mad;
+								sdev = stats->sigma;
+
+								w = (double) var_fit[j].rx;
+								h = (double) var_fit[j].ry;
+
+								free_stats(stats);
+
+								if (!g_strcmp0(function, "mean")) {
+									replace = g_strdup_printf("%g", mean);
+								} else if ((!g_strcmp0(function, "med") || !(g_strcmp0(function, "median")))) {
+									replace = g_strdup_printf("%g", median);
+								} else if (!g_strcmp0(function, "min")) {
+									replace = g_strdup_printf("%g", min);
+								} else if (!g_strcmp0(function, "max")) {
+									replace = g_strdup_printf("%g", max);
+								} else if (!g_strcmp0(function, "noise")) {
+									replace = g_strdup_printf("%g", noise);
+								} else if (!g_strcmp0(function, "adev")) {
+									replace = g_strdup_printf("%g", adev);
+								} else if (!g_strcmp0(function, "bwmv")) {
+									replace = g_strdup_printf("%g", bwmv);
+								} else if ((!g_strcmp0(function, "mad") || !(g_strcmp0(function, "mdev")))) {
+									replace = g_strdup_printf("%g", mad);
+								} else if (!g_strcmp0(function, "sdev")) {
+									replace = g_strdup_printf("%g", sdev);
+								} else if ((!g_strcmp0(function, "width") || !(g_strcmp0(function, "w")))) {
+									replace = g_strdup_printf("%g", w);
+								} else if ((!g_strcmp0(function, "height") || !(g_strcmp0(function, "h")))) {
+									replace = g_strdup_printf("%g", h);
+								}
+								if (replace) {
+									GString *string = g_string_new(expression);
+									g_string_erase(string, pos, len + len2 + 2);
+									g_string_insert(string, pos, replace);
+									g_free(expression);
+									expression = g_string_free(string, FALSE);
+									total_len = strlen(expression);
+									pos = pos + strlen(replace);
+									g_free(replace);
+									siril_debug_print("Expression%d: %s\n", c, expression);
+								}
+							}
+							free(str);
+						}
+					}
+				}
+			}
+		}
+	}
+	for (int j = 0; j < nb_images; j++) {
+		gchar *test =  g_strrstr(expression, image[j]);
+		if (test) {
+			var_fit_mask[j] = TRUE;
+			siril_debug_print("found image name %s in the expression %s\n", image[j], expression);
+		}
+	}
+	return expression;
+}
 gpointer apply_pixel_math_operation(gpointer p) {
 	struct pixel_math_data *args = (struct pixel_math_data *)p;
 
+	te_expr *n1 = NULL, *n2 = NULL, *n3 = NULL;
 	fits *fit = args->fit;
 	int nb_rows = args->nb_rows;
 	gboolean failed = FALSE;
 	args->ret = 0;
+	float maximum = -FLT_MAX;
+	float minimum = +FLT_MAX;
+
+	if (args->single_rgb && args->fit->naxes[2] > 1) {
+		args->expression2 = g_strdup(args->expression1);
+		args->expression3 = g_strdup(args->expression1);
+
+		args->expression1 = parse_image_functions(args, 1, RLAYER);
+		args->expression2 = parse_image_functions(args, 2, GLAYER);
+		args->expression3 = parse_image_functions(args, 3, BLAYER);
+	} else {
+		args->expression1 = parse_image_functions(args, 1, RLAYER);
+		args->expression2 = parse_image_functions(args, 2, RLAYER);
+		args->expression3 = parse_image_functions(args, 3, RLAYER);
+	}
+
 
 #ifdef _OPENMP
-#pragma omp parallel num_threads(com.max_thread)
+#pragma omp parallel num_threads(com.max_thread) firstprivate(n1,n2,n3)
 #endif
 	{
 		// we build the expressions in parallel because tr_eval() is not thread-safe
@@ -396,7 +648,6 @@ gpointer apply_pixel_math_operation(gpointer p) {
 			vars[i].context = NULL;
 			vars[i].type = 0;
 		}
-		te_expr *n1 = NULL, *n2 = NULL, *n3 = NULL;
 		int err = 0;
 		n1 = te_compile(args->expression1, vars, nb_rows, &err);
 		if (!n1) {
@@ -408,7 +659,7 @@ gpointer apply_pixel_math_operation(gpointer p) {
 			goto failure;
 		}
 
-		if (!args->single_rgb) {
+		if (args->expression2) {
 			n2 = te_compile(args->expression2, vars, nb_rows, &err);
 			if (!n2) {
 #ifdef _OPENMP
@@ -430,7 +681,7 @@ gpointer apply_pixel_math_operation(gpointer p) {
 			}
 		}
 #ifdef _OPENMP
-#pragma omp for schedule(static)
+#pragma omp for schedule(static) reduction(max:maximum) reduction(min:minimum)
 #endif
 		for (size_t px = 0; px < var_fit[0].naxes[0] * var_fit[0].naxes[1] * var_fit[0].naxes[2]; px++) {
 			/* The variables can be changed here, and eval can be called as many
@@ -440,18 +691,32 @@ gpointer apply_pixel_math_operation(gpointer p) {
 				x[i] = var_fit[i].fdata[px];
 			}
 
-			if (args->single_rgb) {
-				fit->fdata[px] = (float) te_eval(n1);
-			} else {
+			if (!args->single_rgb) { // in that case var_fit[0].naxes[2] == 1, but we built RGB
 				fit->fpdata[RLAYER][px] = (float) te_eval(n1);
 				fit->fpdata[GLAYER][px] = (float) te_eval(n2);
 				fit->fpdata[BLAYER][px] = (float) te_eval(n3);
+
+				/* may not be used but (only if rescale) at least it is computed */
+				maximum = max(maximum, max(fit->fpdata[RLAYER][px], max(fit->fpdata[GLAYER][px], fit->fpdata[BLAYER][px])));
+				minimum = min(minimum, min(fit->fpdata[RLAYER][px], min(fit->fpdata[GLAYER][px], fit->fpdata[BLAYER][px])));
+			} else {
+				if (px < (var_fit[0].naxes[0] * var_fit[0].naxes[1])) {
+					fit->fdata[px] = (float) te_eval(n1);
+				} else if (px < 2 * (var_fit[0].naxes[0] * var_fit[0].naxes[1])) {
+					fit->fdata[px] = (float) te_eval(n2);
+				} else {
+					fit->fdata[px] = (float) te_eval(n3);
+				}
+
+				/* may not be used but (only if rescale) at least it is computed */
+				maximum = max(maximum, fit->fdata[px]);
+				minimum = min(minimum, fit->fdata[px]);
 			}
 		}
 
 failure: // failure before the eval loop
 		te_free(n1);
-		if (!args->single_rgb) {
+		if (args->expression2) {
 			te_free(n2);
 			te_free(n3);
 		}
@@ -459,12 +724,22 @@ failure: // failure before the eval loop
 		free(x);
 	} // end of parallel block
 
+	if (args->rescale) {
+#ifdef _OPENMP
+#pragma omp parallel for num_threads(com.max_thread) schedule(static)
+#endif
+		for (int i = 0; i < fit->rx * fit->ry * fit->naxes[2]; i++) {
+			fit->fdata[i] = (((args->max - args->min) * (fit->fdata[i] - minimum)) / (maximum - minimum)) + args->min;
+		}
+	}
+
 	if (failed)
 		args->ret = 1;
+	else update_metadata(args->fit);
 
 	/* free memory */
 	g_free(args->expression1);
-	if (args->single_rgb) {
+	if (args->expression2) {
 		g_free(args->expression2);
 		g_free(args->expression3);
 	}
@@ -600,7 +875,7 @@ static int parse_parameters(gchar **expression1, gchar **expression2, gchar **ex
 }
 
 int load_pm_var(const gchar *var, int index, int *w, int *h, int *c) {
-	if (index > MAX_IMAGES) {
+	if (index > MAX_IMAGES - 1) {
 		siril_log_message(_("A maximum of %d images can be used in a single expression.\n"), MAX_IMAGES);
 		return 1;
 	}
@@ -617,6 +892,7 @@ int load_pm_var(const gchar *var, int index, int *w, int *h, int *c) {
 void free_pm_var(int nb) {
 	for (int i = 0; i < nb; i++) {
 		clearfits(&var_fit[i]);
+		var_fit_mask[i] = FALSE;
 	}
 }
 
@@ -628,6 +904,9 @@ static int pixel_math_evaluate(gchar *expression1, gchar *expression2, gchar *ex
 	int channel = -1;
 
 	gboolean single_rgb = is_pm_use_rgb_button_checked();
+	gboolean rescale = is_pm_rescale_checked();
+	float min = get_min_rescale_value();
+	float max = get_max_rescale_value();
 
 	while (nb_rows < get_pixel_math_number_of_rows() && nb_rows < MAX_IMAGES) {
 		const gchar *path = get_pixel_math_var_paths(nb_rows);
@@ -664,12 +943,14 @@ static int pixel_math_evaluate(gchar *expression1, gchar *expression2, gchar *ex
 				_("Parameter symbols could not be parsed."));
 		return 1;
 	}
-	printf("[%s]\n", expression1);
 
 	args->expression1 = expression1;
 	args->expression2 = single_rgb ? NULL : expression2;
 	args->expression3 = single_rgb ? NULL : expression3;
 	args->single_rgb = single_rgb;
+	args->rescale = rescale;
+	args->min = min;
+	args->max = max;
 	args->fit = fit;
 	args->nb_rows = nb_rows;
 	args->ret = 0;
@@ -791,7 +1072,8 @@ static void select_image(int nb) {
 			_("_Cancel"), GTK_RESPONSE_CANCEL, _("_Open"), GTK_RESPONSE_ACCEPT,	NULL);
 	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), com.wd);
 	gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog), TRUE);
-	gtk_filter_add(GTK_FILE_CHOOSER(dialog), _("FITS Files (*.fit, *.fits, *.fts, *.fits.fz)"), FITS_EXTENSIONS);
+	gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(dialog), FALSE);
+	gtk_filter_add(GTK_FILE_CHOOSER(dialog), _("FITS Files (*.fit, *.fits, *.fts, *.fit.fz, *.fits.fz, *.fts.fz)"), FITS_EXTENSIONS);
 	siril_file_chooser_add_preview(GTK_FILE_CHOOSER(dialog), preview);
 
 	res = gtk_dialog_run(GTK_DIALOG(dialog));
@@ -803,7 +1085,7 @@ static void select_image(int nb) {
 		guint channel = 0;
 
 		GtkFileChooser *chooser = GTK_FILE_CHOOSER(dialog);
-		GSList *filenames = gtk_file_chooser_get_filenames(chooser);
+		GSList *filenames = siril_file_chooser_get_filenames(chooser);
 
 		for (l = filenames; l; l = l->next) {
 			char *filename;
@@ -812,8 +1094,9 @@ static void select_image(int nb) {
 			if (filename) {
 				gchar filter[FLEN_VALUE] = { 0 };
 				fits f = { 0 };
-				read_fits_metadata_from_path(filename, &f);
-				if (check_files_dimensions(&width, &height, &channel)) {
+				if (read_fits_metadata_from_path(filename, &f)) {
+					siril_log_color_message(_("Could not open file: %s\n"), "red", filename);
+				} else if (check_files_dimensions(&width, &height, &channel)) {
 					if (width != 0 && (channel != f.naxes[2] ||
 							width != f.naxes[0] ||
 							height != f.naxes[1])) {
@@ -981,20 +1264,25 @@ gboolean query_tooltip_tree_view_cb(GtkWidget *widget, gint x, gint y,
 	GtkTreeModel *model = gtk_tree_view_get_model(tree_view);
 	GtkTreePath *path = NULL;
 
+	_pm_op_func *all_functions = concat_functions();
+
 	char buffer[512];
 
 	if (!gtk_tree_view_get_tooltip_context(tree_view, &x, &y, keyboard_tip,
-			&model, &path, &iter))
+			&model, &path, &iter)) {
+		free(all_functions);
 		return FALSE;
+	}
 
 	gint real_index = get_real_index_from_index_in_list(model, &iter);
 
-	g_snprintf(buffer, 511, "<b>%s</b>\n\n%s", functions[real_index].prototype, _(functions[real_index].definition));
+	g_snprintf(buffer, 511, "<b>%s</b>\n\n%s", all_functions[real_index].prototype, _(all_functions[real_index].definition));
 	gtk_tooltip_set_markup(tooltip, buffer);
 
 	gtk_tree_view_set_tooltip_row(tree_view, tooltip, path);
 
 	gtk_tree_path_free(path);
+	free(all_functions);
 
 	return TRUE;
 }
@@ -1027,13 +1315,16 @@ gboolean query_tooltip_op_tree_view_cb(GtkWidget *widget, gint x, gint y,
 static void add_functions_to_list() {
 	GtkTreeIter iter;
 
+	_pm_op_func *all_functions = concat_functions();
+
 	init_widgets();
-	for (int i = 0; i < MAX_FUNCTIONS; i++) {
+	int i = 0;
+	/* fill functions */
+	for (i = 0; i < MAX_FUNCTIONS + MAX_IMAGE_FUNCTIONS; i++) {
 		gtk_list_store_append(pixel_math_list_store_functions, &iter);
-		gtk_list_store_set(pixel_math_list_store_functions, &iter,
-				COLUMN_NAME, functions[i].name, COLUMN_INDEX, i,
-				-1);
+		gtk_list_store_set(pixel_math_list_store_functions, &iter, COLUMN_NAME, all_functions[i].name, COLUMN_INDEX, i, -1);
 	}
+	free(all_functions);
 }
 
 static void add_operators_to_list() {
@@ -1042,9 +1333,19 @@ static void add_operators_to_list() {
 	init_widgets();
 	for (int i = 0; i < MAX_OPERATORS; i++) {
 		gtk_list_store_append(pixel_math_list_store_operators, &iter);
-		gtk_list_store_set(pixel_math_list_store_operators, &iter,
-				COLUMN_NAME, operators[i].name, COLUMN_INDEX, i,
-				-1);
+		gtk_list_store_set(pixel_math_list_store_operators, &iter, COLUMN_NAME, operators[i].name, COLUMN_INDEX, i, -1);
+	}
+}
+
+static void add_presets_to_list() {
+	GtkTreeIter iter;
+
+	init_widgets();
+
+	com.pref.gui.pm_presets = g_slist_reverse(com.pref.gui.pm_presets);
+	for (GSList *l = com.pref.gui.pm_presets; l; l = l->next) {
+		gtk_list_store_append(pixel_math_list_store_presets, &iter);
+		gtk_list_store_set(pixel_math_list_store_presets, &iter, 0, (gchar *)l->data, -1);
 	}
 }
 
@@ -1053,6 +1354,8 @@ void on_pixel_math_dialog_show(GtkWidget *w, gpointer user_data) {
 		add_functions_to_list();
 	if (!get_pixel_math_operators_number_of_rows())
 		add_operators_to_list();
+	if (!get_pixel_math_presets_number_of_rows())
+		add_presets_to_list();
 }
 
 void on_pixel_math_treeview_functions_row_activated(GtkTreeView *tree_view,
@@ -1149,3 +1452,125 @@ void on_pixel_math_selection_changed(GtkTreeSelection *selection, gpointer user_
 	output_status_bar2(width, height, channel);
 }
 
+
+static void add_expr_to_tree(const gchar *expression) {
+	GtkTreeIter iter;
+
+	gtk_list_store_append(pixel_math_list_store_presets, &iter);
+	gtk_list_store_set(pixel_math_list_store_presets, &iter, 0, expression, -1);
+}
+
+static gboolean foreach_func(GtkTreeModel *model, GtkTreePath *path,
+		GtkTreeIter *iter, gpointer user_data) {
+	gchar *expression;
+
+	gtk_tree_model_get(model, iter, 0, &expression, -1);
+	com.pref.gui.pm_presets = g_slist_prepend(com.pref.gui.pm_presets, expression);
+
+	return FALSE; /* do not stop walking the store, call us with next row */
+}
+
+static void save_presets_list() {
+	/* First we free the original list */
+	g_slist_free_full(com.pref.gui.pm_presets, g_free);
+	com.pref.gui.pm_presets = NULL;
+
+	gtk_tree_model_foreach(GTK_TREE_MODEL(pixel_math_list_store_presets), foreach_func, com.pref.gui.pm_presets);
+	com.pref.gui.pm_presets = g_slist_reverse(com.pref.gui.pm_presets);
+	writeinitfile();
+}
+
+void on_pm_expr1_bt_clicked(GtkButton *button, gpointer user_data) {
+	gchar *str = get_pixel_math_expression1();
+	if (str) {
+		g_strstrip(str);
+		if (str[0] != '\0') {
+			add_expr_to_tree(str);
+			save_presets_list();
+		}
+		g_free(str);
+	}
+}
+
+void on_pm_expr2_bt_clicked(GtkButton *button, gpointer user_data) {
+	gchar *str = get_pixel_math_expression2();
+	if (str) {
+		g_strstrip(str);
+		if (str[0] != '\0') {
+			add_expr_to_tree(str);
+			save_presets_list();
+		}
+		g_free(str);
+	}
+}
+
+void on_pm_expr3_bt_clicked(GtkButton *button, gpointer user_data) {
+	gchar *str = get_pixel_math_expression3();
+	if (str) {
+		g_strstrip(str);
+		if (str[0] != '\0') {
+			add_expr_to_tree(str);
+			save_presets_list();
+		}
+		g_free(str);
+	}
+}
+
+void on_pixel_math_treeview_presets_row_activated(GtkTreeView *tree_view,
+		GtkTreePath *path, GtkTreeViewColumn *column) {
+
+	init_widgets();
+
+	GtkEntry *entry = get_entry_with_focus();
+
+	GtkEntryBuffer *buffer = gtk_entry_get_buffer(entry);
+	gint *i = gtk_tree_path_get_indices(path);
+	const gchar *str = get_preset_expr(i[0]);
+
+	if (str) {
+		guint position = gtk_editable_get_position(GTK_EDITABLE(entry));
+		gtk_editable_delete_selection (GTK_EDITABLE(entry));
+
+		gtk_entry_buffer_insert_text(buffer, position, str, -1);
+		gtk_widget_grab_focus(GTK_WIDGET(entry));
+		gtk_editable_set_position(GTK_EDITABLE(entry), position + strlen(str));
+	}
+}
+
+static void remove_presets_from_list() {
+	GtkTreeSelection *selection;
+	GList *references, *list;
+
+	init_widgets();
+	selection = gtk_tree_view_get_selection(pixel_math_treeview_presets);
+	references = get_row_references_of_selected_rows(selection, pixel_math_tree_model_presets);
+	for (list = references; list; list = list->next) {
+		GtkTreeIter iter;
+		GtkTreePath *path = gtk_tree_row_reference_get_path((GtkTreeRowReference*)list->data);
+		if (path) {
+			if (gtk_tree_model_get_iter(pixel_math_tree_model_presets, &iter, path)) {
+				gtk_list_store_remove(pixel_math_list_store_presets, &iter);
+			}
+			gtk_tree_path_free(path);
+		}
+	}
+	g_list_free(references);
+	gtk_tree_selection_unselect_all(selection);
+	save_presets_list();
+}
+
+gboolean on_pixel_math_treeview_presets_key_release_event(GtkWidget *widget, GdkEventKey *event,
+		gpointer user_data) {
+	if (event->keyval == GDK_KEY_Delete || event->keyval == GDK_KEY_KP_Delete
+			|| event->keyval == GDK_KEY_BackSpace) {
+		remove_presets_from_list();
+
+		return TRUE;
+	}
+	return FALSE;
+}
+
+void on_rescale_pm_button_toggled(GtkToggleButton *button, gpointer user_data) {
+	gtk_widget_set_sensitive(lookup_widget("spin_pm_low"), gtk_toggle_button_get_active(button));
+	gtk_widget_set_sensitive(lookup_widget("spin_pm_high"), gtk_toggle_button_get_active(button));
+}
