@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -32,6 +32,8 @@ struct pixel_math_data {
 	gchar *expression2;
 	gchar *expression3;
 	gboolean single_rgb;
+	gboolean rescale;
+	float min, max;
 };
 
 int load_pm_var(const gchar *var, int index, int *w, int *h, int *c);
@@ -39,4 +41,3 @@ void free_pm_var(int nb);
 gpointer apply_pixel_math_operation(gpointer p);
 
 #endif /* SRC_PIXELMATH_PIXEL_MATH_RUNNER_H_ */
-
