@@ -3,6 +3,7 @@
 #include "core/siril.h"
 #include "core/proto.h"
 #include "core/OS_utils.h"
+#include "core/siril_log.h"
 #include "algos/statistics.h"
 #include "stacking.h"
 #include "io/image_format_fits.h"
@@ -17,6 +18,8 @@ static int compute_normalization(struct stacking_args *args);
 int do_normalization(struct stacking_args *args) {
 	if (args->normalize == NO_NORM) return ST_OK;
 
+	struct timeval t_start, t_end;
+	gettimeofday(&t_start, NULL);
 	int nb_frames = args->nb_images_to_stack;
 	int nb_layers = args->seq->nb_layers;
 
@@ -38,73 +41,88 @@ int do_normalization(struct stacking_args *args) {
 	if (args->seq->needs_saving)	// if we had to compute new stats
 		writeseqfile(args->seq);
 
+	gettimeofday(&t_end, NULL);
+	show_time_msg(t_start, t_end, _("Normalization computation time"));
 	return ST_OK;
 }
 
-/* scale0, mul0 and offset0 are output arguments when i = ref_image, input arguments otherwise */
-static int _compute_normalization_for_image(struct stacking_args *args, int i, int ref_image,
-		double **poffset, double **pmul, double **pscale, normalization mode, double *scale0,
-		double *mul0, double *offset0, gboolean multithread, int thread_id) {
-	gboolean fit_is_open = FALSE;
-	fits fit = { 0 };
+static int _compute_estimators_for_image(struct stacking_args *args, int i,
+		threading_type threading, int image_thread_id) {
+	imstats *stats[3] = { NULL };
+	int nb_layers = args->seq->nb_layers;
+	int retval = ST_OK;
+	g_assert(nb_layers <= 3);
+	g_assert(threading > 0);
+
+	siril_debug_print("computing stats for image %d, %d threads, lite: %d\n", i, threading, args->lite_norm);
+	retval = compute_all_channels_statistics_seqimage(args->seq, args->image_indices[i], NULL, (args->lite_norm) ? STATS_LITENORM : STATS_NORM, threading, image_thread_id, stats);
 
 	for (int layer = 0; layer < args->seq->nb_layers; ++layer) {
-		// try with no fit passed: fails if data is needed because data is not cached
-		imstats *stat;
-		if (!(stat = statistics(args->seq, args->image_indices[i], NULL, layer, NULL, STATS_NORM, multithread))) {
-			if (!(fit_is_open)) {
-				// read frames as float, it's faster to compute stats
-				if (seq_read_frame(args->seq, args->image_indices[i], &fit, TRUE, thread_id)) {
-					return ST_SEQUENCE_ERROR;
-				}
-				fit_is_open = TRUE; // to avoid opening fit more than once if RGB
-				/* check if rgb images are mixed with monochrome images */
-				if (args->seq->nb_layers != (int)fit.naxes[2]) {
-					siril_log_color_message(_("It looks like your sequence contains a mix of monochrome and RGB images.\n"), "red");
-					clearfits(&fit);
-					return ST_GENERIC_ERROR;
-				}
-			}
-			// retry with the fit to compute it
-			if (!(stat = statistics(args->seq, args->image_indices[i], &fit, layer, NULL, STATS_NORM, multithread)))
-				return ST_GENERIC_ERROR;
+		imstats *stat = stats[layer];
+		if (!stat) {
+			retval = ST_GENERIC_ERROR;
+			continue;
 		}
-
-		double sc = stat->scale;
-		double loc = stat->location;
-		free_stats(stat);
-
-		switch (mode) {
+		switch (args->normalize) {
 		default:
+		/* In order to compute quick loc and  scale estimates
+		 * if lite_norm is true
+		 * location is replaced by median
+		 * scale is replaced by 1.5*mad which is a good approximation of
+		 * sqrt(bwmv) for a gaussian distribution (tested of large gaussian samples of 10M values)
+		 * sqrt(bwmv) itself being a good enough fit for IKSS scale estimator
+		 */
 		case ADDITIVE_SCALING:
-			pscale[layer][i] = sc;
-			if (i == ref_image)
-				scale0[layer] = pscale[layer][i];
-			pscale[layer][i] = (pscale[layer][i] == 0) ? 1 : scale0[layer] / pscale[layer][i];
+			args->coeff.pscale[layer][i] = (args->lite_norm) ? 1.5 * stat->mad : stat->scale;
 			/* no break */
 		case ADDITIVE:
-			poffset[layer][i] = loc;
-			if (i == ref_image)
-				offset0[layer] = poffset[layer][i];
-			poffset[layer][i] = pscale[layer][i] * poffset[layer][i] - offset0[layer];
+			args->coeff.poffset[layer][i] = (args->lite_norm) ? stat->median : stat->location;
 			break;
 		case MULTIPLICATIVE_SCALING:
-			pscale[layer][i] = sc;
-			if (i == ref_image)
-				scale0[layer] = pscale[layer][i];
-			pscale[layer][i] = (pscale[layer][i] == 0) ? 1 : scale0[layer]  / pscale[layer][i];
+			args->coeff.pscale[layer][i] = (args->lite_norm) ? 1.5 * stat->mad : stat->scale;
 			/* no break */
 		case MULTIPLICATIVE:
-			pmul[layer][i] = loc;
-			if (i == ref_image)
-				mul0[layer] = pmul[layer][i];
-			pmul[layer][i] = (pmul[layer][i] == 0) ? 1 : mul0[layer] / pmul[layer][i];
+			args->coeff.pmul[layer][i] = (args->lite_norm) ? stat->median : stat->location;
 			break;
 		}
+		free_stats(stat);
+	}
+	return retval;
+}
+
+/* computes normalization factors based on the reference image from the
+ * estimators of each image and channel. Overwrites the estimators */
+static void compute_factors_from_estimators(struct stacking_args *args, int ref_index) {
+	double **poffset = args->coeff.poffset, **pmul = args->coeff.pmul, **pscale = args->coeff.pscale;
+	int nb_layers = args->seq->nb_layers;
+	double offset0[3], mul0[3], scale0[3];
+
+	for (int layer = 0; layer < nb_layers; ++layer) {
+		offset0[layer] = poffset[layer][ref_index];
+		mul0[layer] = pmul[layer][ref_index];
+		scale0[layer] = pscale[layer][ref_index];
+	}
+	int reflayer;
+	for (int layer = 0; layer < nb_layers; ++layer) {
+		reflayer = (args->equalizeRGB) ? args->reglayer : layer;
+		for (int i = 0; i < args->nb_images_to_stack; ++i) {
+			switch (args->normalize) {
+				default:
+				case ADDITIVE_SCALING:
+					pscale[layer][i] = (pscale[layer][i] == 0) ? 1 : scale0[reflayer] / pscale[layer][i];
+					/* no break */
+				case ADDITIVE:
+					poffset[layer][i] = pscale[layer][i] * poffset[layer][i] - offset0[reflayer];
+					break;
+				case MULTIPLICATIVE_SCALING:
+					pscale[layer][i] = (pscale[layer][i] == 0) ? 1 : scale0[reflayer]  / pscale[layer][i];
+					/* no break */
+				case MULTIPLICATIVE:
+					pmul[layer][i] = (pmul[layer][i] == 0) ? 1 : mul0[reflayer] / pmul[layer][i];
+					break;
+			}
+		}
 	}
-	if (fit_is_open && args->seq->type != SEQ_INTERNAL)
-		clearfits(&fit);
-	return ST_OK;
 }
 
 static int normalization_get_max_number_of_threads(sequence *seq) {
@@ -119,7 +137,7 @@ static int normalization_get_max_number_of_threads(sequence *seq) {
 	 * For DATA_FLOAT, we have: the image O(n), rewrite without zeros O(m),
 	 * used directly for IKSS and a copy for MAD O(m).
 	 */
-	guint64 memory_per_image = seq->rx * seq->ry;
+	guint64 memory_per_image = (guint64) seq->rx * seq->ry;
 	if (get_data_type(seq->bitpix) == DATA_FLOAT)
 		memory_per_image *= (seq->nb_layers + 2) * sizeof(float);
 	else memory_per_image *= (seq->nb_layers + 1) * sizeof(WORD) + 2 * sizeof(float);
@@ -142,9 +160,7 @@ static int normalization_get_max_number_of_threads(sequence *seq) {
 }
 
 static int compute_normalization(struct stacking_args *args) {
-	int i, ref_image_filtred_idx = -1, retval = 0, cur_nb = 1;
-	double *scale0, *mul0, *offset0;	// for reference frame
-	char *tmpmsg;
+	int ref_image_filtred_idx = -1, retval = 0, cur_nb = 0;
 	norm_coeff *coeff = &args->coeff;
 	int nb_layers = args->seq->nb_layers;
 	int nb_frames = args->nb_images_to_stack;
@@ -153,24 +169,22 @@ static int compute_normalization(struct stacking_args *args) {
 		coeff->poffset[layer] = coeff->offset + layer * nb_frames;
 		coeff->pmul[layer] = coeff->mul + layer * nb_frames;
 		coeff->pscale[layer] = coeff->scale + layer * nb_frames;
-	}
 
-	for (i = 0; i < args->nb_images_to_stack; ++i) {
-		for (int layer = 0; layer < nb_layers; ++layer) {
+		for (int i = 0; i < args->nb_images_to_stack; ++i) {
 			coeff->poffset[layer][i] = 0.0;
 			coeff->pmul[layer][i] = 1.0;
 			coeff->pscale[layer][i] = 1.0;
 		}
+
+		/* We empty the cache if needed (force to recompute) */
+		if (args->force_norm)
+			clear_stats(args->seq, layer);
 	}
 
 	if (args->normalize == NO_NORM)	// should never happen here
 		return 0;
 
-	scale0 = malloc(nb_layers * sizeof(double));
-	mul0 = malloc(nb_layers * sizeof(double));
-	offset0 = malloc(nb_layers * sizeof(double));
-
-	tmpmsg = siril_log_message(_("Computing normalization...\n"));
+	char *tmpmsg = siril_log_message(_("Computing normalization...\n"));
 	tmpmsg[strlen(tmpmsg) - 1] = '\0';
 	set_progress_bar_data(tmpmsg, PROGRESS_RESET);
 
@@ -180,63 +194,41 @@ static int compute_normalization(struct stacking_args *args) {
 	if (ref_image_filtred_idx == -1) {
 		siril_log_color_message(_("The reference image is not in the selected set of images. "
 				"Please choose another reference image.\n"), "red");
-		free(scale0);
-		free(mul0);
-		free(offset0);
 		return ST_GENERIC_ERROR;
 	}
 
+	const char *error_msg = (_("Normalization failed."));
+
 	// check memory first
-	const char *error_msg = (_("Normalization failed."));
 	int nb_threads = normalization_get_max_number_of_threads(args->seq);
-	if (nb_threads == 0) {
+	if (nb_threads <= 0) {
 		set_progress_bar_data(error_msg, PROGRESS_NONE);
-		free(scale0);
-		free(mul0);
-		free(offset0);
 		return ST_GENERIC_ERROR;
 	}
-
-	/* We empty the cache if needed (force to recompute) */
-	if (args->force_norm)
-		clear_stats(args->seq, args->reglayer);
-
-	// compute for the first image to have scale0 mul0 and offset0 for each layer
-
-	if (_compute_normalization_for_image(args, ref_image_filtred_idx,
-			ref_image_filtred_idx, coeff->poffset, coeff->pmul, coeff->pscale,
-			args->normalize, scale0, mul0, offset0, TRUE, -1)) {
-		siril_log_color_message(_("%s Check image %d first.\n"), "red", error_msg,
-				ref_image_filtred_idx + 1);
-		set_progress_bar_data(error_msg, PROGRESS_NONE);
-		free(scale0);
-		free(mul0);
-		free(offset0);
-		return ST_GENERIC_ERROR;
-	}
-
+	if (nb_threads > args->nb_images_to_stack)
+		nb_threads = args->nb_images_to_stack;
+	int *threads_per_thread = compute_thread_distribution(nb_threads, com.max_thread);
 
 	set_progress_bar_data(NULL, 1.0 / (double)args->nb_images_to_stack);
 
 #ifdef _OPENMP
-#pragma omp parallel for num_threads(nb_threads) private(i) schedule(guided) \
+#pragma omp parallel for num_threads(nb_threads) schedule(guided) \
 	if (args->seq->type == SEQ_SER || ((args->seq->type == SEQ_REGULAR || args->seq->type == SEQ_FITSEQ) && fits_is_reentrant()))
 #endif
 
-	for (i = 0; i < args->nb_images_to_stack; ++i) {
-		if (!retval && i != ref_image_filtred_idx) {
+	for (int i = 0; i < args->nb_images_to_stack; ++i) {
+		if (!retval) {
 			if (!get_thread_run()) {
 				retval = 1;
 				continue;
 			}
 			int thread_id = -1;
+			threading_type threads = SINGLE_THREADED;
 #ifdef _OPENMP
 			thread_id = omp_get_thread_num();
+			threads = threads_per_thread[thread_id];
 #endif
-			if (_compute_normalization_for_image(args, i, ref_image_filtred_idx,
-						coeff->poffset, coeff->pmul, coeff->pscale,
-						args->normalize, scale0, mul0, offset0,
-						FALSE, thread_id)) {
+			if (_compute_estimators_for_image(args, i, threads, thread_id)) {
 				siril_log_color_message(_("%s Check image %d first.\n"), "red",
 						error_msg, args->image_indices[i] + 1);
 				set_progress_bar_data(error_msg, PROGRESS_NONE);
@@ -247,15 +239,15 @@ static int compute_normalization(struct stacking_args *args) {
 #pragma omp atomic
 #endif
 			cur_nb++;	// only used for progress bar
-			set_progress_bar_data(NULL,
-					(double)cur_nb / (double)args->nb_images_to_stack );
+			set_progress_bar_data(NULL, cur_nb / (double)args->nb_images_to_stack);
 		}
 	}
 
+	free(threads_per_thread);
+	if (!retval)
+		compute_factors_from_estimators(args, ref_image_filtred_idx);
+
 	set_progress_bar_data(NULL, PROGRESS_DONE);
-	free(scale0);
-	free(mul0);
-	free(offset0);
 	return retval;
 }
 
