@@ -2,6 +2,7 @@
 #define STACKING_H_
 
 #include "core/processing.h"
+#include "core/sequence_filtering.h"
 
 //#define STACK_DEBUG
 
@@ -13,6 +14,7 @@ typedef struct normalization_coeff norm_coeff;
 
 enum {
 	ST_ALLOC_ERROR = -10,
+	ST_CANCEL = -9,
 	ST_SEQUENCE_ERROR = -2,
 	ST_GENERIC_ERROR = -1,
 	ST_OK = 0
@@ -26,27 +28,6 @@ typedef enum {
 	STACK_MIN,
 } stackMethod;
 
-/* TYPE OF SIGMA CLIPPING */
-typedef enum {
-	NO_REJEC,
-	PERCENTILE,
-	SIGMA,
-	MAD,
-	SIGMEDIAN,
-	WINSORIZED,
-	LINEARFIT,
-	GESDT
-} rejection;
-
-/* TYPE OF NORMALIZATION */
-typedef enum {
-	NO_NORM,
-	ADDITIVE,
-	MULTIPLICATIVE,
-	ADDITIVE_SCALING,
-	MULTIPLICATIVE_SCALING,
-} normalization;
-
 /* identical to the combo box items */
 typedef enum {
 	ALL_IMAGES,
@@ -54,9 +35,19 @@ typedef enum {
 	BEST_PSF_IMAGES,
 	BEST_WPSF_IMAGES,
 	BEST_ROUND_IMAGES,
+	BEST_BKG_IMAGES,
+	BEST_NBSTARS_IMAGES,
 	BEST_QUALITY_IMAGES
 } stackType;
 
+typedef enum {
+	NO_WEIGHT,
+	NBSTARS_WEIGHT,
+	WFWHM_WEIGHT,
+	NOISE_WEIGHT,
+	NBSTACK_WEIGHT
+} weighingType;
+
 struct normalization_coeff {
 	double *offset;
 	double *mul;
@@ -66,35 +57,48 @@ struct normalization_coeff {
 	double *pscale[3];
 };
 
-
+/* Warning: editing this will require changing the long initializer in stacking.c */
 struct stacking_args {
 	stack_method method;
 	sequence *seq;
-	int ref_image;	// takes precedences over seq->reference_image which may not be applicable
+	int ref_image; // takes precedence over seq->reference_image which may not be applicable
 	seq_image_filter filtering_criterion;
 	double filtering_parameter;
-	int nb_images_to_stack; // calculated from the above, for display purposes
-	int *image_indices;	// conversion between selected image indices and sequence image indices
-	char *description;	// description of the filtering
-	const char *output_filename;	// used in the idle function only
-	gboolean output_overwrite;	// used in the idle function only
-	struct timeval t_start;
-	int retval;
-	float sig[2];		/* low and high sigma rejection or GESTD parameters */
-	float *critical_value; /* index of critical_values for GESTD */
-	rejection type_of_rejection;	/* type of rejection */
+	int nb_images_to_stack; 	/* calculated from the above */
+	int *image_indices;		/* mapping between selected and sequence image indices */
+	char *description;		/* description of the filtering */
+
+	const char *output_filename;	/* used in the idle function only */
+	gchar *output_parsed_filename;	/* used in the idle function only */
+	gboolean output_overwrite;	/* used in the idle function only */
+
+	gboolean lite_norm;		/* enable lightweight (med,mad) normalization */
 	normalization normalize;	/* type of normalization */
 	norm_coeff coeff;		/* normalization data */
 	gboolean force_norm;		/* TRUE = force normalization */
 	gboolean output_norm;		/* normalize final image to the [0, 1] range */
 	gboolean use_32bit_output;	/* output to 32 bit float */
-	int reglayer;		/* layer used for registration data */
+	int reglayer;			/* layer used for registration data */
+	gboolean equalizeRGB;		/* enable RGB equalization through normalization */
 
-	gboolean apply_weight;			/* enable weights */
-	double *weights; 				/* computed weights for each (layer,image)*/
+	rejection type_of_rejection;	/* type of rejection */
+	float sig[2];			/* low and high sigma rejection or GESTD parameters */
+	float *critical_value;		/* index of critical_values for GESTD */
+	gboolean create_rejmaps;	/* main activation flag for the rejection maps creation */
+	gboolean merge_lowhigh_rejmaps;	/* create only one map */
+	fits *rejmap_low, *rejmap_high;	/* rejection maps */
+
+	gboolean apply_noise_weights;	/* enable weights */
+	gboolean apply_nbstack_weights;	/* enable weights */
+	gboolean apply_wfwhm_weights;	/* enable weights */
+	gboolean apply_nbstars_weights;	/* enable weights */
+	double *weights; 		/* computed weights for each (layer, image)*/
 
 	float (*sd_calculator)(const WORD *, const int); // internal, for ushort
-	float (*mad_calculator)(const WORD *, const size_t, const double, gboolean) ; // internal, for ushort
+	float (*mad_calculator)(const WORD *, const size_t, const double, threading_type) ; // internal, for ushort
+
+	struct timeval t_start;
+	int retval;
 	fits result;
 };
 
@@ -104,17 +108,28 @@ struct stacking_configuration {
 	gchar *seqfile;
 	gchar *result_file;
 	stack_method method;
-	rejection type_of_rejection;	/* type of rejection */
+	rejection type_of_rejection;
 	double sig[2];
+	gboolean create_rejmaps;
+	gboolean merge_lowhigh_rejmaps;
 	gboolean force_no_norm;
 	gboolean output_norm;
+	gboolean equalizeRGB;
+	gboolean lite_norm;
 	normalization norm;
 	int number_of_loaded_sequences;
-	float f_fwhm, f_fwhm_p, f_wfwhm, f_wfwhm_p, f_round, f_round_p, f_quality, f_quality_p; // on if >0
-	gboolean filter_included;
-	gboolean apply_weight;
+	struct seq_filter_config filters;
+	gboolean apply_noise_weights;
+	gboolean apply_nbstack_weights;
+	gboolean apply_wfwhm_weights;
+	gboolean apply_nbstars_weights;
 };
 
+typedef struct {
+	GDateTime *date_obs;
+	gdouble exposure;
+} DateEvent;
+
 
 /*
  * ESD test statistic data.
@@ -126,16 +141,15 @@ struct outliers {
 };
 
 
-void initialize_stacking_default();
 void initialize_stacking_methods();
-gboolean evaluate_stacking_should_output_32bits(stack_method method,
+gboolean evaluate_stacking_should_output_32bits(const stack_method method,
 		sequence *seq, int nb_img_to_stack, gchar **err);
 
 int stack_median(struct stacking_args *args);
 int stack_mean_with_rejection(struct stacking_args *args);
 int stack_addmax(struct stacking_args *args);
 int stack_addmin(struct stacking_args *args);
-
+gboolean stack_regdata_is_valid(struct stacking_args *args);
 void main_stack(struct stacking_args *args);
 void clean_end_stacking(struct stacking_args *args);
 
@@ -143,39 +157,39 @@ void get_sequence_filtering_from_gui(seq_image_filter *filtering_criterion,
 		double *filtering_parameter);
 void update_stack_interface(gboolean dont_change_stack_type);
 
+void describe_stack_for_history(struct stacking_args *args, GSList **hist, gboolean for_rejmap, gboolean low_rejmap);
 
 	/* normalization functions, normalize.c */
 
 int do_normalization(struct stacking_args *args);
+int *compute_thread_distribution(int nb_workers, int max);
 
 
 	/* median and mean functions */
 
 int check_G_values(float Gs, float Gc);
-void confirm_outliers(struct outliers *out, int N, double median, int *rejected, guint64 rej[2]);
+void confirm_outliers(struct outliers *out, int N, double median, int *rejected, int rej[2]);
 
 struct _image_block {
 	long channel, start_row, end_row, height; // long matches naxes type
 };
 
 int stack_compute_parallel_blocks(struct _image_block **blocksptr, long max_number_of_rows,
-		long naxes[3], int nb_threads, long *largest_block_height, int *nb_blocks);
+		const long naxes[3], int nb_threads, long *largest_block_height, int *nb_blocks);
 
 /* pool of memory blocks for parallel processing */
 struct _data_block {
+	void *tmp;	// the actual single buffer for all others below
 	void **pix;	// buffer for a block on all images
-	void *tmp;	// the actual single buffer for pix
 	void *stack;	// the reordered stack for one pixel in all images
-	int *rejected;  // 0 if pixel ok, 1 or -1 if rejected
+	int *rejected;	// 0 if pixel ok, 1 or -1 if rejected
+	void *o_stack;	// original unordered stack
 	void *w_stack;	// stack for the winsorized rejection
-	void *o_stack;  // original unordered stack
 	float *xf, *yf, m_x, m_dx2;// data for the linear fit rejection
 	int layer;	// to identify layer for normalization
 };
 
-int find_refimage_in_indices(int *indices, int nb, int ref);
-
-int check_fits_params(fitsfile *fptr, int *oldbitpix, int *oldnaxis, long *oldnaxes);
+int find_refimage_in_indices(const int *indices, int nb, int ref);
 
 	/* up-scaling functions */
 
@@ -185,6 +199,11 @@ void remove_tmp_drizzle_files(struct stacking_args *args);
 
 	/* rejection_float.c */
 
-int apply_rejection_float(struct _data_block *data, int nb_frames, struct stacking_args *args, guint64 crej[2]);
+int apply_rejection_float(struct _data_block *data, int nb_frames, struct stacking_args *args, int crej[2]);
+
+	/* keeping metadata */
+void free_list_date(gpointer data);
+DateEvent* new_date_item(GDateTime *dt, gdouble exposure);
+void compute_date_time_keywords(GList *list_date, fits *fit);
 
 #endif
