@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -24,15 +24,18 @@
 #include "core/siril.h"
 #include "core/proto.h"
 #include "core/processing.h"
+#include "core/siril_log.h"
 #include "io/sequence.h"
 #include "io/ser.h"
 #include "io/image_format_fits.h"
 #include "stacking.h"
 #include "gui/progress_and_log.h"
+#include "registration/registration.h"
 
 struct sum_stacking_data {
 	guint64 *sum[3];	// the new image's channels
 	double *fsum[3];	// the new image's channels, for float input image
+	GList *list_date; // list of dates of every FITS file
 	double livetime;	// sum of the exposures
 	int reglayer;		// layer used for registration data
 	int ref_image;		// reference image index in the stacked sequence
@@ -76,25 +79,32 @@ static int sum_stacking_prepare_hook(struct generic_seq_args *args) {
 	}
 
 	ssdata->livetime = 0.0;
+	ssdata->list_date = NULL;
 	return ST_OK;
 }
 
-static int sum_stacking_image_hook(struct generic_seq_args *args, int o, int i, fits *fit, rectangle *_) {
+static int sum_stacking_image_hook(struct generic_seq_args *args, int o, int i, fits *fit, rectangle *_, int threads) {
 	struct sum_stacking_data *ssdata = args->user;
-	int shiftx, shifty, nx, ny, x, y, layer;
+	int shiftx = 0, shifty = 0, nx, ny, x, y, layer;
 	size_t ii, pixel = 0;	// index in sum[0]
+	/* we get some metadata at the same time: date, exposure ... */
 
 #ifdef _OPENMP
 #pragma omp atomic
 #endif
 	ssdata->livetime += fit->exposure;
-	
+
+	if (fit->date_obs) {
+		GDateTime *date = g_date_time_ref(fit->date_obs);
+		ssdata->list_date = g_list_prepend(ssdata->list_date, new_date_item(date, fit->exposure));
+	}
+
 	if (ssdata->reglayer != -1 && args->seq->regparam[ssdata->reglayer]) {
-		shiftx = round_to_int(args->seq->regparam[ssdata->reglayer][i].shiftx * (float)args->seq->upscale_at_stacking);
-		shifty = round_to_int(args->seq->regparam[ssdata->reglayer][i].shifty * (float)args->seq->upscale_at_stacking);
-	} else {
-		shiftx = 0;
-		shifty = 0;
+		double scale = args->seq->upscale_at_stacking;
+		double dx, dy;
+		translation_from_H(args->seq->regparam[ssdata->reglayer][i].H, &dx, &dy);
+		shiftx = round_to_int(dx * scale);
+		shifty = round_to_int(dy * scale);
 	}
 
 	for (y = 0; y < fit->ry; ++y) {
@@ -137,7 +147,6 @@ static int sum_stacking_finalize_hook(struct generic_seq_args *args) {
 	if (args->retval) {
 		if (ssdata->sum[0]) free(ssdata->sum[0]);
 		if (ssdata->fsum[0]) free(ssdata->fsum[0]);
-		free(ssdata);
 		args->user = NULL;
 		return args->retval;
 	}
@@ -172,16 +181,20 @@ static int sum_stacking_finalize_hook(struct generic_seq_args *args) {
 			import_metadata_from_fitsfile(args->seq->fptr[ref], fit);
 			seq_close_image(args->seq, ref);
 		}
+		fit->livetime = ssdata->livetime;
 	} else if (args->seq->type == SEQ_FITSEQ) {
 		if (!fitseq_set_current_frame(args->seq->fitseq_file, ref))
 			import_metadata_from_fitsfile(args->seq->fitseq_file->fptr, fit);
+		fit->livetime = ssdata->livetime;
 	} else if (args->seq->type == SEQ_SER) {
 		import_metadata_from_serfile(args->seq->ser_file, fit);
+		fit->livetime = fit->exposure * args->nb_filtered_images; // ssdata->livetime is null for ser as fit has no exposure data
 	}
 
-	fit->livetime = ssdata->livetime;
 	fit->stackcnt = args->nb_filtered_images;
 	nbdata = args->seq->ry * args->seq->rx;
+	compute_date_time_keywords(ssdata->list_date, fit);
+	g_list_free_full(ssdata->list_date, (GDestroyNotify) free_list_date);
 
 	if (ssdata->output_32bits) {
 		if (ssdata->input_32bits) {
@@ -194,7 +207,7 @@ static int sum_stacking_finalize_hook(struct generic_seq_args *args) {
 				}
 			}
 		} else {
-			double ratio = 1.0 / (double)max;
+			double ratio = 1.0 / (max == 0 ? 1 : (double)max);
 			for (layer=0; layer<args->seq->nb_layers; ++layer){
 				guint64 *from = ssdata->sum[layer];
 				float *to = fit->fpdata[layer];
