@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -25,6 +25,7 @@
 #include "core/OS_utils.h"
 #include "core/processing.h"
 #include "core/sequence_filtering.h"
+#include "core/siril_log.h"
 #include "io/sequence.h"
 #include "io/ser.h"
 #include "io/image_format_fits.h"
@@ -55,7 +56,8 @@ void remove_tmp_drizzle_files(struct stacking_args *args) {
 	seqname = malloc(len);
 	g_snprintf(seqname, len, "%s.seq", basename);
 	siril_debug_print("Removing %s\n", seqname);
-	g_unlink(seqname); // removing the seqfile
+	if (g_unlink(seqname))
+		siril_debug_print("g_unlink() failed\n"); // removing the seqfile
 	free(seqname);
 	g_free(basename);
 
@@ -67,17 +69,20 @@ void remove_tmp_drizzle_files(struct stacking_args *args) {
 			// FIXME: no preallocation of file name
 			fit_sequence_get_image_filename(args->seq, args->image_indices[i], filename, TRUE);
 			siril_debug_print("Removing %s\n", filename);
-			g_unlink(filename);
+			if (g_unlink(filename))
+				siril_debug_print("g_unlink() failed\n");
 		}
 		break;
 	case SEQ_SER:
 		siril_debug_print("Removing %s\n", args->seq->ser_file->filename);
-		g_unlink(args->seq->ser_file->filename);
+		if (g_unlink(args->seq->ser_file->filename))
+			siril_debug_print("g_unlink() failed\n");
 		ser_close_file(args->seq->ser_file);
 		break;
 	case SEQ_FITSEQ:
 		siril_debug_print("Removing %s\n", args->seq->fitseq_file->filename);
-		g_unlink(args->seq->fitseq_file->filename);
+		if (g_unlink(args->seq->fitseq_file->filename))
+			siril_debug_print("g_unlink() failed\n");
 		fitseq_close_file(args->seq->fitseq_file);
 		break;
 	}
@@ -102,7 +107,7 @@ struct upscale_args {
 	double factor;
 };
 
-static int upscale_image_hook(struct generic_seq_args *args, int o, int i, fits *fit, rectangle *_) {
+static int upscale_image_hook(struct generic_seq_args *args, int o, int i, fits *fit, rectangle *_, int threads) {
 	double factor = ((struct upscale_args *)args->user)->factor;
 	/* updating pixel size if exist */
 	fit->pixel_size_x /= factor;
@@ -110,7 +115,7 @@ static int upscale_image_hook(struct generic_seq_args *args, int o, int i, fits
 
 	return cvResizeGaussian(fit,
 			round_to_int(fit->rx * factor),
-			round_to_int(fit->ry * factor), OPENCV_NEAREST);
+			round_to_int(fit->ry * factor), OPENCV_NEAREST, FALSE);
 }
 
 int upscale_sequence(struct stacking_args *stackargs) {
@@ -122,19 +127,9 @@ int upscale_sequence(struct stacking_args *stackargs) {
 
 	upargs->factor = stackargs->seq->upscale_at_stacking;
 
-	if (com.cache_upscaled) {
-		// This won't work if stackargs->filtering_criterion is already a multiple filter
-		args->filtering_criterion = create_multiple_filter(
-				stackargs->filtering_criterion, stackargs->filtering_parameter,
-				create_filter_prefixed_nonexisting_output(TMP_UPSCALED_PREFIX), 0.0,
-				NULL);
-		args->filtering_parameter = 0.0; // not used by multiple filtering
-		args->nb_filtered_images = -1;
-	} else {
-		args->filtering_criterion = stackargs->filtering_criterion;
-		args->filtering_parameter = stackargs->filtering_parameter;
-		args->nb_filtered_images = stackargs->nb_images_to_stack;
-	}
+	args->filtering_criterion = stackargs->filtering_criterion;
+	args->filtering_parameter = stackargs->filtering_parameter;
+	args->nb_filtered_images = stackargs->nb_images_to_stack;
 	args->prepare_hook = seq_prepare_hook;
 	args->finalize_hook = seq_finalize_hook;
 	args->image_hook = upscale_image_hook;
@@ -151,9 +146,10 @@ int upscale_sequence(struct stacking_args *stackargs) {
 	if (nb_threads == 0) {
 		siril_log_color_message(_("Stacking will be done without up-scaling (disabling 'drizzle')\n"), "red");
 		stackargs->seq->upscale_at_stacking = 1.0;
+		free(args);
 		return 0;
 	}
-	args->max_thread = nb_threads;
+	args->max_parallel_images = nb_threads;
 
 	remove_tmp_drizzle_files(stackargs);
 
@@ -167,11 +163,12 @@ int upscale_sequence(struct stacking_args *stackargs) {
 		gchar *basename = g_path_get_basename(stackargs->seq->seqname);
 		char *seqname = malloc(strlen(TMP_UPSCALED_PREFIX) + strlen(basename) + 5);
 		sprintf(seqname, "%s%s.seq", TMP_UPSCALED_PREFIX, basename);
-		g_unlink(seqname);
+		if (g_unlink(seqname))
+			siril_debug_print("g_unlink() failed\n");
 		g_free(basename);
 
 		// replace active sequence by upscaled
-		if (check_seq(0)) {	// builds the new .seq
+		if (check_seq()) {	// builds the new .seq
 			free(seqname);
 			return 1;
 		}
