@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -19,17 +19,17 @@
  */
 
 #include "../core/siril.h"
-
-#define USE_ALL_SORTING_ALGOS
 #include "../algos/sorting.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <time.h>
 
-#define USE_MULTITHREADING TRUE
+#define THREADING_TYPE MULTI_THREADED
 
-cominfo com;	// the main data struct
+cominfo com;	// the core data struct
+guiinfo gui;	// the gui data struct
+fits gfit;	// currently loaded image
 
 double median_from_sorted_array(WORD *arr, int size)
 {
@@ -47,7 +47,7 @@ double _siril_qsort(WORD *data, size_t datasize)
 
 double _histogram_sort(WORD *data, size_t datasize)
 {
-	return histogram_median(data, datasize, USE_MULTITHREADING);
+	return histogram_median(data, datasize, THREADING_TYPE);
 }
 
 clock_t perf_test(double (*function)(WORD *data, size_t datasize),
