@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -49,7 +49,7 @@ guiinfo gui;	// the gui data struct
 fits gfit;	// currently loaded image
 #endif
 
-int check_that_blocks_cover_the_image(long naxes[3], struct _image_block *blocks, int nb_blocks) {
+int check_that_blocks_cover_the_image(const long naxes[3], struct _image_block *blocks, int nb_blocks) {
 	// assuming that blocks are adjacent 
 	long ok_up_to_y = 0L, ok_up_to_z = 0L;
 	for (int i = 0; i < nb_blocks; i++) {
