@@ -213,7 +213,11 @@ template<class T> void gauss3x3div (T** RESTRICT src, T** RESTRICT dst, T** REST
 // use separated filter if the support window is small and src == dst
 template<class T> void gaussHorizontal3 (T** src, T** dst, int W, int H, const float c0, const float c1)
 {
+#ifdef _MSC_VER
+    T *temp = new T[W] ALIGNED16;
+#else
     T temp[W] ALIGNED16;
+#endif
 #ifdef _OPENMP
     #pragma omp for
 #endif
@@ -228,6 +232,9 @@ template<class T> void gaussHorizontal3 (T** src, T** dst, int W, int H, const f
 
         dst[i][W - 1] = src[i][W - 1];
     }
+#ifdef _MSC_VER
+    delete temp;
+#endif
 }
 
 #ifdef __SSE2__
@@ -293,7 +300,11 @@ template<class T> void gaussVertical3 (T** src, T** dst, int W, int H, const flo
 #else
 template<class T> void gaussVertical3 (T** src, T** dst, int W, int H, const float c0, const float c1)
 {
+#ifdef _MSC_VER
+    T *temp = new T[H] ALIGNED16;
+#else
     T temp[H] ALIGNED16;
+#endif
 #ifdef _OPENMP
     #pragma omp for
 #endif
@@ -311,6 +322,9 @@ template<class T> void gaussVertical3 (T** src, T** dst, int W, int H, const flo
 
         dst[H - 1][i] = src[H - 1][i];
     }
+#ifdef _MSC_VER
+    delete temp;
+#endif
 }
 #endif
 
@@ -440,8 +454,11 @@ template<class T> void gaussHorizontal (T** src, T** dst, const int W, const int
             M[i][j] /= (1.0 + b1 - b2 + b3) * (1.0 + b2 + (b1 - b3) * b3);
         }
 
+#ifdef _MSC_VER
+    double *temp2 = new double[W] ALIGNED16;
+#else
     double temp2[W] ALIGNED16;
-
+#endif
 #ifdef _OPENMP
     #pragma omp for
 #endif
@@ -473,6 +490,10 @@ template<class T> void gaussHorizontal (T** src, T** dst, const int W, const int
         }
 
     }
+
+#ifdef _MSC_VER
+    delete temp2;
+#endif
 }
 
 #ifdef __SSE2__
@@ -917,13 +938,20 @@ template<class T> void gaussVertical (T** src, T** dst, const int W, const int H
 
     // process 'numcols' columns for better usage of L1 cpu cache (especially faster for large values of H)
     static const int numcols = 8;
+#ifdef _MSC_VER
+    double** temp2 = new double* [numcols];
+    for (int i = 0; i < numcols; ++i) {
+        temp2[i] = new double[H];
+    }
+#else
     double temp2[H][numcols] ALIGNED16;
+#endif
     double temp2Hm1[numcols], temp2H[numcols], temp2Hp1[numcols];
 #ifdef _OPENMP
     #pragma omp for nowait
 #endif
 
-    for (unsigned int i = 0; i < static_cast<unsigned>(std::max(0, W - numcols + 1)); i += numcols) {
+    for (int i = 0; i < static_cast<int>(std::max(0, W - numcols + 1)); i += numcols) {
         for (int k = 0; k < numcols; k++) {
             temp2[0][k] = B * src[0][i + k] + b1 * src[0][i + k] + b2 * src[0][i + k] + b3 * src[0][i + k];
             temp2[1][k] = B * src[1][i + k] + b1 * temp2[0][k] + b2 * src[0][i + k] + b3 * src[0][i + k];
@@ -981,6 +1009,13 @@ template<class T> void gaussVertical (T** src, T** dst, const int W, const int H
             dst[j][i] = temp2[j][0] = B * temp2[j][0] + b1 * temp2[j + 1][0] + b2 * temp2[j + 2][0] + b3 * temp2[j + 3][0];
         }
     }
+
+#ifdef _MSC_VER
+    for (int i = 0; i < numcols; ++i) {
+        delete[] temp2[i];
+    }
+    delete[] temp2;
+#endif
 }
 
 #ifndef __SSE2__
@@ -996,7 +1031,14 @@ template<class T> void gaussVerticaldiv (T** src, T** dst, T** divBuffer, const
 
     // process 'numcols' columns for better usage of L1 cpu cache (especially faster for large values of H)
     static const int numcols = 8;
+#ifdef _MSC_VER
+    double** temp2 = new double* [numcols];
+    for (int i = 0; i < numcols; ++i) {
+        temp2[i] = new double[H];
+    }
+#else
     double temp2[H][numcols] ALIGNED16;
+#endif
     double temp2Hm1[numcols], temp2H[numcols], temp2Hp1[numcols];
 #ifdef _OPENMP
     #pragma omp for nowait
@@ -1060,6 +1102,13 @@ template<class T> void gaussVerticaldiv (T** src, T** dst, T** divBuffer, const
             dst[j][i] = librtprocess::max(divBuffer[j][i] / (temp2[j][0] = B * temp2[j][0] + b1 * temp2[j + 1][0] + b2 * temp2[j + 2][0] + b3 * temp2[j + 3][0]), 0.0);
         }
     }
+
+#ifdef _MSC_VER
+    for (int i = 0; i < numcols; ++i) {
+        delete[] temp2[i];
+    }
+    delete[] temp2;
+#endif
 }
 
 template<class T> void gaussVerticalmult (T** src, T** dst, const int W, const int H, const double sigma)
@@ -1074,7 +1123,14 @@ template<class T> void gaussVerticalmult (T** src, T** dst, const int W, const i
 
     // process 'numcols' columns for better usage of L1 cpu cache (especially faster for large values of H)
     static const int numcols = 8;
+#ifdef _MSC_VER
+    double** temp2 = new double* [numcols];
+    for (int i = 0; i < numcols; ++i) {
+        temp2[i] = new double[H];
+    }
+#else
     double temp2[H][numcols] ALIGNED16;
+#endif
     double temp2Hm1[numcols], temp2H[numcols], temp2Hp1[numcols];
 #ifdef _OPENMP
     #pragma omp for nowait
@@ -1138,6 +1194,13 @@ template<class T> void gaussVerticalmult (T** src, T** dst, const int W, const i
             dst[j][i] *= (temp2[j][0] = B * temp2[j][0] + b1 * temp2[j + 1][0] + b2 * temp2[j + 2][0] + b3 * temp2[j + 3][0]);
         }
     }
+
+#ifdef _MSC_VER
+    for (int i = 0; i < numcols; ++i) {
+        delete[] temp2[i];
+    }
+    delete[] temp2;
+#endif
 }
 #endif
 
@@ -1177,8 +1240,11 @@ template<class T> void gaussianBlurImpl(T** src, T** dst, const int W, const int
         double mIdeal = (12 * sigma * sigma - n * wl * wl - 4 * n * wl - 3 * n) / (-4 * wl - 4);
         int m = round(mIdeal);
 
+#ifdef _MSC_VER
+        int* sizes = new int[n];
+#else
         int sizes[n];
-
+#endif
         for(int i = 0; i < n; i++) {
             sizes[i] = ((i < m ? wl : wu) - 1) / 2;
         }
@@ -1188,6 +1254,10 @@ template<class T> void gaussianBlurImpl(T** src, T** dst, const int W, const int
         for(int i = 1; i < n; i++) {
             librtprocess::boxblur(dst, dst, buffer, sizes[i], sizes[i], W, H);
         }
+
+#ifdef _MSC_VER
+        delete sizes;
+#endif
     } else {
         if (sigma < GAUSS_SKIP) {
             // don't perform filtering
